function imgNoisy = addGaussian(img, guassianStd, maxPxlSensor)
%TODO: change inputs from guassianStd, maxPxlSensor to guassianStdSensor, scaler) where scaler defines current pixel /
%sensorPixel
%Add Gaussian Noise to a 2D/3D/nD image: imgNoisy = noise.addGaussian(img, guassianStd, maxPxlSensor)
% Input image must be uint8/uint16/single/double class
% Output image will be single/single/single/double class correspondingly (in future may change to keep uint8/uint16 only if neccessary)

% 
% Inputs:
%   img:        Input 2D/3D/nD image array
%   guassianStd: std of Gaussian noise to be added, in the sensor pixel units, NOT in the current pixel units
%               However, if the img is the direct read out from the sensor without any scaling, or it is the direct read out 
%               subtract a background, then the img is not scaled, we should havemaxPxlSensor == max(img(:)).
%               If it is very small (such as 0), then we just return img.
%   maxPxlSensor:   Opitional argument, the pixel counts for the brightest pixel in the sensor pixel units.
%               If it is not specified, we assume current pixel count is the actual pixel count
%               
% Outputs:
%   imgNoisy:   Input image with Poisson noise added.
%               It has the same dimension as img. 
%               It is the same class as img, except when img is uint8/uint16 but maxPxlSensor is not equal to max(img(:))
%               then it will be single float
% Usage:
%   imgNoisy = noise.addGaussian(img, 38); % 38 is typical MFM sensor std
%   imgNoisy = noise.addGaussian(img, 38, 60000); % 60000 is the brightest pixel in original sensor pixel unit
%
%   If we were given the pixelMean instead of maxPxlSensor, we should compute maxPxlSensor before call the function
%       maxPxlSensor = single(pixelMean) / mean(single(img(:))) * single(max(img(:));
%   Similarly if we were given the pixelSum,
%       maxPxlSensor = single(pixelSum) / sum(single(img(:))) * single(max(img(:));
%
%   
% 2017.03.27 created
% Author: Xiang Huang: xianghuang@gmail.com
%
% NOTEs:
%   1. If guassianStd=0 or maxPxlSensor=Inf, then there is no noise to add, and we simply return img
%   2. Do we threshold negative pixels to 0 in imgNoisy? Currently No.

if nargin == 0; testMe(); return; end % Unit Test

if isa(img, 'uint8') || isa(img, 'uint16') || isa(img, 'single')
    outClass = 'single';
elseif isa(img, 'double')
    outClass = 'double';
else    
    error('unsupported input data precision %s', class(img));
end

imgMax = cast(max(img(:)), outClass);
if ~exist('maxPxlSensor', 'var') || isempty(maxPxlSensor)
    maxPxlSensor = imgMax;
end

guassianStdCurPixelUnit = guassianStd * imgMax/maxPxlSensor;
if guassianStdCurPixelUnit < eps(outClass)
    imgNoisy = img;
else
    imgNoisy = cast(img, outClass) + cast(normrnd(0, guassianStdCurPixelUnit, size(img)), outClass);
end

end

%% Unit Test
function testMe()
%% Group #1
fprintf('-------------Test group #1: all have std 1 but in different classes.\n');
rng(0, 'twister'); I0 = noise.addGaussian(ones(100,100,4, 'uint8')*100,  1);  
rng(0, 'twister'); I1 = noise.addGaussian(ones(100,100,4, 'uint16')*100, 1); 
rng(0, 'twister'); I2 = noise.addGaussian(ones(100,100,4, 'single')*100, 1); 
rng(0, 'twister'); I3 = noise.addGaussian(ones(100,100,4, 'double')*100, 1); 
fprintf('Std should be exactly the same and around 1: results = %.4f, %.4f, %.4f, %.4f.\n',  std(double(I0(:))), std(double(I1(:))), std(double(I2(:))), std(double(I3(:))))
fprintf('Classes should be single, single, single, double: results = %s, %s, %s, %s.\n', class(I0), class(I1), class(I2), class(I3));
% Group #2
fprintf('-------------Test group #2: all have std 1 in sensor pixel units, but scaled by different amount to generate current pixel counts.\n');
rng(0, 'twister'); I0 = noise.addGaussian(ones(100,100,4, 'uint8')*100,  1, 10);  
rng(0, 'twister'); I1 = noise.addGaussian(ones(100,100,4, 'uint16')*100, 1, 100); 
rng(0, 'twister'); I2 = noise.addGaussian(ones(100,100,4, 'single')*100, 1, 1000); 
rng(0, 'twister'); I3 = noise.addGaussian(ones(100,100,4, 'double')*100, 1, 10000); 
fprintf('Std should be around 1e1, 1e0, 1e-1, 1e-2: results = %.4e, %.4e, %.4e, %.4e.\n',  std(double(I0(:))), std(double(I1(:))), std(double(I2(:))), std(double(I3(:))))
fprintf('Classes should be single, single, single, double: results = %s, %s, %s, %s.\n', class(I0), class(I1), class(I2), class(I3));

end