function imgNoisy = addPoisson(img, photonMax)
%Add Poisson Noise to a 2D/3D/nD image: imgNoisy = noise.addPoisson(img, photonMax)
% Input image must be uint8/uint16/single/double class
% 
% Inputs:
%   img:        Input 2D/3D/nD image array
%   photoMax:   Opitional argument of the photon received at the brightes pixel
%               If it is not specified, we assume it is the same as the pixel counts, i.e., pixel counts = photon counts             
%               If it is inf, we just return img
% Outputs:
%   imgNoisy:   Input image with Poisson noise added.
%               It has the same dimension as img. 
%               It is the same class as img, except when img is uint8/uint16 but pixel counts is not equal to photon counts,
%               then it will be single float
% Usage:
%   imgNoisy = noise.addPoisson(img)
%   imgNoisy = noise.addPoisson(img, 100)
%
%   If we were given the photonMean instead of photonMax, we should compute photonMax before call the function
%   photonMax = single(photonMean) / mean(single(img(:))) * single(max(img(:));
%   Similarly if we were given the photonSum,
%   photonMax = single(photonSum) / sum(single(img(:))) * single(max(img(:));

%   
% 2017.03.27 created
% Author: Xiang Huang: xianghuang@gmail.com
%
% NOTEs:
%   1. If photonMax is Inf, then there is no noise to add, and we simply return img
%   2. the scaled version only have integer number of photons, with mean = input photon,  though input could have non-interger photons after scale
%   3. If use Gaussian noise with the variance adapted to the pixel values, then it's very close except for low pixel value regions When you apply Poisson noise,
%   4. Explain of imnoise function called
%   J = imnoise(I,'poisson') generates Poisson noise from the data instead of adding artificial noise to the data.  
%   If I is double precision, then input pixel values are interpreted as means of Poisson distributions scaled up by 1e12.  
%   For example, if an input pixel has the value 5.5e-12, then the corresponding output pixel will be generated from a Poisson 
%   distribution with mean of 5.5 and then scaled  back down by 1e12.  
%   If I is single precision, the scale factor used is 1e6.  
%   If I is uint8 or uint16, then input pixel values are used directly without scaling.  For example, if a pixel in a uint8 input
%   has the value 10, then the corresponding output pixel will be generated from a Poisson distribution with mean 10.
if nargin == 0; testMe(); return; end % Unit Test
    
if ~(isa(img, 'uint8') || isa(img, 'uint16') || isa(img, 'single') || isa(img, 'double'))
    error('unsupported input data precision %s', class(img));
end

imgMax = max(img(:));
if ~exist('photonMax', 'var') || isempty(photonMax)
    photonMax = imgMax;
elseif isinf(photonMax)
    imgNoisy = img;
    return
end

if (isa(img, 'uint8') || isa(img, 'uint16')) && (imgMax == photonMax)
    % integer and pixel count = photon count, imgNoisy will be same uint8/uint16 class as img
    imgNoisy = imnoise(img, 'poisson');
else
    % single or double or uint8/uint16 converted to single, all need to be scaled
    % 1. uint8/uint16 convert to single
    if isa(img, 'uint8') || isa(img, 'uint16')
        img = single(img);
    end
    % 2. scale to img*scale when call imnoise, so that (img*scale)*imnoiseScale  = photon    
    imgClass = class(img);
    imnoiseScaleMap = containers.Map({'single', 'double'}, {single(1e6), double(1e12)});
    imnoiseScale = imnoiseScaleMap(imgClass);
    scale = cast(photonMax, imgClass) / cast(imgMax, imgClass) / imnoiseScale; % NOTE: scale = 1/gain * 1/imnoiseScale, where gain = imgMax / photonMax;
    imgNoisy = 1/scale * imnoise(img*scale, 'poisson');
end
end

%% Unit Test
function testMe()
%% Group #1
fprintf('-------------Test group #1: all have 1 photon but in different classes.\n');
rng(0, 'twister'); I0 = imnoise(ones(100,100,4, 'uint8'), 'poisson');  
rng(0, 'twister'); I1 = noise.addPoisson(ones(100,100,4, 'uint8')); 
rng(0, 'twister'); I2 = noise.addPoisson(ones(100,100,4, 'single')); 
rng(0, 'twister'); I3 = noise.addPoisson(ones(100,100,4, 'double')); 
fprintf('Variances should be exactly the same and around 1: results = %.4f, %.4f, %.4f, %.4f.\n',  var(double(I0(:))), var(double(I1(:))), var(double(I2(:))), var(double(I3(:))))
fprintf('Classes should be uint8, uint8, single, double: results = %s, %s, %s, %s.\n', class(I0), class(I1), class(I2), class(I3));
% Group #2
fprintf('-------------Test group #2: all have 10 photons, but scaled by different gain to generate pixel counts.\n');
rng(0, 'twister'); I0 = imnoise(ones(100,100,4, 'double')*10*1e-12, 'poisson');  % img: 10 Photon scaled by 1e-12
rng(0, 'twister'); I1 = noise.addPoisson(ones(100,100,4, 'uint8')*10);  %  img: 10 Photon
rng(0, 'twister'); I2 = noise.addPoisson(ones(100,100,4, 'uint8')*100, 10); %  img: 10 Photon scaled/gain by 10
rng(0, 'twister'); I3 = noise.addPoisson(ones(100,100,4, 'single'), 10); %  img: 10 Photon scaled/gain by 0.1
fprintf('Variances should be around 1e-23, 1e1, 1e3, 1e-1: results = %.4e, %.4e, %.4e, %.4e.\n',  var(double(I0(:))), var(double(I1(:))), var(double(I2(:))), var(double(I3(:))))
fprintf('Classes should be double, uint8, single, single: results = %s, %s, %s, %s.\n', class(I0), class(I1), class(I2), class(I3));

end