function tiles = getMaskOfFieldStop(floodedImage, isShow)
%preprocess.getMaskFieldStop compute the mask for each tile caused by field stop, becasue there is gutter among filed stops
% Input: floodedImage is a .tif file of flooded image 
%%
if ~exist('floodedImage', 'var') || isempty(floodedImage)
    % floodedImage = uigetfile();
    %floodedImage = '/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/flooded_iris_100slope.tif';
    floodedImage = '/sandbox/smallWorlds/20171204_Insulin_PSF_200nm/floodImage.tif';
    %floodedImage = 'C:\Dropbox\smallWorlds\20171204_Insulin_PSF_200nm\floodImage.tif';
end
if ~exist('isShow', 'var') || isempty(isShow)
    isShow = true;
end

timeTagAll = {'20160809', '20171204', '20180516'};
for n = 1:numel(timeTagAll)
    if strfind(floodedImage, timeTagAll{n})
        timeTag = timeTagAll{n};
        break;
    end
end

%% read floodedImage  it is a file
[folder, floodedImageName, ~] = fileparts(floodedImage);
I = tif.read(floodedImage);
I = double(I(:,:,end));


%% 1. Compute the tile's flooded mask by first find the edges of the floodedImage (rectangles), then fill in each rectangle
%use the last page (20160809 dataset last page has the longest exposure) to find the mask
if strcmp(timeTag, '20171204')
   tiles.maskEdgeFlooded = imread(fullfile(folder, [floodedImageName, '_maskFieldStopEdge.bmp']));
else
   tiles.maskEdgeFlooded = edge(I, 'canny', [], 6);
end
tiles.maskFlooded  = imfill(tiles.maskEdgeFlooded, 'holes');

%%
imwrite(tiles.maskFlooded, fullfile(folder, [floodedImageName, '_maskFieldStop.bmp']));
imwrite(tiles.maskEdgeFlooded, fullfile(folder, [floodedImageName, '_maskFieldStopEdge.bmp']));

%%
if isShow
    figure(91), subplot(121); imshow(tiles.maskFlooded);  title('mask of field of view');
    subplot(122); imshow(tiles.maskEdgeFlooded); title('edge of mask');
end

%% 2. Split the whole mask into 25 masks, one mask for each tile, use connected component analysis
[Ny, Nx] = size(tiles.maskFlooded);
CC = bwconncomp(tiles.maskFlooded);
NTile = CC.NumObjects; Nr = sqrt(NTile); 
fprintf('The number of connected components in mask is assumed to be the number of tiles %d (which should be 25 or 9)!\n', CC.NumObjects);

%% compute centroids of each tile
tiles.centroidXYs = zeros(2, NTile); % x, y
for t = 1:NTile    
    [y, x] = ind2sub([Ny, Nx], CC.PixelIdxList{t});
    tiles.centroidXYs(:,t) = [mean(x); mean(y)];    
end   
%% '20160809':  sort 5x5 centroids as
% 5 10 15 20 25
% 4  9 14 19 24
% 3  8 13 18 23
% 2  7 12 17 22
% 1  6 11 16 21
%% '20171204': sort 3x3 centroids as
% 1  4  7
% 2  5  8
% 3  6  9
[~, ix] = sort(tiles.centroidXYs(1,:), 'ascend');
idxTiles = zeros(1, NTile);
for n = 1:sqrt(NTile)
    % when n=1, among the first five tiles of ix, sort descend by y
    idx = (1:Nr) + (n-1)*sqrt(NTile);
    tileNos = ix(idx); 
    switch timeTag
        case '20160809'    
            [~, iy] = sort(tiles.centroidXYs(2,tileNos), 'descend');
        case '20171204'
            [~, iy] = sort(tiles.centroidXYs(2,tileNos), 'ascend');
        otherwise
            error('unknown timeTag');            
    end
    idxTiles(idx) = tileNos(iy);
end

% correct the orders 
CC.PixelIdxList = CC.PixelIdxList(idxTiles);
tiles.centroidXYs = [tiles.centroidXYs(1, idxTiles); tiles.centroidXYs(2, idxTiles)];

if isShow
    figure(92), clf; imshow(tiles.maskEdgeFlooded); hold on; plot(tiles.centroidXYs(1,:), tiles.centroidXYs(2,:), 'rx-'); title('tileMask with centriods: in order');
end

%% Compute a outer rectangle, which is the rectangular convex hull that covers mask region for each tile
%and a minimum rectangle includes the mask
tiles.outerRectPosXYWHs = zeros(4, NTile); % [xMin, yMin, width, height]
for t = 1:NTile    
    [y, x] = ind2sub([Ny, Nx], CC.PixelIdxList{t});
    tiles.outerRectPosXYWHs(:, t) = [min(x); min(y); max(x)-min(x); max(y)-min(y)];    
end

%% Compute a inner rectangle, which is the largest rectangle that inside mask region for each tile
tiles.innerRectPosXYWHs = zeros(4, NTile); % [xMin, yMin, width, height]
for t = 1:NTile    
    pos = tiles.outerRectPosXYWHs(:,t);
    rectPos = preprocess.calcLargestInscribedImage(tiles.maskFlooded(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3)))';
    tiles.innerRectPosXYWHs(:,t) = [rectPos(1:2) + pos(1:2)-1; rectPos(3:4)];    
end

%% Generate mask for outer & inner rectangle region & edge
rectPosAll = {tiles.outerRectPosXYWHs, tiles.innerRectPosXYWHs};
maskAll = {false(Ny, Nx), false(Ny, Nx)};
maskEdgeAll = {false(Ny, Nx), false(Ny, Nx)};
for n=1:numel(maskAll)    
    for t = 1:NTile
        pos = rectPosAll{n}(:,t);
        maskAll{n}(pos(2):pos(2) + pos(4), pos(1):pos(1) + pos(3)) = true;
        maskEdgeAll{n}(pos(2),                 pos(1):pos(1) + pos(3)) = true;
        maskEdgeAll{n}(pos(2) + pos(4),        pos(1):pos(1) + pos(3)) = true;
        maskEdgeAll{n}(pos(2):pos(2) + pos(4), pos(1))                 = true;
        maskEdgeAll{n}(pos(2):pos(2) + pos(4), pos(1) + pos(3))        = true;
    end
end
tiles.maskOuterRect = maskAll{1};
tiles.maskInnerRect = maskAll{2};
tiles.maskEdgeOuterRect = maskEdgeAll{1};
tiles.maskEdgeInnerRect = maskEdgeAll{2};
if isShow
    figure(93); clf; 
    subplot(121); imshow(maskAll{1} & ~maskAll{2}); title('OuterRect - InnerRect');
    subplot(122); imshow(maskEdgeAll{1} | maskEdgeAll{2}); title('Inner Rect Edge + Outer Rect Edge');
end

save(fullfile(folder, 'tiles.mat'), 'tiles');

%% Draw both 
if isShow
    figure(94), clf; 
    subplot(121); imshow(tiles.maskEdgeFlooded); hold on; title('tileMaskEdge: White, outerRectangle: red, innerRectangle: blue');
    for t = 1:NTile    
        %imrect(gca, tiles.outerRectPosXYWHs(:,t)', 'edgecolor', 'r'); 
        rectangle('Position', tiles.outerRectPosXYWHs(:,t)', 'edgecolor', 'r'); 
        rectangle('Position', tiles.innerRectPosXYWHs(:,t)', 'edgecolor', 'b'); 
    end   
    subplot(122); plot([tiles.outerRectPosXYWHs; tiles.innerRectPosXYWHs]', '-x'); 
    legend('outer: xMin', 'outer: yMin', 'outer: width', 'outer: height', 'inner: xMin', 'inner: yMin', 'inner: width', 'inner: height');
    grid minor;
end
%% add multiple maskEdge to floodedImage and IAll
switch timeTag
    case '20160809'
        maskCircle3D = tif.read('/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/PSF_Ave_maskBg3D_151to251useful.tif');
        maskEdgeCircle3D = false(size(maskCircle3D));
        for n = 1:size(maskCircle3D,3)
            maskEdgeCircle3D(:,:,n) = edge(maskCircle3D(:,:,n));
        end
        sourceFiles = {'/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/flooded_iris_100slope.tif';
                       '/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/PSF_Ave.tif';
                       '/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/Bacteria1_DoubleLobes_MFM_10Measures.tif';                   
                       '/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/Bacteria2_LongZ_MFM_10Measures.tif'};
    case '20171204'
        sourceFiles = {'/sandbox/smallWorlds/20171204_Insulin_PSF_200nm/floodImage.tif';
                       '/sandbox/smallWorlds/20171204_Insulin_PSF_200nm/PSF_200nm_ex900em30_f241.tif';
                       '/sandbox/smallWorlds/20171204_Insulin_PSF_200nm/insulin_ex98em750_f1000.tif';};        
    otherwise
        error('unknown timeTag');
end

%%
for nFile = 1:numel(sourceFiles)
    IAll = tif.read(sourceFiles{nFile});
    for n = 1:size(IAll, 3)
        I = IAll(:,:,n);
        maxI = max(I(:));
        I(tiles.maskEdgeOuterRect) = cast(round(maxI*0.6), class(I));
        I(tiles.maskEdgeFlooded) = cast(round(maxI*0.8), class(I));               
        I(tiles.maskEdgeInnerRect) = maxI;
        if strcmp(timeTag, '20160809')
            % currently, only add circle for 20160809 as only it has 3D circle mask of psf
            if nFile == 2
                I(maskEdgeCircle3D(:,:,n)) = maxI;
            elseif nFile == 1
                I(maskEdgeCircle3D(:,:,round(size(maskEdgeCircle3D,3)/2))) = maxI;
            end
        end
        IAll(:,:,n) = I;
    end
    retFile = [sourceFiles{nFile}(1:end-4) '_withMask.tif'];
    tif.write(IAll, retFile);
end


