function f = generateCylinder(radius, len, varargin)
%generate a cylinder shapped 3D volumetric image with given radius and length
%function f = generateCylinder(radius, len, fSz=MinimumVolumeContainsCylinder, 'intensity'='random', 'direction'='z')
% Input:    
%   radius:  radius of the cylinder's cycle
%   len:    length/height of the cylinder
%   fSz (opitiona):     size of the volumetric image (typically row vector), if not specified, we use the most compact size computed from radius/len
%   'intensity' (optional): the intensity distribution inside the cylinder, any of {'uniform' (default), 'random', 'highcenter', 'lowcenter'}
%   'direction' (optional): the cylinder direction, any of {'y', 'x', 'z'(default)}
% Output:
%   f:  3D volumetric image contains the cylinder with size fSz.
% 2017.02.24:  Xiang Huang xianghuang@gmail.com

if nargin == 0; testMe(); return; end % Unit Test

%% Parse Inputs & Validation
p = inputParser;  % CaseSensitive=0, PartialMatching=1
p.addRequired('radius',                 @(x) validateattributes(x, {'numeric'}, {'scalar', 'positive'}));
p.addRequired('len',                    @(x) isnumeric(x) && isscalar(x) && (x>0));
p.addOptional('fSz',        [1,1,1],    @(x) isnumeric(x) && (numel(x)==3) && (all(x>0))); % dummy default fSz, will be updated later
p.addParameter('direction', 'z',        @(x) any(validatestring(x, {'y', 'x', 'z'}))); 
p.addParameter('intensity', 'highcenter',	@(x) any(validatestring(x,  {'uniform', 'random', 'highcenter', 'lowcenter'})));
p.parse(radius, len, varargin{:});
cellfun(@(x) evalin('caller', sprintf('%s=p.Results.%s;', x, x)), fields(p.Results)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;

%% Default cylinder is along 'z', handle input direction of {'y', 'x', 'z'} by shiftdim
% In the beginning, shift input volume size fSz so the designed axis is in the last dim. Use circshift(fSz, nShift) which shift from left to right:  yxz -> zyx -> xzy.
% In the end, shift the generated volume f back to yxz.  Use shiftdim(f, nShift) which shift from right to left:  xzy -> zyx -> yxz
nShiftMap = containers.Map({'y', 'x', 'z'}, {2, 1, 0});  
nShift = nShiftMap(direction);


%% Advancec Parse Input & Validation that depends on the input values, and shift the fSz
% If no fSz specified,  computer the exact fSz as compact volume includes the cylinder
%(S1+1)/2-1 >= radius, (S3+1)/2-1 > = len/2
fSzCompact = [[1, 1]*ceil(radius*2+1), ceil(len+1)];
if any(strcmp('fSz', p.UsingDefaults))
    fSz = fSzCompact;
else
    fSz = circshift(fSz(:)', nShift, 2); % circshift with 2nd dimenion of row vector,  convert to column vector so that code is compatiable with Matlab2015b and earlier
    assert(all(fSz >= fSzCompact), sprintf('User specified output volume size: [%d, %d, %d] is smaller the minimu size to include the object [%d, %d, %d].', fSz, fSzCompact));    
end

%% genertate all 0 volume and its coordinates
f = zeros(fSz);
[y, x, z] = ndgrid(1:fSz(1), 1:fSz(2), 1:fSz(3));

% mask within the cylinder
fCenter = (fSz+1)/2;
dist2MedialAxis = sqrt((y-fCenter(1)).^2 + (x-fCenter(2)).^2); 
mask =  (abs(z-fCenter(3)) <= len/2)  &  (dist2MedialAxis <= radius);
switch lower(intensity)
    case 'uniform'
        f(mask) = 1;
    case 'random'
        rng(0, 'twister');
        f(mask) = rand(nnz(mask), 1);
    case 'highcenter'
        f(mask) = 1- dist2MedialAxis(mask) / radius;
    case 'lowcenter'
        f(mask) = dist2MedialAxis(mask) / radius;
end

%% shift volume back so that yxz are the default dim 1/2/3
f = shiftdim(f, nShift);
end




%% Unit Test
function testMe()
%%
fSz =  [101   151   201];
radius = 8;
len = 60;
directionAll = {'y', 'x', 'z'}; 
intensityALL =  {'uniform', 'random', 'highcenter', 'lowcenter'};
%%
direction = 'z';
fAll = cell(1);
for n = 1:numel(intensityALL)
    fAll{n} = generateCylinder(radius, len, fSz, 'direction', direction, 'intensity', intensityALL{n});
end
figure; show3dPointsWithProjections(fAll); multAxes(@title, intensityALL);
%%
intensity = 'random';
fAll = cell(1);
for n = 1:numel(directionAll)
    fAll{n} = generateCylinder(radius, len, fSz, 'direction', directionAll{n}, 'intensity', intensity);
end
figure; show3dPointsWithProjections(fAll); multAxes(@title, directionAll);

end

%%
function trashBox()
varargin = {'intensity', 'centerHigh', 'direction', 'z', 'fSz', [0,0,0]}; %uniform, random/rand, highCenter, lowCenter 
opt = struct('intensity', 'uniform', 'direction', 'z', 'fSz', [0,0,0]);
optNames = fieldnames(opt); %cell
for pair = reshape(varargin, 2, []) % pair is {propName;propValue}
    inName = pair{1};
    if any(strcmpi(inName, optNames))
        opt.(inName) = pair{2};
    else
        error('%s is not a recognized parameter name', inName);
    end
end

if strcmpi(direction, 'z')
    [xx, yy] = meshgrid(1:fSz(2), 1:fSz(1)); %[xx, yy] = meshgrid(1:xmax, 1:ymax);
    for z = max(1, fCenter(3)-halfLen):min(fSz(3), fCenter(3)+halfLen)
        mask = ((xx-fCenter(2)).^2 + (yy-fCenter(1)).^2) <= radius.^2;                
        f(:,:,z) = mask;
    end
elseif strcmpi(direction, 'y')
    [xx, zz] = meshgrid(1:fSz(2), 1:fSz(3));
    for y = max(1, fCenter(1)-halfLen):min(fSz(1), fCenter(1)+halfLen)
        mask = ((xx-fCenter(2)).^2 + (zz-fCenter(3)).^2) <= radius.^2;
        f(y,:,:) = mask';
    end    
else
    error('no such direction');
end
end