function IMeanAll = projXYZ(tiffFile, pagesORxyz)
%IMeanAll = tif.projXYZ(tiffFile, pagesORxyz) saves the 2D projXYZ image of 3D multi-page tiff file along page(default, aka z)/x/y directions.
% Usage:
%   projXYZ();  % User interactively select a tiff file, average all pages, save as tiffFile+suffix, where suffix is "_Mean"
%   projXYZ(tiffFile);  % User provided a tiff file, average all pages
%   projXYZ([], 1:10);  % User interactively select a tiff file, average only page 1 to 10, typically pages need to be a:b,
%   whenever we specify pages, we assume xyz = 'z'
%   projXYZ(tiffFile, 'xyz'); % separately save 'x' & 'y' & 'z' projections
% Inputs:
%   pagesORxyz:  % edither the pages of average or the xyz direction(s) to average, such as 1:10, 'x', 'y', 'z', 'xy', 'xyz'...

if ~exist('tiffFile', 'var') || isempty(tiffFile)
    % user interactively specify tiffFile
    [filename, pathStr] = uigetfile('*.tif','Select tiff image stack file');
    fprintf('Read file: %s\n', fullfile(pathStr, filename));
    tiffFile = [pathStr,filename];
end

% parse pages & direction
pages = [];
xyz = 'z';
if ~exist('pagesORxyz', 'var') || isempty(pagesORxyz)
    % Both pages & xyz not specified, both use default, do nothing
elseif isnumeric(pagesORxyz) && isvector(pagesORxyz) && all(pagesORxyz>0) && all(pagesORxyz==round(x));
    % pagesORxyz specified pages when it is numberic positive integer vector, xyz is default
    pages = pagesORxyz;
elseif ischar(pagesORxyz) && isvector(pagesORxyz) && all(ismember(lower(pagesORxyz), 'xyz'))
    % pagesORxyz specified xyz when it is 1D char array whose elements are member of 'xyz', pages are default
    xyz = lower(pagesORxyz);
else
    error('pagesORxyz format error, need to either specify pages or xyz');
end

IAll = tif.read(tiffFile, pages);
if ndims(IAll) == 2
    % Do nothing if the tif file is a single frame tif
    IMeanAll = IAll;
    return
end

xyz2dimMap = containers.Map({'x', 'y', 'z'}, {2, 1, 3});
if isempty(pages)
    pageStr = '';
else
    pageStr = sprintf('_Page%d2%d', min(pages), max(pages));
end
xyz2suffixMap = containers.Map({'x', 'y', 'z'}, {'_MeanX',  '_MeanY', ['_MeanZ', pageStr]});
[pathStr, fName, fExt] = fileparts(tiffFile);
IMeanAll = cell(1,numel(xyz));
for n = 1:numel(xyz)
   IMeanAll{n} = cast(round(squeeze(mean(double(IAll), xyz2dimMap(xyz(n))))), 'like', IAll);
   imwrite(IMeanAll{n}, fullfile(pathStr, [fName, xyz2suffixMap(xyz(n)), '.tif']));
end

% if only compute the average along one direction, no need to use cell array, extract content and set IMeanAll as 2D array
if numel(xyz) == 1
    IMeanAll = IMeanAll{1};
end

end
