function imgAllPages = read(tiffFile, pages, verbose)
%imgAllPages = tif.read(tiffFile, pages)
%read all or selected pages from a .tiff file, the .tif file could be multi
%
% Usage:
%       imgAllPages = tif.read(tiffFile); %  Just use this one if you don't want to dig into the details.
%       imgAllPages = tif.read(tiffFile, pages)
%
% Input:
% tiffFile: location of multipage tiff image file, including the path. e.g. 'C:\Dropbox\smallworlds\data\MFM_PSF.tif'
% pages:   opitional parameter, user specified a subset of pages to read, 1d array from small to large, e.g. pages = 120:140. ,
%           If not specified, default is read all pages
%
% Output:
%       imgAllPages: selected image pages of tiffFile, could be binary/uint8/uint16/unint32/unit64 for 1/8/16/32/64 bit tiffFile
% 2016.02.02:   Created by Xiang Huang. xianghuang@gmail.com
% 2016.03.10:   added Gamma encoding opition
% 2016.03.23:   fixed pagesShow for a subset of pages
% 2016.08.18:   Rename tiffReadShowWrite --> read
%               change output imgAllPages from double 
%               removed the remove hotnoise opition.
%               save .jpg files has maximum intensity normalized to 255 for each individual image, rather than by the maximum of
%               all the images. 
% 2016.09.28    MH changed isGammaEncoding and isJPGsGammaEncode to isGammaEncodeJPGs
% 2016.09.29    XH changed the name of isGammaEncodeJPGs to isGammaEncodedShow
% 2017.03.15    XH moved to the subfolder ../+tif for better orginazation/namespacing along with other tiff related functions
% 2018.02.10    XH changed to read 32-bit float tiff, simplified code,  and moved the old bad complicated code to trashBox()
% 2018.05.09    XH changed, expelicitly specifies 'info' in imread(...,  ..., 'info', fileInfo), which runs about 30% faster
% refer to matlab's imread() and Tiff() function, the latter uses more advanced libtiff library functionality

if ~exist('tiffFile', 'var') || isempty(tiffFile)
    [filename, pathStr] = uigetfile('*.tif',' select tiff image stack file');
    tiffFile = fullfile(pathStr, filename);        
end

if ~exist('verbose', 'var') || isempty(verbose)
    verbose = true;
end

fileInfo = imfinfo(tiffFile);   % return tiff structure, one element per page
if ~exist('pages', 'var') || isempty(pages)
    % Default, read all pages.
    pages = 1:numel(fileInfo);
else
    % User specified pages, verify if they are the subset of 1:numel(tiffFileInfo)
    assert(all(ismember(pages, 1:numel(fileInfo))), sprintf('User specified pages are not subset of the valid page from 1 to %d', numel(fileInfo)));
end

if verbose
    fprintf('Read tiff file: %s\n', tiffFile);
end

Npage = numel(pages);


I = imread(tiffFile, pages(1), 'info', fileInfo); % I = imread(tiffFile, pages(1)); % use ('info', fileInfo) requires less time

if Npage > 1
    imgAllPages = zeros([size(I), Npage], 'like', I);
    imgAllPages(:,:,1) = I;
    for n = 2:Npage
       imgAllPages(:,:,n) = imread(tiffFile, pages(n), 'info', fileInfo); % imgAllPages(:,:,n) = imread(tiffFile, pages(n),);
    end
else
    imgAllPages = I;
end

end


function trashBox()
% Old Bad complicated code to initialize imgAllPages, unnessary as it won't handle floating point format etc. which is bad
imageSize = [fileInfo(1).Height, fileInfo(1).Width, Npage];
bitsPerSample = fileInfo(1).BitsPerSample; % strange sometimes, the MFM 16bit sensed image is written as 32bit images
bits2Class = containers.Map({1, 8, 16, 32, 64}, {'logical', 'uint8', 'uint16', 'uint32', 'uint64'});
if bitsPerSample ~= 1    
    imgAllPages = zeros(imageSize, bits2Class(bitsPerSample)); % imgAllPages = zeros(imageSize); 
else
    imgAllPages = false(imageSize); % imgAllPages = zeros(imageSize); 
end
%imgAllPages = constructBlankOutputImage(imageSize, bitsPerSample); % imgAllPages = zeros(imageSize);  % to remove, old code

end


