function imgAllPages = readShowWrite2JPGs(tiffFile, pages, isShowAndSaveJPGs, isGammaEncodedShow)
%imgAllPages = tif.readShowWrite2JPGs(tiffFile, pages, isShowAndSaveJPGs, isGammaEncodedShow)
% read all or selected pages from a .tiff file
% (opitionally) show a selected sample pages, and (opitionally) save all the pages to separate.jpg files with
% optional Gamma encoding to be viewed easier future.
%
% Usage:
%       imgAllPages = tif.readShowWrite2JPGs(tiffFile); %  Just use this one if you don't want to dig into the details.
%       imgAllPages = tif.readShowWrite2JPGs(tiffFile, pages)
%       imgAllPages = tif.readShowWrite2JPGs(tiffFile, pages, isShowAndSaveJPGs, isGammaEncodeJPGs)
%
% Input:
% tiffFile: location of multipage tiff image file, including the path. e.g. 'C:\Dropbox\smallworlds\data\MFM_PSF.tif'
% pages:   opitional parameter, user specified a subset of pages to read, 1d array from small to large, e.g. pages = 120:140. ,
%           If not specified, default is read all pages
% isShowAndSaveJPGs: opitional parameter, true/fasle with default as false (prefer to use imageJ to view all pages)
%                We will show 12 pages, and save all pages as .jpg files unless they already exist
% isGammaEncodeJPGs: optitional parameter, true/false with default as true
%                The data from tiffFile is typically linear, we generally need to apply gamma encoding before show/save each
%                pages as an image, because image display software will typcially apply gamma decoding to the values, i.e.,
%                Gamma encoding used by default to store the pixel's intensity values in a given image file; values are stored represent the light intensity via gamma-compressed values instead of a linear encoding.
%
% Output:
%       imgAllPages: selected image pages of tiffFile, could be binary/uint8/uint16/unint32/unit64 for 1/8/16/32/64 bit tiffFile
% 2016.02.02:   Created by Xiang Huang. xianghuang@gmail.com
% 2016.03.10:   added Gamma encoding opition
% 2016.03.23:   fixed pagesShow for a subset of pages
% 2016.08.18:   Rename tiffReadShowWrite --> readShowWrite2JPGs
%               change output imgAllPages from double 
%               removed the remove hotnoise opition.
%               save .jpg files has maximum intensity normalized to 255 for each individual image, rather than by the maximum of
%               all the images. 
% 2016.09.28    Mark Hereld     changed isGammaEncoding and isJPGsGammaEncode to isGammaEncodeJPGs
% 2016.09.29    Xiang Huang     changed the name of isGammaEncodeJPGs to isGammaEncodedShow
% 2017.03.15    Xiang moved to the subfolder ../+tif for better orginazation/namespacing along with other tiff related functions
% refer to matlab's imread() and Tiff() function, the latter uses more advanced libtiff library functionality

if ~exist('tiffFile', 'var') || isempty(tiffFile)
    [filename, pathStr] = uigetfile('*.tif',' select tiff image stack file');
    fprintf('Read file: %s\n', fullfile(pathStr, filename));
end


fileInfo = imfinfo(tiffFile);   % return tiff structure, one element per page
if ~exist('pages', 'var') || isempty(pages)
    % Default, read all pages.
    pages = 1:numel(fileInfo);
else
    % User specified pages, verify if they are the subset of 1:numel(tiffFileInfo)
    assert(all(ismember(pages, 1:numel(fileInfo))), sprintf('User specified pages are not subset of the valid page from 1 to %d', numel(fileInfo)));
end


if ~exist('isShowAndSaveJPGs', 'var') || isempty(isShowAndSaveJPGs)
    isShowAndSaveJPGs = false;
end

if ~exist('isGammaEncodedShow', 'var') || isempty(isGammaEncodedShow)
    isGammaEncodedShow = true;
    gamma = 2.2;
else
    gamma = 1.0;
end

imgAllPages = tif.read(tiffFile, pages);

%{
%the old psfs have hot noises for very few pixels, but new psf doesn't have the hot noise any more, so remove this step
% pxlValThreshold = max(tiffImg(:))? No. it is not robust to noise, very few meaningless pixels much brighter (10x) than others
% Robust to noise: set the top 0.1% brightest pixels of useful portion(only 1% of whole region) images (to filter out the bright the noise) as same intensity 1
% This is only required for 16bit images (but also works for 8bit as pxlValThreshold will be 255), for 8bit, simply divide by 255 or max(tiffImg(:))
if isRemoveHotNoise
    pxlValThreshold = prctile(imgAllPages(:), 99.999); %99.9-wrong-not consideringt the real image is small portion, 99
    pxlValMax = max(imgAllPages(:)); % compute the pxlValMax just for comparison to pxlValThreshold
    imgAllPages = imgAllPages/pxlValThreshold;
end
%}


if isShowAndSaveJPGs
    % pxlValMax = double(max(imgAllPages(:))); % compute the pxlValMax just for comparison to pxlValThreshold
    
    %% Show a maximum of 12 pages with page number linear spaced from the readed pages
    idxShow = round(linspace(1, Npage, min(Npage, 12)));
    if isGammaEncodedShow
        figname = sprintf('Show %d out of %d pages in the tiff file with gamma encoding', min(Npage, 12), Npage);
    else
        figname = sprintf('Show %d out of %d pages in the tiff file without gamma encoding', min(Npage, 12), Npage);
    end
    figure('name',figname);
    for i = 1:numel(idxShow)
        subplot(3, 4, i);
        img = double(imgAllPages(:, :, idxShow(i)));
        % Always save the normalzied pages, as saved images is only for visualzation not for computation anyway
        img = (img/max(img(:))).^(1/gamma); %img = (img/pxlValMax).^(1/gamma); % gamma correction since original intensity was linear.        
        img = uint8(round(255*img));
        imshow(img); title(sprintf('page %d', pages(idxShow(i))));
    end    
    
    %% Write all pages as .jpg files in a subfolder of the tiff file
    [tiffFolder, tiffImgName, ~] = fileparts(tiffFile);
    jpgFolder = fullfile(tiffFolder, [tiffImgName '_jpgs']);
    if ~exist(jpgFolder, 'dir')
        mkdir(jpgFolder);
    end
    for i = 1:Npage
        jpgFile = fullfile(jpgFolder, sprintf('%s_%04d.jpg', tiffImgName, pages(i)));
        if ~exist(jpgFile, 'file')
            % If .jpg file for current page already exist, don't overwrite, ignore and check next page
            img = double(imgAllPages(:, :, i));
            img = (img/max(img(:))).^(1/gamma); %img = (img/pxlValMax).^(1/gamma); 
            img = uint8(round(255*img));
            imwrite(img, jpgFile);
        end
    end
end
end

