function write(imgAllPages, tiffFile, pages)
%tif.write(imgAllPages, tiffFile, pages)
% write all or selected pages to a .tiff file from an array imgAllPages
%
% Usage:
%       tif.write(imgAllPages, tiffFile)
%       tif.write(imgAllPages, tiffFile, pages)
%
% Input:
%   imgAllPages:  image to be written to disk.
%                 2D/3D/4D matrix. Gray scale image with one (2D) or multiple pages (3D), or rgb image with multiple pages   (4D)
%                 binary/uint8/uint16/single-float/double-float format
%   tiffFile: location of multipage tiff image file, including the path. e.g. 'C:\Dropbox\smallworlds\data\MFM_PSF.tif'
%   pages:   opitional parameter, user specified a subset of pages to write, 1d array from small to large, e.g. pages = 120:140. ,
%           If not specified, default is to write all pages
%
% Output:
%       None
% 2016.12.15_clean up
% 2018.01.22 fixed a bug when imgAllPages is 2D, and pages is not specified.
% 2018.05.09 added support for write float images, by using the complicated Tiff class from libtiff. 
%            fixed a bug to write specific pages
% refer to matlab's imwrite() and Tiff.write() function, later used more advanced libtiff library

pageDim = ndims(imgAllPages);
if ~exist('pages', 'var') || isempty(pages)
    % Default, write all pages.
    if pageDim < 3 
        pages = 1;  % special case of 2D images
    else
        pages = 1 : size(imgAllPages, pageDim); % NOTE: pages = 1 for single page tiff write, as size(rand(5,5),3) = 1.
    end
else
    % User specified pages, verify if they are the subset of 1:size(imgAllPages,3)
    assert(all(ismember(pages, 1:size(imgAllPages,3))), sprintf('User specified pages are not subset of the valid page from 1 to %d', size(imgAllPages,3)));
end

if exist(tiffFile, 'file')
    b = questdlg(sprintf('Overwrite Existing File: %s?', tiffFile), 'Warning!', 'Yes', 'No', 'No');
    if strcmp(b, 'No') || isempty(b); return; end
    delete(tiffFile); % delete file first, otherwise, imwrite start with append mode will append to the old file.
end

if isfloat(imgAllPages)
    % single/double float precision
    tiffwriteFloat(imgAllPages, tiffFile, pages);
else
    % uint8/uint16 precision
    % We may handle data the same way as tiffwriteFloat using Tiff.write instead of imwrite, but no change now as current code works. 
    for n = 1:numel(pages)
        if pageDim <= 3
            % 2D or 3D
            imwrite(imgAllPages(:,:,pages(n)), tiffFile, 'writemode', 'append');
        elseif pageDim == 4
            % 4D = multiple page colorred image
            imwrite(imgAllPages(:,:,:,pages(n)), tiffFile, 'writemode', 'append');
        end
    end
end

end


function tiffwriteFloat(imgAllPages, tiffFile, pages)
%write to single float point multiple page tiff file
%The matlab imwrite() function does NOT support float tiff, so we need to use 
pageDim = ndims(imgAllPages);

if ~isa(imgAllPages, 'single')
    warning(sprintf("Original data is in double float format, but we write to tif file in single float precision.\nBecause single float has 24 bit precision which is more than practical sensor precision, we use it to save disk space without loss information.\nAlso because we use ImageJ which do not suppor double float."));
    imgAllPages = single(imgAllPages);
end

t = Tiff(tiffFile, 'w');
tagstruct.ImageLength = size(imgAllPages, 1);
tagstruct.ImageWidth = size(imgAllPages, 2);
tagstruct.Compression = Tiff.Compression.LZW; % None or LZW is suitable for float image. LZW may achieve 50% lossese compressiion rate for PSF images.
tagstruct.SampleFormat = Tiff.SampleFormat.IEEEFP; % IEEE floating point data
tagstruct.Photometric = Tiff.Photometric.MinIsBlack; % This is the default, so may no need to set it.
tagstruct.PlanarConfiguration = Tiff.PlanarConfiguration.Chunky;
%tagstruct.SubFileType = Tiff.SubFileType.Page; % No need
tagstruct.BitsPerSample = 32; % 32bit single float
if pageDim == 4 
    % colored when imgAllPages is a 4D matrix. Multiple page colorred image, each page could be colorred image (rgb or rgba)
    tagstruct.SamplesPerPixel = size(imgAllPages, 3);  % SamplesPerPixel = 3 for rgb, or 4 for rgba
else
    % grayscale
    tagstruct.SamplesPerPixel = 1; 
end

for n = 1:numel(pages)
    if n ~= 1
        t.writeDirectory(); % similar to create a new page, which is required in create a multiple-page tiff
    end
    t.setTag(tagstruct);
    if pageDim <= 3
        % 2D or 3D
        t.write(imgAllPages(:,:,pages(n)));
    elseif pageDim == 4
        % 4D = multiple page colorred image
        t.write(imgAllPages(:,:,:,pages(n)));
    end
end
t.close();


end