classdef CAMERA % use value class instead of handle class as it doesn't use large memory
    %CAMERA Specification of the MFM andor iXon3 EMCCD - 888 High Sensitivity Camera
    %   2017.03.15_Xiang Creadted
    %   2017.03.16_Xiang Updated
    %Usage:
    %   camPsf = CAMERA('EMGain', 750);
    properties (Constant)
        EMAmp = 1/5.28; % digital counts / gained electrons,    Andor888_9177_MFM_2017.01.30_from MattDaddyMan.pdf setting of 30MHz
        QE = 0.98;    % quantum effiecieny = electrons /photon, constant for a given wavelenth (520nm etc)
    end
    properties
        % User inputed properties        
        EMGain;   %  e.g. 1000, gained electrons / original electrons
        exposure; % ms
        camOffsetFile; % offset image file, e.g., C:\Dropbox\smallWorlds\20170314_Camera\camOffset.tif has mean 491.5, std 5.7, min=467, max=514 in DU
        % Computed properties
        camOffSet;   % camera's offset image need to be subtracted
        camOffSetConstant = 491.5;
        gaussianStdDU;    % the gaussian readout noise in DU, only depend on gain (may also depend on temporature etc, but not considered yet), depends on EMGain calibrated, depends on EMAmp from the manual
        gaussianStdPhoton;    % the gaussian std in photon unit will be gaussianSTD/gain
        gain;     % gain = QE * EMGain * EMAmp, pixel counts/ photon =  electrons /photon * gained electrons / original electrons * counts / gained electrons        
    end    
    methods
        function self =  CAMERA(varargin)
            %Usage: cam = CAMERA();
            %       cam = CAMERA('EMGain', 1000, 'exposure', 100, 'camOffsetFile', 'camOffSet.tif');
           %% Parse inputs to set: EMGain/exposure/camOffsetFile
            p = inputParser;  % CaseSensitive=0, PartialMatching=1
            camOffsetFile0 = fullfile('..', '..', '..',  'smallWorlds', '20170314_Camera', 'camOffset.tif');
            isPositiveFloatOrEmpty     = @(x) isempty(x) || isnumeric(x) && isscalar(x) && (x>0);
            p.addParameter('EMGain',    1000,   isPositiveFloatOrEmpty);
            p.addParameter('exposure',  400,    isPositiveFloatOrEmpty);
            p.addParameter('camOffsetFile',  camOffsetFile0); %existing valid file; or don't check.    @(x) isempty(x) || exist(x, 'file'
            p.parse(varargin{:});
            cellfun(@(x) evalin('caller', sprintf('self.%s=p.Results.%s;', x, x)), fields(p.Results)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;            
            
            %% compute to set: camOffSet
            if exist(self.camOffsetFile, 'file')
                self.camOffSet = tif.read(self.camOffsetFile);
            else
                warning('Camera offset file %s does not exist. We assume the offset is not space varying, but constant %.2f for all the pixels!',self.camOffsetFile,  self.camOffSetConstant); 
                self.camOffSet = self.camOffSetConstant;
            end
            self.gaussianStdDU = self.getReadOutNoise(self.EMGain);
            self.gain = self.getGain(self.EMGain);
            self.gaussianStdPhoton = self.gaussianStdDU / self.gain;            
        end
        
        function setParasFromName(self, fileName)
            %Set Gain Exp Std From FileName
            % Usage:
            %    setParasFromName('bg_I10ex400em1000_shutterOn.tif')
            fileInfo = parseName(fileName); 
            if ~isempty(fileInfo.EMGain)
                self.EMGain = fileInfo.EMGain;
                self.gaussianStdDU = self.getReadOutNoise(self.EMGain);
            end
            if ~isempty(fileInfo.exposure)
                self.exposure = fileInfo.exposure;
            end                             
        end
        
    end

    methods(Static)
        function gaussianSTD = getReadOutNoise(EMGain, model)
            % calibrated 2017.03.16
            if ~exist('EMGain', 'var') || isempty('EMGain')
                warning('Noise depends on EMGain which is not specified. Use default EMGain=1000.');
                EMGain = 1000;                
            end
            if ~exist('model', 'var') || isempty('model')
                model = 'linear';
            end            
            if strcmpi(model, 'linear')
                gaussianSTD =[33.6358, 0.0045]*[1, EMGain]'; 
            elseif strcmpi(model, 'quadratic') || strcmpi(model, 'quad')
                gaussianSTD = [34.2183, 0.0017, 2.4412e-06]*[1, EMGain, EMGain^2];
            end
         
        end
        
        function gain = getGain(EMGain)
            gain = CAMERA.QE * EMGain * CAMERA.EMAmp;
        end
        
        function calibrate()            
            %% conclusion@2017.03.16: the average pixel value is non-uniform, but independent of exposure time and gain
            % the noise of each pixel is independent of exposure time, but increase with EM-Gain, 
            %   linear approximation: approximately std = [33.6358, 0.0045]*[1, EMGain]';
            %   quaratic approximation: better: std = [34.2183, 0.0017, 2.4412e-06]*[1, EMGain, EMGain^2]';
            %% read all calibration files
            precision = 'double'; % 'single' or 'double' or 'uint16';
            camFolder = '/sandbox/smallWorlds/20170314_Camera/';
            %tiffFilesStruct = dir(fullfile(camFolder, '*_camOffset_*f100.tif'));
            tiffFilesStruct = [dir(fullfile(camFolder, '20170216_camOffset_*f100.tif')); dir(fullfile(camFolder, '20170314_camOffset_*f100.tif'))]; % 1+17 files
            tiffFileNames = {tiffFilesStruct.name};                        
            Nfile = numel(tiffFileNames);            
            [Nr, Nc, Npage] = size(tif.read(fullfile(camFolder, tiffFileNames{1})));
            IAll = zeros(Nr, Nc, Npage, Nfile, precision); %Total memory: 6.8GB / 13.6GB in single /double = (1024*1024*4/8)*100*17 for 17 files each has 100 pages            
            EMGainAll = zeros(Nfile, 1);
            exposureAll = zeros(Nfile, 1);
            for n = 1:Nfile                
                tiffFileName = tiffFileNames{n};            
                fprintf('read %d of %d file: %s.\n', n, Nfile, tiffFileName);
                tiffInfo = tif.parseName(tiffFileName);
                EMGainAll(n) = tiffInfo.EMGain;
                exposureAll(n) = tiffInfo.exposure;
                IAll(:,:,:,n) = cast(tif.read(fullfile(camFolder, tiffFileName)), precision); 
            end
            %% compute the mean and std, among all pages, for each pixel for each file
            ImeanAll = squeeze(mean(IAll, 3)); % zeros(Nr, Nc, Nfile);
            IstdAll = squeeze(std(IAll, 0, 3)); % 0 means default normalization by N-1, or 1 to use N            
            %% compute the mean of mean and std, among all pixels (acutally mean of std is sqrt of mean of variance)  for each file                       
            ImeanFiles = squeeze(mean(mean(ImeanAll,1), 2));
            IstdFiles = squeeze(sqrt(mean(mean(IstdAll.^2,1), 2)));
            %% show results
            em = 1000; idxSameEm = find(EMGainAll == em); [~, idxSort] = sort(exposureAll(idxSameEm)); idxSameEm = idxSameEm(idxSort);
            ex = 400; idxSameEx = find(exposureAll == ex); [~, idxSort] = sort(EMGainAll(idxSameEx)); idxSameEx = idxSameEx(idxSort);            
            figure(1); multAxes(@plot, EMGainAll(idxSameEx), {ImeanFiles(idxSameEx), IstdFiles(idxSameEx)}, '-x', 'MarkerEdgeColor', 'r'); multAxes(@xlabel, 'EM-Gain');  multAxes(@ylabel, {'Mean', 'Std'}); multAxes(@title, sprintf('fixed exposure time =%d', ex));
            figure(2); multAxes(@plot, exposureAll(idxSameEm), {ImeanFiles(idxSameEm), IstdFiles(idxSameEm)}, '-x','MarkerEdgeColor', 'r');  multAxes(@xlabel, 'ExposureTime');  multAxes(@ylabel, {'Mean', 'Std'}); multAxes(@title, sprintf('fixed EM-Gain =%d', em));
            tilefigs
            %% compute offSetImage as the mean among all pages and all files, as the it doesn't depends on exposure/EMgain settings
            camOffSetOrig = mean(ImeanAll,3);
            tif.write(cast(round(camOffSetOrig), 'uint16'), fullfile(camFolder, 'camOffset_OriginalHotNoise.tif'));
            % found 4 hot pixels, remove them with median of 8 neigbors. NOTE: the position of hot pixels is almost random (may at same location in one day, but appear at other locations in another day)            
            camOffSet = camOffSetOrig;
            th = 515;
            idxHot = find(camOffSetOrig>th);
            [row, col] = ind2sub(size(camOffSetOrig), idxHot);
            for i = 1:numel(idxHot)
                Icp = camOffSetOrig(row(i)+(-1:1), col(i)+(-1:1));
                camOffSet(idxHot(i)) = median(Icp(:));                
            end
            tif.write(cast(round(camOffSet), 'uint16'), fullfile(camFolder, 'camOffset.tif'));
            %% Not doing for the whole image, but for a corpped region of image
%             w = 10;
%             Icp = reshape(Iall(1:w, 1:w,:), 1, []);
%             Imean = mean(Icp);
%             Istd = std(Icp);            
%             figure; hist(Icp, 100); title(sprintf('mean=%.2f, std =%.2f', Imean, Istd));
            
        end
    end

end

