classdef MFM < handle
    %MFM: typically with instance named as mfm contains the following parameters
    % We use handle class instead of value class, so to avoid copy of large class variables and set propertiy will be simpler
    % and don't require return a copy of the object. The handle class is similar to reference in C++.
    %
    % 2017.01.06: (1) Avoid duplicates (2) seperate meta data (3) minimize user input,
    % pixel coorniates
    % ----------> x
    % |
    % |
    % v y
    properties (Constant)
        %
    end
    properties
        % (1) User Input Parameters
        % (1.1) User Input Opitional Positional Parameters
        % NOTE: psfFile and gFile do NOT need to be in the same folder. One psfFile could corresponding to many gFile(s)
        psfFile;    % Specifies 3D psf image for a given MFM setup (a stack of 2D mfm images). Tif/mat file (with full path) or pre-computed psf array
        gFile;      % Specifies 2D MFM (g) image for a object (e.g., bacteria) captured with the same psf. Tif/mat file (with full path) or pre-computed g array
        % (1.2) User Input Opitional Name-Value Pair Parameters
        psfBgFile;  % Specifies background to be subtracted from psf, which takes following values
                    %  0 (default, recommended) or all 0 array of same size as psf  if we don't want to subtract any background
                    % .tif/.mat file or pre-computed psfBg array. e.g.: 'C:\Dropbox\smallWorlds\20161221_Cytoplasm_100mw\psf1_mfm_100nm_ex500em1000_f254_backgroudFromTop20Bottom20Frames.png';
                    %  [] if background is estimated via a function such as preprocess.calcBackgroundFrom4Corners(), depreciated after 2018.01.22
        gBgFile;    % Specifies background to be subtracted from g, takes similar format (above) as psfBgFile
        gFrameNo;   % If gFile has multiple frames, only compute one selected frames
        fSz;        % 3D object image size, 1x3 vector [fy, fx, fz] such as [150, 150, 101].
        voxSz;      % Voxel size (nm) in 3D recovered image, 1x3 array such as [98, 98, 50].
        fGT;        % Ground Truth of f, may be avaliable in simulation or via confocal
        isUseGPU;   % 1/0. whether we use GPU acceleration to compute the forward/backward function (specificially fft/ifft inside it)
        PRECISION;  % 'single' or 'double'. 'single' in GPU is about 2-3 times faster. raw sensor only has 14bit/16bit depth, which 32bit single with 23 bit precision is enough. but double precision is actually required for the objective function evaluation as the change will be very small, single precision is NOT enough found @2017.09.18
        CONVOLUTION_BOUNDARY; % 'Linear' or 'Circular', zero or circular boundary condition
        verbose;    % Whether to display and save intermediate results

        % (2) Computed Parameters
        g;          % Measured mfm image with background removed and normalized by gMax (currently assume single frame), 2d array of size gSz = [gy, gx],
        %gUint16;    % raw 16bit interger image read from .tif/.mat file (may only keep central z portion), same size as g. May be removed since 2017.05
        %gSz;        % Size of g. Either get from gFile if it exist, or set as same size as psf's y-x size
        gBg;        % Backround of g. 2d/scalar when background is spacial variant/invariant.
        gMax;       % Maximum of mfm (g) image after background removal, but before normalization
        gNorm;      % 2-norm of mfm (g) image  after background removal, but before normalization
        % function handles of forward model g = A*f, where A is gy*gx by fy*fx*fz matrix. AFun/AtFun are construct function handles for A/A'
        psf;        % psf with background removed, and normalized by AMaxSgVal, with PRECISION format. 3d array of size pSz = [gy, gx, fz]
        psfzEnergy; % computed energy of psf at each z slice, which is the sum of squares of each pixel at a z slice, maybe needed to compute diagATA
        diagATA;    % a N*1 column vector equal to diag(A'*A), maybe needed for preconditioner
        Vk;         % first k scaled eigenvectors after singular decomposition of A'*A = V*Lambda*V'. Assume Lambda is sorted from large eigen values to small eigen values.  Vk is the first k components Vk = V(:, 1:k)*sqrt(Lambda(1:k, 1:k)). 

        % psfUint16;  % Raw 16bit interger psf image read from .tif/.mat file (may only keep central z portion), same size as psf. May be removed since 2017.05
        psfBg;      % Background of psf. 3d/2d/scalar when background is spacial variant, z invariant, or xyz invariant.
        psfMax;     % Maximum of PSF after background removal, but before normalization        
        OTF;        % pre-computed OTF from the psf for forward/backward function handles.
                    % Fourier transfer of either original psf or zero-padded psf of size [gSz(1:2)+fSz(1:2)-1, fz]
                    % Could be either stored as matlab array or gpuarray (when the isUseGPU is true)
        ASgVals;    % Singular value of A matrix after nomarlization.

        % (2.3) Forward/Backward function handle
        ASz;          % [M, N] is the size of A matrix
        AFun;         % Forward function handle:  AFun(f)  = reshape(A*f(:), gSz)
        AtFun;        % 'Backward' function handle:  AtFun(g) = reshape(A'*g(:), fSz), where A' is conjugate transpose and A'=A.' (non-conjugate transpose) for real matrix.

        % (3) Noise parameters to be removed
        stdSensorNoise;
        stdPsfNoise;    % computed from background , probably is the gausian + Poisson noise


    end

    methods
        function self =  MFM(varargin)
            %Usage: self = MFM();
            %       self = MFM(psfFile, gFile)
            %       self = MFM(psfFile, gFile, 'fGT', fGT, 'fSz', fSz, 'voxSz', voxSz, 'gFrameNo', gFrameNo, 'psfBgFile', psfBgFile, 'gBgFile', gBgFile, 'verbose', verbose, 'PRECISION', 'single')
            % 1. Parameters from Inputs (Opitional Positional): psfFile, gFile
            %   psfFile: empty (let user interactively select a file), or a .mat/.tif file, or 3D psf matrix
            %   gFile:   empty (let user interactively select a file), or a .mat/.tif file, or 2D g matrix

            %% (1) Parse inputs to set: self.psfFile/gFile/fSz/voxSz/gFrameNo/psfBgFile/gBgFile/fGT/verbose
            p = inputParser;  % CaseSensitive=0, PartialMatching=1
            p.addOptional('psfFile',    [],     @(x) isempty(x) || (ischar(x) && exist(x, 'file')) || (isnumeric(x) && ndims(x)==3));
            p.addOptional('gFile',      [],     @(x) isempty(x) || (ischar(x) && exist(x, 'file')) || (isnumeric(x) && ndims(x)==2));
            p.addParameter('fGT',       [],     @(x) isempty(x) || (ischar(x) && exist(x, 'file')) || (isnumeric(x) && ndims(x)==3)); % 3D numeric array
            p.addParameter('psfBgFile', 0,     @(x) isempty(x) || (ischar(x) && exist(x, 'file')) || (isnumeric(x) && ndims(x)<=3)); %1D/2D/3D bg
            p.addParameter('gBgFile',   0,     @(x) isempty(x) || (ischar(x) && exist(x, 'file')) || (isnumeric(x) && ndims(x)<=2)); %1D/2D bg
            p.addParameter('gFrameNo',  1,      @(x) isempty(x) || isvector(x) && all(x>0) && all(x==round(x))); % positive interger array
            p.addParameter('fSz',       [151,151,101],  @(x) isempty(x) || isvector(x) && numel(x)==3 && all(x>0) && all(x==round(x))); % 3x1 or 1x3 positive interger
            p.addParameter('voxSz',     [108,108,50],     @(x) isempty(x) || isvector(x) && numel(x)==3 && all(x>0)); % 3x1 or 1x3 positive float
            p.addParameter('isUseGPU',  false,          @(x) isempty(x) || isscalar(x) && ismember(x, [true, false, 1, 0])); % scalar true/false
            p.addParameter('verbose',   false,          @(x) isempty(x) || isscalar(x) && ismember(x, [true, false, 1, 0])); % scalar true/false
            p.addParameter('PRECISION', 'double',       @(x) isempty(x) || ischar(x) && ismember(lower(x), {'single', 'double'})); % char 'single', 'double'
            p.addParameter('CONVOLUTION_BOUNDARY', 'linear',   @(x) isempty(x) || ischar(x) && ismember(lower(x), {'linear', 'circular'})); % char  'Linear' or 'Circular', zero or circular boundary condition
            p.parse(varargin{:});
            cellfun(@(x) evalin('caller', sprintf('self.%s=p.Results.%s;', x, x)), fields(p.Results)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;

            if isempty(self.psfFile)
                [filename, pathStr] = uigetfile('*.tif; *.mat', 'Pick the PSF file');
                self.psfFile = fullfile(pathStr, filename);
            end

            if  isempty(self.gFile) && isempty(self.fGT) % only if both gFile & fGT not specified, as empty gFile is fine if we known fGT in simulation
                [filename, pathStr] = uigetfile(fullfile(fileparts(self.psfFile), '*.tif; *.mat'), 'Pick a MFM Image (One or Multiple Frames)'); % start with psf folder
                self.gFile = fullfile(pathStr, filename);
            end

            %% (2.1) Set g with background removed, gBg and gFile to oberserved MFM 2D image to set:  self.g/gBg
            % We may not have gFile when fGT exist, when doing simulation
            self.setG();

            %% (2.2) Set psf with background removed and normlized by AMaxSV, will set:  self.psf/psfBg/psfMax/AMaxSV/stdPsfNoise/psfzEnergy/diagATA/Vk
            self.setPsf();

            %% (2.3) Compute forward model: self.OTF/AFun/AtFun
            self.setForwardModel();

            %% manually cast
            self.fGT = cast(self.fGT, self.PRECISION);

        end

        %% Add Noise to PSF or G
        function psfNoisy = addNoisePsf(self, varargin)
            psfNoisy = noise.add(self.psf, varargin{:});
        end
        function gNoisy = addNoiseG(self, varargin)
            gNoisy = noise.add(self.g, varargin{:});
        end

        %% (2.1)
        function setG(self, gFile)
            % (A) user specified gFile
            if exist('gFile', 'var') && ~isempty(gFile)
                self.gFile = gFile;
                self.setG();
                return
            end

            % (B) user not specified gFile. Set self.g from self.gFile, such as when called in class constructor.
            % Read/load g, only subtract background if it is from .tif/.tiff file
            if ischar(self.gFile) && exist(self.gFile, 'file') && (strcmp(self.gFile(end-3:end), '.tif') || strcmp(self.gFile(end-4:end), '.tiff'))
                % 1. Read g from a tiff file, subtract the background
                gUint16 = tif.read(self.gFile, self.gFrameNo);
                self.gBg = extractContent(self.gBgFile, self.PRECISION);
                if isempty(self.gBg)
                    % Note psf we used w = 100 as default, here we only use w = 10 (avoid objects);
                    warning('Using a DEPRECIATED INACCURATE method to cacluate the g image background!');
                    self.gBg = preprocess.calcBackgroundFrom4Corners(mean(double(gUint16), 3), 'verbose', self.verbose, 'w', 11); self.gBg = cast(self.gBg, self.PRECISION);
                end
                self.g = bsxfun(@minus, cast(gUint16, self.PRECISION), self.gBg);
                if any(self.g(:)<0)
                    warning('g has negative pixels after subtracting its background! Threshold those negative values to 0!');
                    self.g(self.g<0) = 0;
                end
            else
                % 2. Load or copy g from a .mat file or a 2D array, where its background is already removed
                self.g = extractContent(self.gFile, self.PRECISION);
                if ~isempty(self.gFile) && isnumeric(self.gFile)
                    self.gFile = 'NONE'; %User inputed an non-empty array, which will be copied to self.g), delete original copy in gFileto save memory
                end
                self.g = cast(self.g, self.PRECISION);
            end

            % Normalize g as a unit vector, i.e. ||g||_2 = 1  (Comments ro remove: earlier than 2018.01.22 set the maximum as 1)
            if ~isempty(self.g)
                self.gMax = max(self.g(:));
                self.gNorm = sqrt(sum(self.g(:).^2));
                self.g = self.g /self.gNorm; %self.g = self.g/self.gMax;
            end
        end

        %% (2.2)
        function setPsf(self)
            self.psfBg = extractContent(self.psfBgFile, self.PRECISION);  % change to set it to 0
            % 1. Read/load psf, only subtract background if it is from .tif/.tiff file
            if ischar(self.psfFile) && exist(self.psfFile, 'file') && (strcmp(self.psfFile(end-3:end), '.tif') || strcmp(self.psfFile(end-4:end), '.tiff'))
                % 1.1 Read psf from a tiff file, subtract the background
                % (a.) read the psf raw in uint16 format, crop central z slices corresponding to object's z space
                NPage = numel(imfinfo(self.psfFile));   % imfinfo return tiff structure, one element per page
                zCenter = ceil((1+NPage)/2); % good approximation as zCenter = 119/62/200/201 for 20160215_.../20160418_.../20160616_.../20160809... data with 331/125/399/401 slices
                halfSz = floor(self.fSz(3)/2);
                psfUint16 =  tif.read(self.psfFile, zCenter-halfSz:zCenter+halfSz);
                % (b.) subtract background psfBg from psf, after converting to self.PRECISION format, and
                if isempty(self.psfBg)
                    warning('Using a DEPRECIATED INACCURATE method to cacluate the psf background!');
                    [self.psfBg, self.stdPsfNoise]  = preprocess.calcBackgroundFrom4Corners(psfUint16, 'verbose', self.verbose, 'w', 101); self.psfBg = cast(self.psfBg, self.PRECISION);
                end
                self.psf = bsxfun(@minus, cast(psfUint16, self.PRECISION), self.psfBg);
                if any(self.psf(:)<0)
                    warning('psf has negative pixels after subtracting its background! Threshold those negative values to 0!');
                    self.psf(self.psf<0) = 0;
                end
            else
                % 1.2 Load or copy psf from a  .mat file or a 3D array
                self.psf = extractContent(self.psfFile, self.PRECISION);
                assert(size(self.psf,3) == self.fSz(3), 'The z dimension of psf mat file and fSz does not match');
                if isnumeric(self.psfFile) && ndims(self.psfFile)==3
                    self.psfFile = 'NONE'; %. User inputed an array (copied to self.psf) rather than a file.'; % save the memory, no need to keep two copies
                end
            end

            % 2. Compute OTF (before normalization  (tocheck: does compute the OTF from psf/psfMax gave better accuracy?)
            pSz = size(self.psf);
            switch lower(self.CONVOLUTION_BOUNDARY) % 'Linear' or 'Circular'
                case  'linear'  % Default: boundary handled with linear convolution, pad psf/OTF
                    oSz = [(pSz(1:2)+self.fSz(1:2)-1), pSz(3)]; % () is to avoid stupid matlab [1 -1] will be [1, -1] not 0
                case {'circular', 'circ', 'cir'}    % Boundary handled with circular convolution
                    oSz = pSz;
                otherwise
                    error('Unkown boundary condition (need to be either linear or circular) %s: ', self.CONVOLUTION_BOUNDARY);
            end
            % 2017.04.09_Unified handling of circular/linear OTF with the new MFMForward() and MFMBackward() function.
            if ~self.isUseGPU
                % CPU
                self.OTF = complex(zeros(oSz, self.PRECISION));
            else
                % GPU
                self.OTF = complex(gpuArray.zeros(oSz, self.PRECISION));
            end
            for z = 1:size(self.psf,3)
                self.OTF(:,:,z) = fftn(self.psf(:,:,z), oSz(1:2)); %fftn(psf(:,:,z), oSz(1:2))=fft2(padarray(I, oSz(1:2)-size(psf(:,:,z)), 0, 'post'));
            end


            % 3. Normalize psf/OTF so that ||A||_2 = 1, i.e. maximum singular value of A' = maximum eigen value of A*A' = 1
            % 3.1 backup the maximum psf intensity
            self.psfMax = max(self.psf(:));

            % 3.2 Compute the maximum singular value of A
            self.ASgVals = sqrt(sum(abs(self.OTF).^2, 3));% M = Ny*Nx size, all the singular values of A matrix, read Xiang's paper for why
            % 2018.02.09_old code below, remove if above work fine
            %self.ASgVals = sqrt(sum(conj(self.OTF.*conj(self.OTF)), 3)); % all the singular values of A matrix, read Xiang's paper for why
            %self.ASgVals = real(self.ASgVals); % Though ASgVals should always be real, if we use gpuArray, it will have 0 complex and cause error if not convert to real            
            if isa(self.ASgVals, 'gpuArray');
                self.ASgVals = gather(self.ASgVals); 
            end
            AMaxSgVal = max(self.ASgVals(:));
            if self.verbose
                ASgValsDC = sqrt(sum(sum(sum(self.psf, 1), 2).^2)); % this value should be same or close to AMaxSgVal
                fprintf('AMaxSgVal=%.5e, ASgValsDC=%.5e\n;', AMaxSgVal, ASgValsDC); % 98595496 vs 98595520 for 20160809_Bacterial_Confocal_PSFs\PSF_Ave.tif this may due to numerical computation error, because 2018.01.22 found same 170481617 for 20160809_Bacterial_Confocal_PSFs\PSF_Ave_objWithoutBgNonNeg.tif
                figure; imagesc(self.ASgVals); axis image; title('Singular Values of A');
            end

            normalizer =  AMaxSgVal;
            self.psf = self.psf / normalizer; % normalize to make the Twist solver more robust.
            self.OTF = self.OTF / normalizer;
            self.stdPsfNoise = self.stdPsfNoise / normalizer; 
            self.ASgVals = self.ASgVals(:) / normalizer; % convert to M*1 vector

           % (4) Compute additional information of the model which may be useful for the solver (preconditioner etc)
           %TODO: currently assume circular convolution,  g(:) = A*f(:), in future crop boundary g(:) = crop(A*f(:)) or pad?
            [Ny, Nx, Nz] = size(self.OTF);
            self.ASz = [Ny*Nx, Ny*Nx*z];
            self.psfzEnergy = squeeze(sum(sum(self.psf.*self.psf, 1), 2));
            self.diagATA = reshape(repmat(self.psfzEnergy(:)', self.ASz(1), 1), [], 1); %diag(ATA): make psfzEnergy a row vector, repat each elements by xSz(1)*xSz(2), to fix if there convolution is padded?
            % Vk: first k scaled eigenvectors after singular decomposition of A'*A = V*Lambda*V'. Assume Lambda is sorted from large eigen values to small eigen values.  Vk is the first k components Vk = V(:, 1:k)*sqrt(Lambda(1:k, 1:k)).
            M = Ny*Nx; 
            % 2018.02.10_just compute Vk = v = V(:,1)*sqrt(Lambda(1,1)) = V(:,1) * Sigma(1,1); where Lambda(1,1) is the largest
            % eigenvalue and Sigma(1,1) is the largest singular value.
            psfZSum = reshape(squeeze(sum(sum(self.psf, 1), 2)), 1, [])/sqrt(M);  % 1*Nz matrix
            self.Vk = repmat(psfZSum, M, 1);
            self.Vk = self.Vk(:);
            %self.Vk =  9*ones(size(self.diagATA)); % place holder;
            

            % Old before 2017.04 normalizer
            %normalizer = self.psfMax; % normalize self.psf so that maximum is 1,
            %normalize rows of A matrix with norm 1, so that A*A'*g has same energy as g
            % i.e., size(A,1) = sum(A(:).^2) = sum(psf(:).^2)/size(psf,3)*size(A,2)
            % i.e., sum(psf(:).^2) = numel(g)*size(psf,3)/numel(f) = numel(psf)/numel(f);
            %normalizer = sqrt(sum(self.psf(:).^2)/numel(self.psf)*prod(self.fSz));

            % TODO: Maybe, normalize each column of A matrix, i.e., make the sum((psf(:,:,z1).^2)) = sum((psf(:,:,z2).^2))
            % L1 norm -weighted, otherwise we will learn to pickup columns with large norm as the corresponding coefficient in f could be smaller for same amount of energy in g
            % psfZNorm= sqrt(squeeze(sum(sum(self.psf.*self.psf, 1), 2))); self.psf = bsxfun(@rdivide, self.psf, psfZNorm);% ?
            % AColNorm = repmat(reshape(psfZNorm/max(psfZNorm), 1, 1, size(self.psf,3)), [size(self.psf,1), size(self.psf,2), 1]); % Apply weights 1./W.*f(:)
        end



        %% (2.3): 2017.04.22 Can handle both withfiled stop and without fieldstop
        function setForwardModel(self)
            % Compute size of g. Either get from gFile if it exist, or set as same size as psf's y-x size
            if ~isempty(self.g)
                gSz = size(self.g);
            else
                gSz = [size(self.psf, 1), size(self.psf, 2)]; % not set g, infer gSz from the psf
            end
            self.AFun  = @(f) MFMForward(f,  self.OTF, gSz); % AFun  = @(f) reshape(A*f(:),  gSz);
            self.AtFun = @(g) MFMBackward(g, self.OTF, self.fSz); % AtFun = @(g) reshape(A'*g(:), fSz);
        end


        %% copy all the properties except the large data associated with psf and OTF to a new structure.
        function mfmPara = copyParasWithoutBigData(self)
            mfmPara = struct();
            % large array and function handles (that may implicilty requires large array as function handle parameters)
            excludedFieldsWithLargeData = {'psfUint16', 'psf', 'psfBg', 'gBg', 'psfNoisy', 'OTF', 'AFun', 'AtFun', 'diagATA', 'Vk'};
            p = properties(self);
            for i = 1:numel(p)
                if ~any(strcmp(p{i}, excludedFieldsWithLargeData))
                    mfmPara.(p{i}) = self.(p{i});
                end
            end
        end

        %% get the parent folder of a folder
        function parentFolder = getParentFolder(self, curFolder)
            if ~exit ('curFolder', 'var') || isempty(curFolder)
                curFolder = self.retFolder;  % default get the parent of result folder
            end
            dirParts = strsplit(fileparts(curFolder), filesep); %use fileparts to eliminate the last filesep
            parentFolder = strjoin(dirParts(1:end-1), filesep);
        end

    end

    methods(Static)
    end

    methods (Access = private)
%         function simulationSetup(self)
%
%
%             if ( isfield(self, 'simulatedObjShape') && ~isempty(self.simulatedObjShape) )
%                 shapeChoice = self.simulatedObjShape;
%             end
%             switch lower(shapeChoice)
%                 case 'confocal' %150x150x101
%                     % 0: load shape from confocal imaging
%                     self.fGT = load('20160809_Bacteria1_DoubleLobes_confocal_deconvolved.mat', 'fConfocalInterpPaddedShifted'); self.fGT = self.fGT.fConfocalInterpPaddedShifted;
%                 case 'confocalcrop' %150x150x61 cropped from 150x150x101 confocal
%                     self.fGT = load('20160809_Bacteria1_DoubleLobes_confocal_deconvolved.mat', 'fConfocalInterpPaddedShifted'); self.fGT = self.fGT.fConfocalInterpPaddedShifted; self.fGT = self.fGT(:,:,21:81);
%                 case 'cubic'
%                     % I: cubic shape of (2*winSz+1)^3, at the center shifted by beadShift space
%                     beadShift = 0; %beadShift = round([-78.3, -86.6, 121-119]); % bead shifted from the center to [y, x, z]
%                     beadPos = fCenter + beadShift;
%                     winSz = 10; % bead as a cube of size (2*winSz+1), if winSz=20, pixelSz = 100nm, then cube of about 4um
%                     self.fGT = zeros(self.fSz, self.PRECISION);
%                     self.fGT(beadPos(1)+(-winSz:winSz), beadPos(2)+(-winSz:winSz), beadPos(3)+(-winSz:winSz))= 1;
%                 case 'sphere'
%                     % II: Sphere shape of pi*4/3*radius^3, at the center of object space
%                     radius = 10;
%                     self.fGT = shape.generateSphere(self.fSz, radius);
%                 case 'cylinder'
%                     % III: cylinder shape of pi*radius*radius*cLen, at the center of object space
%                     radius = 8; cLen = 60;
%                     self.fGT = shape.generateCylinder(self.fSz, radius,cLen, 'z'); % 'y'
%                 otherwise
%                     error('unkown shape');
%                     %                     % assume it is a filename
%                     %                     fGTstruct = load(shapeChoice);
%                     %                     fGTfields = fieldnames(fGTstruct);
%                     %                     fGT0 = getfield(fGTstruct,fGTfields{1});
%                     %                     targetsize = min([size(fGT0);self.fSz]);
%                     %                     LL = floor((size(fGT0)-targetsize)/2);
%                     %                     UR = LL + targetsize - 1;
%                     %                     self.fGT = fGT0(LL(1):UR(1),LL(2):UR(2),LL(3):UR(3));
%                     %                     clear('fGTstruct','fGTfields','fGT0','targetsize','LL','UR');
%             end
%
%         end
     end

end
