function f = MFMBackward(g, OTF, fSz)
%function f = MFMBackward(g, OTF, fSz) compute backward (forward matrix transpose) model for MFM: f(:) = A'*g(:), where A' is (conjugate if complex) transpose of A.
% Inputs: g and OTF are required, fSz is opitional: not required for linear convolution (assume oSz(1:2)=fSz(1:2)+gSz(1:2)-1), only required for circular convolution.
% f:    3d array of size fSz = [fy, fx, fz], e.g. [150, 150, 101].  NOTE: The voxel origin of f is ceil((fSz+1)/2)
% g:    2d array of size gSz = [gy, gx],     e.g. [1024, 1024],  we always have gy>fy, gx>fx in practice.
% OTF:  3d array of size oSz = [oy, ox, fz] in MFM. The oSz determines the Discrete Fourier Transform (DFT) size.
%       Corresponding psf has size pSz = [gy, gx, fz] in MFM
%       2D Fourier Transform for each z Slice:  OTF(:,:,z) = fft2(psf(:,:,z), [oy, ox])
%       (1) linear convolution (default),   oSz(1:2) = pSz(1:2)+fSz(1:2)-1, e.g. [1024,1024]+[150,150]-1. The OTF is FFT of padded psf z slice
%       (2) circular convolution,           oSz(1:2) = pSz(1:2). The OTF is FFT of psf z slice
% fSz:  Opitional input. Not required for linear convolution. Only required for circular convolution.
%
% Author: Xiang Huang (xianghuang@gmail.com)
% 2016.09.13-14: add comments, cleaned code, verified for cases that fy:gy are even:even, odd:odd, even:odd, odd:even
% 2016.09.15-19: Change from circular convolution to linear convolution, Added zero-padding of FFT to handle boundary of two convolution, cleaned code
% 2016.09.20-26: unified handling of both circular and linear convolution
% 2017.04.07:    Code clean for reuse in I5M
% 2017.09.17:    GPU acceleration.
% Refer to: MFMForward()

% NOTE: Given real psf, let PSFz = ifft2(OTF(:,:,z)),  PSFz2 = ifft2(conj(OTF(:,:,z))); 
%   then PSFz2(y, x) = PSFz(oy-y, ox-x), i.e., PSFz2 and PSFz are symmetric around center

% (0) Preprocess
gSz = size(g);
oSz = size(OTF);
assert(~exist('fSz', 'var') || isempty(fSz) || oSz(3)==fSz(3), 'The depth of PSF/OTF and object volume f do not match!')
if ~exist('fSz', 'var') || isempty(fSz) || any(fSz ==0)
    fSz = [oSz(1:2) - gSz(1:2) + 1, oSz(3)];  % Default is linear convolution, where psf is padded before computing OTF
end

% (1) Pad if necessary. Inverse of MFMForward Step (3) Crop.
g = padarray(g, oSz(1:2)-gSz, 0, 'post'); % When circular convolution, g is not changed

% (2) Circular shift. Inverse of MFMForward Step (2) Circular shift
yxShift = ceil((fSz(1:2)-1)/2); % derived from origin - [1,1] = ceil((fSz(1:2)+1)/2) - 1, also same as floor(fSz(1:2)/2)
g = circshift(g, +yxShift);

% (3) Compute f padded then crop. Refer to MFMForward Step (1) where g = sum_z(A_z*f_z), where A_z*f_z is 2D circular convolution of psf(:,:,z) and f(:,:,z) padded
if ~isa(OTF, 'gpuArray')
    f = zeros(fSz);
    G = fftn(g, oSz(1:2));
    %parfor z = 1:fSz(3) %parfor (z = 1:fSz(3), 2) % 2017.09.17 cputime "for" vs "parfor 2/12/unspecified (though maximum 8 workers on shawn-ubuntu)":  55 vs 108/50/50 seconds. Because fft is already multi-threaded in core MATLAB on or before 2012.
    for z = 1:fSz(3) 
        fzSlicePadded = real(ifftn(conj(OTF(:,:,z)).* G)); % use real() in case small numerical computation erros to result non real after ifftn(). This won't happen in CPU, but happen in GPU
        f(:,:,z) = fzSlicePadded(1:fSz(1), 1:fSz(2));
    end
else
    fGPU = gpuArray.zeros(fSz);
    gGPU = gpuArray(g);    
    GGPU = fftn(gGPU, oSz(1:2));    
    for z = 1:fSz(3) 
        fzSlicePaddedGPU = real(ifftn(conj(OTF(:,:,z)).* GGPU));
        fGPU(:,:,z) = fzSlicePaddedGPU(1:fSz(1), 1:fSz(2));
    end
    f = gather(fGPU);
end
end
