function g = MFMForward(f, OTF, gSz)
%function g = MFMForward(f, OTF, gSz) computes forward model for MFM: g(:) = A*f(:).
% Inputs: f and OTF are required, gSz is opitional: not required for linear convolution (assume oSz(1:2)=fSz(1:2)+gSz(1:2)-1), only required for circular convolution.
% f:    3d array of size fSz = [fy, fx, fz], e.g. [150, 150, 101]. NOTE: Define the voxel origin of f as ceil((fSz+1)/2)
% g:    2d array of size gSz = [gy, gx],     e.g. [1024, 1024], we always have gy>fy, gx>fx in MFM.
% OTF:  3d array of size oSz = [oy, ox, fz] in MFM. The oSz determines the Discrete Fourier Transform (DFT) size.
%       Corresponding psf has size pSz = [gy, gx, fz] in MFM
%       2D Fourier Transform for each z Slice:  OTF(:,:,z) = fft2(psf(:,:,z), [oy, ox])
%       (1) linear convolution (default),   oSz(1:2) = pSz(1:2)+fSz(1:2)-1, e.g. [1024,1024]+[150,150]-1. The OTF is DFT of padded psf z slice
%       (2) circular convolution,           oSz(1:2) = pSz(1:2). The OTF is FFT of psf z slice
%       OTF could be either regular CPU array (then use CPU for fft/ifft) or GPU array (then use GPU for fft/ifft)
%
% Author: Xiang Huang (xianghuang@gmail.com)
% 2016.09.13-14: add comments, cleaned code, verified for cases that fy:gy are even:even, odd:odd, even:odd, odd:even
% 2016.09.15-19: Change from circular convolution to linear convolution, Added zero-padding of FFT to handle boundary of two convolution, cleaned code
% 2016.09.20-26: unified handling of both circular and linear convolution
% 2017.04.07:    Code clean for reuse in I5M
% 2017.04.17:    Handeling field of view removed
% 2017.09.17:    GPU acceleration.
% 2017.09.23:    Save almost half computation, by only do one ifftn on the final G, rather doing g = g + ifftn(OTF(:,:,z).*fftn(f(:,:,z), oSz(1:2)));
% Refer to: MFMBackward()

%global OTFGPU

% (0) Preprocess
oSz = size(OTF); 
fSz = size(f); 
assert(oSz(3) == fSz(3), 'The depth of psf/OTF and object volume f does not match!')
if ~exist('gSz', 'var') || isempty(gSz) || any(gSz ==0)
    gSz = oSz(1:2) - fSz(1:2) + 1;  % Default is linear convolution, where psf is padded before computing OTF
end

% (1) Compute padded g as circular convolution, where f/g is padded to the same size as OTF.
% If linear convolution, oSz(1:2) > gSz(1:2), computed g is padded version; If circular convolution oSz(1:2) = gSz(1:2)
if ~isa(OTF, 'gpuArray')
    G = complex(zeros(oSz(1:2), class(f)));
    %parfor z = 1:oSz(3) %parfor (z = 1:oSz(3), 2)  % 2017.09.17 cputime "for" vs "parfor 2/12/unspecified (though maximum 8 workers on shawn-ubuntu)":  55 vs 108/50/50 seconds. Because fft is already multi-threaded in core MATLAB on or before 2012;
    for z = 1:oSz(3) 
        % g = sum_z(A_z*f_z), where A_z*f_z is 2D circular convolution of psf(:,:,z) and f(:,:,z) padded
        G = G  +  OTF(:,:,z).*fftn(f(:,:,z), oSz(1:2));  % fftn(f(:,:,z), oSz(1:2)) =  fft2(padarray(f(:,:,z), oSz(1:2)-fSz(1:2), 0, 'post'));
    end
    g = real(ifftn(G));
else
    fGPU = gpuArray(f);
    GGPU = complex(gpuArray.zeros(oSz(1:2), class(f)));    
    for z = 1:oSz(3)        
        GGPU = GGPU  +  OTF(:,:,z).*fftn(fGPU(:,:,z), oSz(1:2));  % fftn(f(:,:,z), oSz(1:2)) =  fft2(padarray(f(:,:,z), oSz(1:2)-fSz(1:2), 0, 'post'));
    end    
    g = gather(real(ifftn(GGPU)));
end

% (2) Circular shift so that the 'origin' of f will change from [1,1] to origin = ceil((fSz+1)/2). HERE we assume gSz = psfSz(1:2)
% When f = zeros(fSz); f(origin(1), origin(2), z) = 1; we have
%       g = psf2(:,:,z) circular shift by origin - [1, 1] before step (2), and have
%       g = psf2(:,:,z) after step (2), where
%       psf2(:,:,z) is defined as ifftn(OTF(:,:,z)).
% This is because before step (2), when f = zeros(fSz); f(1,1,z) = 1; we have g = psf2(:,:,z).
yxShift = ceil((fSz(1:2)-1)/2); % derived from origin - [1,1], also same as floor(fSz(1:2)/2)
g = circshift(g, -yxShift);  % shiftback, i.e., g = circshift(g, floor((fSz(1:2)+1)/2));

% (3) Crop if necessary: 2017.05.08_check: we assume g size is always the same as psf size for MFM
% When f = zeros(fSz); f(origin(1), origin(2), z) = 1; where origin = ceil((fSz+1)/2), we have 
%   g = psf2(:,:,z) before step (3), and 
%   g = psf(:,:,z) after step (3), where
%   psf2 = psf for circular convolution, or
%   psf2 = padarray(psf, [fy-1, fx-1], 0, 'post') for linear convolution.
g = g(1:gSz(1), 1:gSz(2)); % When circular convolution, g is not changed
% g = g((1:gSz(1))+yxShift(1), (1:gSz(2))+yxShift(2)); % For linear convolution, same result as two steps g = circshift(g, -yxShift); g = g(1:gSz(1), 1:gSz(2));
end