%
%
% This function solves the regularization problem 
%
%     arg min_x = 0.5*||A*x - b||_2^2 + regWt * regFun(x), 
%
% where A is a generic matrix and regFun(.) is a regularizarion function such that the solution is known of the denoising problem 
%
%     denoiseFun(b, regWt) = arg min_x = 0.5*|| x - b ||_2^2 + regWt * regFun( x ).
%
% The x is either unconstrained or constrained to W*x >= 0. 
% 
% For further details about the TwIST algorithm, see the paper:
%
% J. Bioucas-Dias and M. Figueiredo, "A New TwIST: Two-StepIterative Shrinkage/Thresholding Algorithms for Image Restoration",  IEEE Trans on Image processing, 2007.  
% J. Bioucas-Dias and M. Figueiredo, "A Monotonic Two-Step Algorithm for Compressive Sensing and Other Ill-Posed Inverse Problems", submitted, 2007.
%
% Authors: Jose Bioucas-Dias and Mario Figueiredo, October, 2007.
% 
% Please check for the latest version of the code and papers at www.lx.it.pt/~bioucas/TwIST
%
% -----------------------------------------------------------------------
% Copyright (2007): Jose Bioucas-Dias and Mario Figueiredo
% 
% TwIST is distributed under the terms of the GNU General Public License 2.0.
% 
% Permission to use, copy, modify, and distribute this software for any purpose without fee is hereby granted, provided that 
% this entire notice is included in all copies of any software which is or includes a copy or modification of this software 
% and in all copies of the supporting documentation for such software. This software is being provided "as is", without any 
% express or implied warranty.  In particular, the authors do not make any representation or warranty of any kind concerning 
% the merchantability of this software or its fitness for any particular purpose."
% ----------------------------------------------------------------------
% 
%  ===== Required inputs =============
%multiplication
%  b: 1D vector (sound or vectorized 2D image or vectorized 3D volume) or 2D array (non-vectorized image) observations.
%     
%  A: If b and x are both 1D vectors, A can be a m*n (where m is the size of b and n the size of x) matrix or a handle to 
%     a function that computes products of the form A*x.
%     In any other case (if b and/or x are 2D arrays), A has to be passed as a handle AFun to a function which computes products of 
%     the form A*x; another handle to a function AtFun which computes products of the form A'*x is also required in this case. 
%     The size of x is determined as the size of the result of applying AtFun.
%
%  regWt: regularization parameter, usually a non-negative real parameter of the objectiv function (see above). 
%  
%
%  ===== Optional inputs =============
%  'verbose'  = work silently (0) or verbosely (1)
%  
%  'regFun' = function handle to regularizer needed to compute the objective function.
%          Default = ||x||_1  
% 
%  'denoiseFun' = denoising function handle; handle to denoising function
%          Default = soft threshold.
%
%  'AtFun'    = function handle for the function that implements the multiplication by the conjugate of A, when A is a function handle. 
%            If A is an array, AtFun is ignored.
%
%  'stopCriterion' = type of stopping criterion to use
%                    0 = algorithm stops when the relative change in the number of non-zeros of x falls below 'tolA'
%                    1 = stop when the relative change in the objective function falls below 'tolA'
%                    2 = stop when the relative norm of the difference between two consecutive estimates falls below tolA
%                    3 = stop when the objective function becomes equal or less than tolA.
%                    Default = 1.
%
%  'tolA' = stopping threshold/tolerance for the main phase of the algorithm; 
%
%  'maxIterA' = maximum number of iterations allowed in the main phase of the algorithm.
%
%  'minIterA' = minimum number of iterations performed in the main phase of the algorithm.
% 
%
%  'x0' must be one of {'zero', 'random', 'Atb', array}
%               'zero'   -> Initialization at zero. 
%               'random' -> Random initialization.
%               'Atb'    -> initialization with A'*b.
%               array    -> initialization provided by the user.
%               Default = 'zero';
%
%  'isMonotone' = enforce monotonic decrease in objFunVals. 
%               true -> enforce monotonicity
%               false -> don't enforce monotonicity.
%               Default = true;
%
%  'sparse'   = {false,true} accelarates the convergence rate when the regularizer 
%               Phi(x) is sparse inducing, such as ||x||_1.
%               Default = true               
%             
%  'xGT' = if the ground truth underlying x is passed in this argument, MSE evolution is computed
%
%  'lambda1' = lambda1 parameters of the  TwIST algorithm:  
%           NOTE: Do NOT confuse this internal optimization parameter 'lambda1' with regulazier 'regWt'
%             Optimal choice: lambda1 = min eigenvalue of A'*A.
%             If min eigenvalue of A'*A == 0, or unknwon, set lambda1 to a value much smaller than 1.
%           NOTE: TwIST is not very sensitive to this parameter
%             Rule of Thumb: 
%                 lambda1=1e-4 for severyly ill-conditioned problems
%                 lambda1=1e-2 for mildly  ill-conditioned problems
%                 lambda1=1    for A unitary direct operators
%
%             Default: lambda1 = 0.04.
%
%             Important Note: If (max eigenvalue of A'*A) > 1, the algorithm may diverge. This is be avoided 
%             by taking one of the follwoing  measures:
% 
%                1) Set 'isMonontone' = 1 (default)
%                  
%                2) Solve the equivalenve minimization problem
%
%             min_x = 0.5*|| (A/c)*x - (b/c) ||_2^2 + (regWt/c^2) * regFun( x ), 
%
%             where c > 0 ensures that  max eigenvalue of (A'A/c^2) <= 1.
%
%  'alpha' = parameter alpha of TwIST (see ex. (22) of the paper)         
%             Default alpha = alpha(lambdaN=1, lambda1)
%   
%  'beta'  =  parameter beta of twist (see ex. (23) of the paper)
%              Default beta = beta(lambdaN=1, lambda1)          
%
%
% ===================================================  
% ============ Extra Step of Debias ==============================
%  'isDebias'     = debiasing option: 1 = yes, 0 = no.
%                 Note: Debiasing is an operation aimed at the computing the solution of the LS problem 
%
%                         arg min_x = 0.5*|| b - As * xs ||_2^2 
%
%                 where As is the  submatrix of A obatained by  deleting the columns of A corresponding of components
%                 of x set to zero by the TwIST algorithm                 
%
%  'tolD' = stopping threshold/tolerance for the debiasing phase:
%
%  'maxIterD' = maximum number of iterations allowed in the debising phase of the algorithm.
%
%  'minIterD' = minimum number of iterations to perform in the debiasing phase of the algorithm.
%
%  If no debiasing takes place, parameter 'tolD'/'maxIterD'/'minIterD', if present, is ignored.

% ===================================================  
% ============ Outputs ==============================
%   x = solution of the main algorithm
%
%   x_debias = solution after the debiasing phase;
%              If no debiasing phase took place, this variable is empty, x_debias = [].
%
%   objFunVals = sequence of values of the objective function
%
%   cpuTimes = CPU time after each iteration
%
%   debias_start = iteration number at which the debiasing phase started. 
%                  If no debiasing took place, this variable is returned as zero.
%
%   SNRs = sequence of SNR values, with respect to xGT, if it was given; SNR(x, xGT) = 10*log10( sum(xGT(:).^2)  / sum((x(:)-xGT(:)).^2));
%          if it was not given, SNRs is empty, SNRs = [].
%
%   max_svd = inverse of the scaling factor, determined by TwIST, 
%          applied to the direct operator (A/max_svd) such that  every IST step is increasing.
%
%       2017.04 Xiang added
%  isNonNegative = whether the signal is non negative in original or transformed domain.
%           With this constraint, the algorithm converges much faster at least for original domain
%
%  Dic =  Dictionary matrix or handle so that    Dic*x >= 0
%          
%  DicInv = matrix or handle so that  
% 
% 2016.10.12: Xiang modified the "Read the optional parameters" Section: 
%           empty input [] will be ignored
%           original code: par = xxx; 
%           new code: if ~isempty(xxx); par = xxx; end
% 2017.04:  Xiang Modified: Unified Parse Input for TwIST and FISTA
%           Opitional Input to handle 'isNonNegative' true or false
%           NOTE, the isNonNegative maybe used for both with and without additional tranform matrix
% ========================================================

% test for number of required parametres

classdef SOLVER < handle
    %SOLVER class with selection of optimization algorithms and corresponding parameters
    properties (Constant)
        %SOLVER_TYPE2FUNstr = containers.Map({1, 2, 3}, {'twist', 'fista', 'spiral'}); %{'1:twist', '2:fista', '3:spiral'};
    end
    properties
        
        
        %% 1. Input Parameters
        % (1.1) Optimization  Model:
        %   minimize  0.5*||A*T(x) - b||_2^2 + regWt*regFun(x),
        %   (opitional) s.t. x >= 0
        %  T is a function hanlde named Transform. If it is not specified, T(x) = x;        
        opts = struct( ...
            ... % (I) Model parameters
            ... % (1) Model Data
            'b', [], ...
            'bSz', [], ... 
            'AFun', [], ...  % function handle same as MFM.AFun, 3D input and 2D output
            'AtFun', [], ... % function handle same as MFM.AtFun, 2D input and 3D output
            'A',   [], ... % @A_operator class object that includes both AFun and AtFun, takes any 1D/2D/3D inputs and gives 1D output
            'AMatrix',   [], ... % Opitional matrix form, only used for small scale problem, or for debugging:  2017.09.23 Changed name from A to AMatrix as we want to use A as @A_operator            
            'xGT', [], ...  % maybe arguable to where put xGT & x0, we just put it the model, as it is more related to the model and maybe directly specified from the model without or with little compution
            'x0',  [], ...
            'xSz', [], ... % Inferred from AtFun(b), or speficied when xSz can't be inferred: if AFun is matrix A and both x0/xGT are not numeric array. NOTE: we always specify b, so no need for bSz
            ... %  (2) Model function
            'regWt', [], ... %10.^[-1:1:3];  regularization weight parameter,
            'regFun', [], ... % regularizer function handle or a specify string choose from {'L0', 'L1', 'TV', 'Wavelet'} TV norm is not stable@2016.11.13, change back to the old-fashioned L1norm
            'objFun', [], ... % currently should be 0.5*||A*T(x) - b||_2^2 + regWt*regFun(x), or 0.5*||resid(x)||_2^2 + regWt*regFun(x);
            'denoiseFun', [], ...
            'isNonNegative', [], ...
            'Transform', [], ...            
            ... %%% (II) Optimization Algorithm Parameters
            ... % (0a) Parameters for Any Algorithm: Main Step
            'solType', [], ...         % solver choose from {'twist', 'fista', 'spiral'}
            'verbose', [], ...        % whether show intermediate results
            'stopCriterion', [], ...              %  choose from {'objectiveChange', 'xChange', 'nonzeroChange', 'differenceFromXGT', 'smallDerivative', 'dualityGap'}
            'tolA', [], ...                       % stop theshold: 1e-4~1e-6 if normalize psf etc..  % 60/80dB SNR  for tolA = 1e-3/1e-4 when stopCeriterion = 2
            'maxIterA', [], ...
            'minIterA', [],...
            'sampleInterval', [], .... % specifies when to keep intermediate result. refer to solver.updateSampledHist for usage
            ...% (0b) Parameters for Any Algorithm: Debais Step (optional step)
            'isDebias', [], ...   %1, debias, 0, no-debias, only used for 'twist': % Debias is good for simulated solid object with uniform intensity, but maybe bad for real object
            'tolD', [], ...
            'maxIterD', [], ...
            'minIterD', [], ...
            ... %%% Preconditioner 
            'preconditioner', [],...  % preconditioner could be []/M (Materix)/function handle or string from {'', 'zeroATA', 'identityATA', 'diagATA',  {'lowRankATA', k}}; 
                                ...   % 0:  '' or []:     no preconditioner
                                ...   % 1.  'zeroATA':    approximate  A'*A as a all zero matrix
                                ...   % 2:  'identityATA':  approximate A'*A as a scaled identity matirx rho*ATA
                                ...   % 3:  'diagATA':     approximate A'*A as diagonal matrix
                                ...   % 4:  'lowRankATA'/{'lowRankATA', k}: approximate A'*A as rank k (currently rank 1) matrix Vk*Vk' for its k eigenvectors corresponding to the largest k eignvalue
                                ...   %                               2018.02.05_ just use 'lowRankATA' use its largest eigenvalue lambda with corresponding eigenvector v rank 1, future textned to k                                
            'diagATA',  [],...
            'Vk',  [],...
            ... %%% IP Parameters
            'muCG', [], ...
            ... %%% Twist Parameters
            'isMonotone', [], ...
            'lambda1', [], ...
            'lambdaN', [], ...
            'sparse', [], ...
            'alpha', [], ...
            'beta', [], ...
            ... % (2) Fista Parameters
            'Lipschitz0', []);                                


        %% 2. Output result from optimization
        % retPath:    retPath stores the result, default it's the subfolder 'results' of the mfm.gFile folder
        retPath  = [];  % e.g. C:\Dropbox\smallWorlds\20161213_Periplasm\results\
        % prefix of save file(s), it has mfm part and solver part:  mfmFileName + frameNo + solverStartTime + computationTime + solType + regFun + regWt + nVoxel .        
        retPrefix = [];  %  e.g. 'bacteria1_mfm_ex750em750_f100_mean10_f1__20170117_0121_3000s_solType=Twist_regFun=1_regWt=1.0e+00_nVoxel=52466
        startTime = []; % e.g. 20170107_2323 computed from datestr(now, 'yyyymmdd_hhMM');
        
        %[f, iter, objective, reconerror, cpuTimes, solPath]
        f = [];     % recovered object value, e.g., 3D volumetric object image of size [fy, fx, fz]
        iter = [];   % total number of iterations performed by the algorithm. Clearly this number will be between minIter and maxIter and will depend on the chosen stopping criteria.
        objFunVals = []; % The evolution of the objective function f(x) +regWt*regFun(x) with the number of iterations.  The initial value of the objectivefunction is stored in objective(1), so length of objective is iter + 1.
        regFunVals = []; % The evolution of the regula function f(x) +regWt*regFun(x) with the number of iterations.  The initial value of the objectivefunction is stored in objective(1), so length of objective is iter + 1.
        sampledHist = []; % sampledHist is a structure array that keeps intermediate result of x, and corresponding k, cpuTime after a iteration that satitisfy opts.sampleInterval
        
        % dualFunVals/logBarriers/ts/itersCG for each iteration, are currently only avaliable for interior point method solveIP
        dualFunVals = []; % The function values for dual function, which is a lower bound on the optimal function value
        logBarriers = [];
        ts = [];
        itersCG = [];
        
        
        cpuTimes = [];   %total elapsed time to reach each teration. Clock starts at time cpuTimes(1) = 0 , so its length is iter + 1.
        reconRelativeRMSE = [];  %The evolution of the relative-RMSE with the number of iterations: sqrt(sum((f(:)-fGT(:)).^2))./sqrt(sum(fGT(:).^2))
        % It can only be computed if the ground truth is rovided using the 'TRUTH' option.  The error corresponding to the initial value is stored in reconerror(1), so the length of reconRelativeRMSE is iter + 1.
        solPath = [];  %Record of the intermediate iterates reached while computing the solution x.
        % Both the "noisy" step solutionpath.step and the "denoised" iterate solutionpath.iterate are saved.
        % The initialialization for the algorithm is stored in solutionpath(1).iterate.
        % Since there is no corresponding initial step, solPath(1).step contains all zeros, so solPath has length iter + 1.
        fDebias = []; % 'twist' only, the solution after debiase phase. If no debias phase, it will be [];
        iterDebiasStart = []; % iteration number at which the debaising phase started. If no debiase phase, it will be 0.
        
    end
    methods        
        %function self =  SOLVER(mfm, varargin)
        function self =  SOLVER(varargin)
            solOpts = solver.parseInput(varargin{:});
            self.opts = struct.copy(solOpts, self.opts);
            %cellfun(@(x) evalin('caller', sprintf('self.%s=opts.%s;', x, x)), fields(opts)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;              
            self.startTime = datestr(now, 'yyyymmdd_hhMM'); % e.g. 20170107_2323                    
        end

        function solve(self)
            solOpts = self.opts;
            fprintf('Solve with paramaters: solType=%s, regFun=%s, denoiseFun=%s, regWt=%.2e, tolA=%.2e, .\n', solOpts.solType, func2str(solOpts.regFun), func2str(solOpts.denoiseFun), solOpts.regWt, solOpts.tolA);                        
            switch lower(solOpts.solType)
                case lower('FISTA')
                      [self.f, self.objFunVals, self.cpuTimes, SNRs, self.sampledHist]= solveFista(solOpts);
                case lower('TwIST')                    
                     [self.f, self.fDebias, self.objFunVals, self.cpuTimes, self.iterDebiasStart, SNRs, self.sampledHist, self.regFunVals] = solveTwist(solOpts);
                case lower('IP')
                    [self.f, self.objFunVals, self.cpuTimes, SNRs, self.sampledHist, self.regFunVals, self.dualFunVals, self.logBarriers, self.ts, self.itersCG] = solveIP(solOpts);
                case lower('IPPDInfeasible') % primal-dual interior point method - infeasible
                    [self.f, self.objFunVals, self.cpuTimes, SNRs, self.sampledHist, self.regFunVals, self.dualFunVals, self.ts, self.itersCG] = solveIPPDInfeasible(solOpts);                                        
                case lower('IPPD') % primal-dual interior point method
                    [self.f, self.objFunVals, self.cpuTimes, SNRs, self.sampledHist, self.regFunVals, self.dualFunVals, self.ts, self.itersCG] = solveIPPD(solOpts); 
                case lower('IPPDGond') % 2014_Fountoulakis, Gondzio_Matrix-free Interior Point Method for Compressed Sensing Problems
                    [self.f, self.objFunVals, self.cpuTimes, SNRs, self.sampledHist, self.regFunVals, self.dualFunVals, self.ts, self.itersCG] = solveIPPDGond(solOpts); 
                case {lower('TwIST_NonNegative'), lower('TwIST_Original')}
                    [self.f, self.fDebias, self.objFunVals, self.cpuTimes, self.iterDebiasStart, mses]= ...
                        TwIST_NonNegative(solOpts.b, solOpts.AFun, solOpts.regWt,... 
                        'isNonNegative', solOpts.isNonNegative,... % Opitional constraint f is non-negative: The only modification of original Twist.
                        'AT', solOpts.AtFun, ...                        
                        'Psi', solOpts.denoiseFun,...
                        'Phi', solOpts.regFun,...
                        'Initialization', solOpts.x0,...
                        'TRUE_X', solOpts.xGT,...
                        'MaxiterA', solOpts.maxIterA,...                        
                        'StopCriterion',1,... % always use realtive objective function change
                        'ToleranceA', solOpts.tolA,...
                        'Debias', solOpts.isDebias, ...
                        'MaxiterD', solOpts.maxIterD,... % NOTE MISSPELLING OF MaxiterD in the original TwIST code which was corrected here
                        'Monotone', solOpts.isMonotone, ...
                        'Verbose', solOpts.verbose);
                case lower('GPSR_BB')   % three times faster than TwIST (without nonnegative constraint)                    
                    [self.f, self.fDebias, self.objFunVals, self.cpuTimes, self.iterDebiasStart, mses]= ...
                        GPSR_BB(solOpts.b, solOpts.AFun, solOpts.regWt,... 
                        'AT', solOpts.AtFun, ...
                        'Initialization', solOpts.x0,...
                        'TRUE_X', solOpts.xGT,...
                        'MaxiterA', solOpts.maxIterA,...                        
                        'StopCriterion', solOpts.stopCriterion,...                                                
                        'ToleranceA', solOpts.tolA,...
                        'Debias', solOpts.isDebias, ...
                        'MaxiterD', solOpts.maxIterD,...
                        'Monotone', solOpts.isMonotone, ...
                        'Verbose', solOpts.verbose);
                case lower('GPSR_BB_NonNegative')   % Xiang modified GPSR_BB for non-negative x                   
                    [self.f, self.fDebias, self.objFunVals, self.cpuTimes, self.iterDebiasStart, mses]= ...
                        GPSR_BB_NonNegative(solOpts.b, solOpts.AFun, solOpts.regWt,... %TwIST(g, AFun, self.regWt,...
                        'AT', solOpts.AtFun, ...
                        'Initialization', solOpts.x0,...
                        'TRUE_X', solOpts.xGT,...
                        'MaxiterA', solOpts.maxIterA,...                        
                        'StopCriterion', solOpts.stopCriterion,...                                                
                        'ToleranceA', solOpts.tolA,...
                        'Debias', solOpts.isDebias, ...
                        'MaxiterD', solOpts.maxIterD,...
                        'Monotone', solOpts.isMonotone, ...
                        'Verbose', solOpts.verbose);                                        
                case lower('GPSR_Basic')   % three times faster than TwIST (without nonnegative constraint)                    
                    [self.f, self.fDebias, self.objFunVals, self.cpuTimes, self.iterDebiasStart, mses]= ...
                        GPSR_Basic(solOpts.b, solOpts.AFun, solOpts.regWt,... %TwIST(g, AFun, self.regWt,...
                        'AT', solOpts.AtFun, ...
                        'Initialization', solOpts.x0,...
                        'TRUE_X', solOpts.xGT,...
                        'MaxiterA', solOpts.maxIterA,...                        
                        'StopCriterion', solOpts.stopCriterion,...                                                
                        'ToleranceA', solOpts.tolA,...
                        'Debias', solOpts.isDebias, ...
                        'MaxiterD', solOpts.maxIterD,...
                        'Verbose', solOpts.verbose);
                case lower('spiral')
                    gNonNeg = self.b; gNonNeg(gNonNeg<0) = 0;
                    %[self.f, self.iter, self.objFun, self.reconRelativeRMSE, self.cpuTimes]= ...
                    [self.f, self.iter, self.objFunVals]= ...
                       SPIRALTAP_Xiang(gNonNeg, solOpts.AFun, solOpts.regWt,...
                        'AT', solOpts.AtFun, ...
                        ...'truth', solOpts.xGT,...
                        'Initialization', solOpts.x0,...
                        'stopcriterion',4,... % spriral 4 = twist 1, relative changes in objective: abs(f-prev_f)/prev_f;
                        'tolerance', solOpts.tolA,...
                        'maxIter', solOpts.maxIterA,...
                        'minIter',50,...%...'minIter',5,...
                        'alphainit',1,...
                        'alphamin', 1e-30,...
                        'alphamax', 1e30,...
                        'alphaaccept',1e30,...
                        'logepsilon',1e-10,...
                        'saveobjective',1,...
                        'savereconerror',0,... %1
                        'savecputime',0,... %1
                        'savesolutionpath',0,...
                        'verbose', solOpts.verbose);
                otherwise
                    disp('Unknown method.')
            end
            
            % Compute iteration number and relative RMSE for the reconstruction (if xGT exist)
            self.iter = numel(self.cpuTimes)-1;
            if ~isempty(solOpts.xGT)
                if exist('SNRs', 'var') && ~isempty(SNRs)
                    mses = mean(solOpts.xGT(:).^2) * 10.^(-SNRs/10);
                end
                if exist('mses', 'var') && ~isempty(mses)
                    self.reconRelativeRMSE = sqrt(mses) ./ sqrt(mean(solOpts.xGT(:).^2));                
                end
            end            
        end
        

        function showSaveResult(sol, mfm, extraPrefix)
            % TODO: either remove mfm, or if keep mfm and delete the sol.b/sol.A/sol.At/sol.xGT/sol.x0 to save disk space            
            %Save the result as .mat file. NOTE: we want to name it as sol rather than sol as we will save the whole structure
            % extraPrefix is very useful for debugging of diffent opitions
            if (~exist('th', 'var') || isempty(th))
                th = 10/2^16; %i.e., 1.5e-4, pixel value 10 is purely noise in 16bit sensor. Another choice is to use 10*eps('single') = 1e-6
            end
            if ~exist('extraPrefix') || isempty(extraPrefix)
                extraPrefix = '';
            end
            % 1. save mfmPara, sol and gFrom f
            mfmPara = mfm.copyParasWithoutBigData();     
            solPara = sol.copyParasWithoutBigData();
            gFromAf = mfm.AFun(sol.f); %gFromAfDebias = mfm.AFun(mfm.fDebias);;            
            nVoxel = compare.nnz(sol.f, 1e-4); % TODO: CHANGE 1e-4 to flexible threshold
                        
            % 2. Set retPath/retPrefix
            [sol.retPath, gFileNameNoExt] = fileparts(mfm.gFile);
            if strcmpi(mfm.gFile(1:4), 'none')                            
                [sol.retPath, ~] = fileparts(mfm.psfFile);
            end
            sol.retPath = fullfile(sol.retPath, 'results'); % append subfolder
            if ~exist(sol.retPath, 'dir') 
                mkdir(sol.retPath); 
            end            
            
            % prefix of save file(s), it has mfm part and solver part:  mfmFileName + frameNo + solverStartTime + computationTime + solType + regFun + regWt + nVoxel + (nVoxelGT + psnrSparse if has groundTruth) .
            % We do NOT put the psfFileName here as it maybe cause the final name to be too long, and psf is usually fixed one, not
            % change as often as the mfmFileName
            %  e.g. 'bacteria1_mfm_ex750em750_f100_mean10_f1__20170117_0121_3000s_twist_L1_reg1.0e+00_nnz52466
            timeStr = sol.startTime;
            if ~isempty(sol.cpuTimes)
                timeStr = sprintf('%s_%ds', timeStr, round(sol.cpuTimes(end)));
            end
            % 2017.06.14: file name can't contain string of regFun = @(f)sum(abs(f(:))) in windows (ok in linux), so drop it for now 
            %sol.retPrefix = sprintf('%s%s_f%03d_%s_solType=%s_regFun=%s_regWt=%.1e_nVoxel=%d', extraPrefix, gFileNameNoExt, mfm.gFrameNo, timeStr, sol.solType, sol.regFun, sol.regWt, nVoxel);
            % 2018.05.25_shorten file name (260 limit reached!)  by remove 'solType=', change 'regWt=' to 'reg', change 'nVoxel=' to 'nnz'
            %sol.retPrefix = sprintf('%s%s_f%03d_%s_solType=%s_regWt=%.1e_nVoxel=%d', extraPrefix, gFileNameNoExt, mfm.gFrameNo, timeStr, sol.opts.solType, sol.opts.regWt, nVoxel);
            sol.retPrefix = sprintf('%s%s_f%03d_%s_%s_reg%.1e_nnz%d', extraPrefix, gFileNameNoExt, mfm.gFrameNo, timeStr, sol.opts.solType, sol.opts.regWt, nVoxel);
           
            if ~isempty(mfm.fGT) &&  any(mfm.fGT(:) ~= 0)
                nVoxelGT = compare.nnz(mfm.fGT);
                %Nf = numel(mfm.fGT);
                [psnr_, psnrSparse] = difference(sol.f, mfm.fGT); %mse_ = sum((f(:)-fGT(:)).^2)/Nf;   psnr_ = 10*log10(max(fGT(:))^2/mse_);
                sol.retPrefix = sprintf('%s_nnzGT%d_psnrSparse%.2fdB',  sol.retPrefix, nVoxelGT, psnrSparse);
%                 if mfm.isAddImgNoise
%                     sol.retPrefix = sprintf('mfmGain=%.3e_%s', mfm.gain, sol.retPrefix);
%                 end
            end
                        
            %% hack 2017.04.24_            
            save(fullfile(sol.retPath, [sol.retPrefix '.mat']), 'mfmPara', 'solPara', 'gFromAf')
            imwrite(gFromAf/max(gFromAf(:)), fullfile(sol.retPath, [sol.retPrefix '_gFromAf.png']), 'BitDepth', 16);
            
            % 2. show reconstructed volume
            if (mfm.verbose)
                hf = figure(11);
            else
                hf = figure('Visible','off');
            end
            show3dPointsWithProjections(sol.f, mfm.voxSz);
            set(gcf, 'outerposition', [500, 500, 1600, 1000], 'PaperPositionMode','auto');
            print(hf, fullfile(sol.retPath, [sol.retPrefix '_f.png']), '-dpng'); 
            tif.write(single(sol.f), fullfile(sol.retPath, [sol.retPrefix '_f.tif'])); %2018.05.20 added write single float .tif
        end

        function solPara = copyParasWithoutBigData(sol)
            solPara = struct(); %solPara = SOLVER();  
            % large array and function handles (that may implicilty requires large array as function handle parameters)
            excludedFieldsWithLargeData = {'x0', 'b', 'AFun', 'AtFun', 'A', 'AMatrix', 'xGT', 'diagATA', 'Vk'};
            excludedStr = 'Not saved: a too large array or a function handle involes other classes or duplicated data in mfm';            
%             if compare.nnz(sol.f, 1e-2) > 1e7
%                 excludedFieldsWithLargeData = [excludedFieldsWithLargeData, 'f'];
%             end
            p = properties(sol);
            % two level deep copy to handle structure of structure
            for i = 1:numel(p)
                nameStr = p{i};                
                if ~isstruct(sol.(nameStr))
                    % a none Structure Properties
                    if ~any(strcmp(nameStr, excludedFieldsWithLargeData))
                        solPara.(nameStr) = sol.(nameStr);
                    else
                        solPara.(nameStr) = excludedStr;
                    end                                        
                else
                    % a Stucture Property (including structure array)
                     solPara.(nameStr) = struct.copy(sol.(nameStr), [], excludedFieldsWithLargeData);
                end
            end
        end
        
        function analyzeShowResult(sol, mfm)
            % 3. show compuational result, commented on 2017.01.16_            
            if (mfm.verbose)
                hf = figure(12);
            else
                hf = figure('Visible','off');
            end
            if mfm.isSimulatedObject; subplot(121); end
            if (sol.solType == 1) && sol.isDebias % twist solver and debias                
                iterSparseSolver = sol.iterDebiasStart-1;
                myLegend={'Sparse-Solver','Debias'};
            else
                iterSparseSolver = sol.iter;
                myLegend = {'Sparse-Solver'};
            end
            semilogy(sol.cpuTimes(1:iterSparseSolver), sol.objFunVals(1:iterSparseSolver),'b','LineWidth',2); 
            if (sol.solType == 1) && sol.isDebias % twist solver and debias                
                hold on; semilogy(sol.cpuTimes(sol.iterDebiasStart:end), sol.objFunVals(sol.iterDebiasStart:end),'r','LineWidth',2); hold off
            end            
            title(sprintf('gain (relative) =%.1e, reg=%.1e',mfm.gain/mfm.gain0, sol.regWt));
            xlabel('CPU time (sec)'); ylabel('Objective function');  legend(myLegend); grid on;
            if mfm.isSimulatedObject
                subplot(122);
                psnrAll = 10*log10(max(mfm.fGT(:)).^2./ ((sol.reconRelativeRMSE).^2 * mean(mfm.fGT(:).^2)));
                plot(1:iterSparseSolver, psnrAll(1:iterSparseSolver),'b','LineWidth',2); 
                if (sol.solType == 1) && sol.isDebias % twist solver and debias                
                    hold on;plot(sol.iterDebiasStart:numel(psnrAll), psnrAll(sol.iterDebiasStart:end),'r','LineWidth',2); hold off;
                end
                title(sprintf('gain (relative) =%.1e, reg=%.1e',mfm.gain/mfm.gain0, sol.regWt));
                xlabel('Iteration #'); ylabel('PSNR_{Sparse}');  legend(myLegend); grid on;
            end
            set(gcf, 'outerposition', [500, 500, 1600, 1000], 'PaperPositionMode','auto');
            if mfm.verbose 
                print(hf, fullfile(sol.retPath, [sol.retPrefix '.png']), '-dpng'); 
            end            
        end 
        
        function display(sol)
            % overload the default display function, so that if we type sol in the command window, this function will be called
            struct.display(sol);
        end

    end
    
        
    methods(Static)
        function solAll = testMe()
            solAll = testMe();
        end                       
    end
    
end

% test different solvers
function solAll = testMe()
% Simulation of MFM 3D reconstruction to test different Solver
smallWorldsPath = fullfile('..', '..', '..',  'smallWorlds');
voxSz = [108, 108, 50]; %old is [98, 98, 50];   % default size (nm) for each voxel in 3D recovered image
psfFile = fullfile(smallWorldsPath, '20160809_Bacterial_Confocal_PSFs', 'PSF_Ave.tif');
psfBgFile = [];

simuTypeAll = {'twoPointsZ', 'twoPointsX', 'extended'}; % extended object or two points along z/x (y is simular to z)

%% Uncomment if test for several choices
%simuChoiceAll = 1; %[1, 2, 3]; % 'twoPointsZ', 'twoPointsX'
%for simuChoice = simuChoiceAll
%simuType = simuTypeAll{simuChoice};

%% simuChoice = 2; % 1 for twoPointsZ, 2 for twoPoints X
simuChoice  = 1;
simuType = simuTypeAll{simuChoice};
isSimuTwoPoints = any(simuChoice == [1,2]);

switch lower(simuType)
    case lower('twoPointsZ')
        fSz = [151, 151, 7];  % [150, 150, 21]; default 3D recovered image size, we may only use central part of psf
        gap = 2;ed
        fGT = single(shape.generateShape(simuType, fSz, gap));    
    case lower('twoPointsX')
        fSz = [151, 151, 7];  % [150, 150, 21]; default 3D recovered image size, we may only use central part of psf
        gap = 2;
        fGT = single(shape.generateShape(simuType, fSz, gap));         
    case lower('extended')
        fGT = load(fullfile(smallWorldsPath,  '20160809_Bacterial_Confocal_PSFs', 'Bacteria2_LongZ_confocal_Deconvolved.mat')); fGT=single(fGT.fConfocalInterpPadded);
        fSz = size(fGT);
    otherwise
        error('unkown simulation type %s', simuType);
end

fGT = fGT/sum(fGT(:));  % so that the total f should be 1
show3dPointsWithProjections(fGT, voxSz.*[1,1,10], [], 'markersize', 10, 'cMap', 'jet'); 

%% construct mfm0: the noiseless MFM system and use the forward model to construct a simulated noiseless g
mfm0 = MFM(psfFile, [], 'fGT', fGT, 'fSz', fSz, 'voxSz', voxSz, 'psfBgFile', psfBgFile, 'verbose', 0); %, 'gFrameNo', 1, 'verbose', 0)
gNoiseless = mfm0.AFun(mfm0.fGT); % Exact forward model, no psf noise, no sensored g noise
mfm0.setG(gNoiseless);  mfm0.fGT = mfm0.fGT/mfm0.gNorm; % mfm.setG(g) set mfm.g = g/gNorm; so remember to also normalize fGT by gNorm
%% Just assign name to mfm for convenience: uniform handling current noiseless and future noisy cases
mfm = mfm0;
self = mfm; normalizer = sqrt(sum(self.psf(:).^2)/numel(self.psf)*prod(self.fSz));  % we known regWt=1 is good when normalizer =1, and generally regWt=normalizer^2 is good.
%% Fixed Parameters
maxiterA = 6;
regWt = 0.1; %1.0;
stopCriterion = 'objectiveChange'; % choose from {'objectiveChange', 'xChange', 'nonzeroChange', 'differenceFromXGT', 'smallDerivative', 'dualityGap'}
tolA = 1e-8;
isDebias = 0;
isNonNegative = 1;

% Test different solvers
solTypeAll = {'FISTA'};     %solTypeAll = {'FISTA', 'TwIST', 'GPSR_BB', 'GPSR_BB_NonNegative', 'GPSR_Basic', 'TwIST_NonNegative'}; 
solAll = cell(1, numel(solTypeAll));
for n = 1:numel(solTypeAll)
    solType = solTypeAll{n};
    fprintf('Solve with %s:  %d of total %d solvers:\n', solType, n, numel(solTypeAll));
    %sol = SOLVER('solType', solType, 'regFun', 'L1', 'isNonNegative', isNonNegative, 'stopCriterion',stopCriterion, 'maxiterA', maxiterA, 'regWt', regWt*normalizer^2, 'tolA', tolA, 'verbose', 1, 'isDebias', isDebias, 'Lipschitz0', 5e3); % 40s for twist
    %sol.solve(mfm);
    sol = SOLVER(mfm, 'solType', solType, 'regFun', 'L1', 'isNonNegative', isNonNegative, 'stopCriterion',stopCriterion, 'maxiterA', maxiterA, 'regWt', regWt*normalizer^2, 'tolA', tolA, 'verbose', 1, 'isDebias', isDebias, 'Lipschitz0', 5e3); % 40s for twist
    sol.solve();
    solAll{n} = sol;
end


%% Simulation with Noise, todo in future
simulateWithNoise = false;
if simulateWithNoise
    %% Add noise to psf and g.  Different noise level is controled by total photon number received in sensor of one mfm image
    % 1. Various Photon/Noise Level for psf.
    %NOTE: if psf only measured once, the noise is killer, can't have good reconstruction
    camPsf = CAMERA('EMGain', 750); % 750 EMGain = 140 Gain = 140 pixel count / photon
    gaussianStd = camPsf.gaussianStdDU; % always around 37, independent of object, slightly dependendt on EMGain
    gain = single(camPsf.gain);  % 139.2
    % Real one measurement psf has parameters named with 0 : For brighest pixel in psf ; the Possion std for brightest pixel is around 2000 = gain*sqrt(maxPhoton), Gaussian Std
    maxPxlSensorPsf0 = mfm0.psfMax;
    maxPhotonPsf0 = maxPxlSensorPsf0 / gain; % 350 = 48645 / 139.2
    sumPhotonPsfZslice0 =  maxPhotonPsf0/max(mfm0.psf(:)) * (sum(mfm0.psf(:))/size(mfm0.psf,3)); % total photon per psf z slice = 1.93e5 = 26913736/139.2
    % Kowning sumPhotonPsfZslice0 is around 2e5
    sumPhotonPsfZsliceAll = [inf, 2e5*10, 2e5];  % 2e5 is similar to one measurement, 2e5*10 is similar to 10 measurements or 10 longer exposure
    maxPhotonPsfAll = sumPhotonPsfZsliceAll / (sum(mfm0.psf(:))/size(mfm0.psf,3)) * max(mfm0.psf(:));
    maxPxlSensorPsfAll = maxPhotonPsfAll * gain;
    NPsf = numel(sumPhotonPsfZsliceAll);
    
    % 2. Various Photon/Noise Level for g.  Assume g is captured with same 750 EMGain/same gain, (not consider overexposure)
    sumPhotonGAll = [inf, 2e5, 2e3]; %sumPhotonGAll = [inf];
    maxPhotonGAll = sumPhotonGAll / sum(mfm0.g(:)) * max(mfm0.g(:));
    maxPxlSensorGAll = maxPhotonGAll * gain;
    NG = numel(sumPhotonGAll);
    
    
    %% Solve
    % General parameters for both noiseless and noisy
    % regWt = 1;  % fixed regWt? no, try all the regWt
    regWtAll = 10.^(2:-1:-2);
    maxiterA = 100; % 20, 100
    tolA = 1e-8;
    if isSimuTwoPoints
        isDebias = 1;
    end
    solType = 'twist';
    
    
    solAll = cell(NPsf, NG);
    regWtBestAll = zeros(NPsf, NG);
    psnrBestAll = -inf*ones(NPsf, NG); % Intialize to the worest possible psnr
    for nPsf = 1:NPsf
        fprintf('-------------Start processing psf noise level: %dth out of %d----------------.\n',nPsf, NPsf);
        psfNoisy = mfm0.addNoisePsf('gaussianStd', gaussianStd, 'maxPxlSensor', maxPxlSensorPsfAll(nPsf), 'maxPhoton', maxPhotonPsfAll(nPsf), 'rngState', 'default');
        % since the psfNoisy is not changing, we pre-construct the mfm without g, and add g with different Photon/Noise Level
        mfm = MFM(psfNoisy, [], 'fGT', fGT, 'fSz', fSz, 'voxSz', voxSz, 'psfBgFile', psfBgFile, 'verbose', 0); %, 'gFrameNo', 1, 'verbose', 0)
        self = mfm; normalizer = sqrt(sum(self.psf(:).^2)/numel(self.psf)*prod(self.fSz));  % we known regWt=1 is good when normalizer =1, and generally regWt=normalizer^2 is good.
        for nG=1:NG
            fprintf('-------------Start processing g noise level: %dth out of %d----------------.\n',nG, NG);
            gNoisy = mfm0.addNoiseG('gaussianStd', gaussianStd, 'maxPxlSensor', maxPxlSensorGAll(nG), 'maxPhoton', maxPhotonGAll(nG), 'rngState', 'default');
            mfm.setG(gNoisy);  mfm.fGT = mfm.fGT/mfm.gNorm; % mfm.setG(g) set mfm.g = g/gNorm; so remember to also normalize fGT by gNorm
            for regWt = regWtAll
                sol = SOLVER('solType', solType, 'regFun', 'L1', 'isNonNegative', 1, 'maxiterA', maxiterA, 'regWt', regWt*normalizer^2, 'tolA', tolA, 'verbose', 1, 'isDebias', isDebias, 'Lipschitz0', 5e3); % 40s for twist
                sol.solve(mfm);
                if ~isempty(sol.fDebias) && ~any(isnan(sol.fDebias(:)))% sum(sol.f(:)) > eps(class(sol.f))*numel(sol.f)
                    difference(sol.f, fGT)
                    difference(sol.fDebias, fGT)
                    psnrCur =  difference(sol.fDebias, fGT);
                    if  psnrCur > psnrBestAll(nPsf, nG);
                        psnrBestAll(nPsf, nG) = psnrCur;
                        regWtBestAll(nPsf, nG) = regWt;
                        solAll{nPsf, nG} = sol;
                    end
                end
            end
            fprintf('-------------Finished processing g noise level: %dth out of %d. Found the best regWt = %5.2e, with psnr = %5.2e----------------.\n',nG, NG, regWtBestAll(nPsf, nG), psnrBestAll(nPsf, nG));
        end
    end
end
%end %for simuChoice = simuChoiceAll
end