function DFTMatrix = calcDFTMatrix(N1, N2, N3)
% caclulate Discrete Fourier Transform matrix for 1D vector of size N1 or 2D image of size N1*N2
%   Usage:
%       DFTMatrix = calcDFTMatrix(N1)
%       DFTMatrix = calcDFTMatrix(N1, N2)
%       DFTMatrix = calcDFTMatrix(N1, N2, N3)
%   Input:
%       N1: number of rows, or number of samples in y/row direction
%       N2: number of columns, or number of samples in x/column direction
%       N3: number of depths
%   Output:
%       DFTMatrix = calcDFTMatrix(N1) returns N1*N1 Fourier transform matrix for 1D signal
%       DFTMatrix = calcDFTMatrix(N1, N2) returns (N1*N2) * (N1*N2) Fourier transform matrix for 2D images
%       DFTMatrix = calcDFTMatrix(N1, N2) returns (N1*N2) * (N1*N2) Fourier transform matrix for 2D images
%
%   For 1d DFT: DFTMatrix*x yields the same result as fft(x) / sqrt(N1); however,  fft(X) is more efficient.
%   For 2d DFT: DFTMatrix*X(:) yields the same result as reshape(fft2(X), N1*N2,1) / sqrt(N1*N2);
%   Relationship between the 1d DFT and 2d DFT:
%       fft2 = fft(fft(X).').' computes the 1d DFT of each column of X, then of each row of the result.
%       fft2 = fft(fft(X.').') computes the 1d DFT of each row of X, then of each column of the result.
%       Let Dxy = calcDFTMatrix(N1, N2); Dx = calcDFTMatrix(N1); Dy = calDFTMatrix(N2), then
%       Dxy = kron(Dx, Dy);
%       reshape(Dxy*X(:), [N1,N2]) = (Dx*((Dy*X).')).' = Dy*X*(Dx.') = Dy*X*Dx
%
%   DFTMatrix property:
%       It's symmetric (equal to its transpose);
%       It's unitary (each column has unit norm and orthogonal),
%       as its inverse is symmetric and equal to its conjugate, thus equal to its conjugate transpose
%           DFTMatrix = DFTMatrix.';
%           inv(DFTMatrix) = conj(DFTMatrix) = conj(DFTMatrix).' = DFTMatrix'
%
%   refer to dftmtx, as for 1D input, we have dftmtx(N1)/sqrt(N1) = DFTMatrix
%  2017.04.08_extended from 1D/2D to 3D

if nargin == 0; testMe(); return; end % Unit Test

if nargin == 1
    % 1D Fourier Transform: DFTMatrix(p,q) = 1/sqrt(N1) * exp(-1j*2*pi *(p*q/N1)) for p = 0,..N1-1; m = q,..N1-1;
    DFTMatrix = 1/sqrt(N1) * exp(-1j*2*pi/N1 * bsxfun(@times, (0:N1-1)', 0:N1-1)); % based on definition,  O(N^2).
    %DFTMatrix = 1/sqrt(N1) * dftmtx(N1);  % based on FFT, same result as above, O(N^2*log(N)) but in practical faster due tothe c code, though require Signal Process Toolbox
elseif nargin ==2
    % 2D Fourier Transform: DFTMatrix(k,l, m1,n) = DFTMatrix1D(k,m)*DFTMatrix1D(l,n) = 1/sqrt(N1*N2) * exp(-1j*2*pi  *(k*m/N1 + l*n/N2))
    DFTMatrix = kron(calcDFTMatrix(N2), calcDFTMatrix(N1));  %
elseif nargin == 3
    DFTMatrix = kron(calcDFTMatrix(N3), kron(calcDFTMatrix(N2), calcDFTMatrix(N1)));  %
end
end

%% Unit Test
function testMe()
%% Test 2D
N1 = 5; N2 = 3;
fprintf('Test with N1=%d, N2=%d.\n', N1, N2);   
DFTMatrix = calcDFTMatrix(N1, N2);
fprintf('Computed matrix has size %d, %d.\n', size(DFTMatrix, 1), size(DFTMatrix, 2));
X = rand(N1, N2);
psnr_ = abs(difference(reshape(DFTMatrix*X(:), [N1, N2]), fft2(X)/sqrt(N1*N2)));  %  High PSNR
fprintf('PSNR between reshape(DFTMatrix*X(:), [N1, N2] and fft2(X)/sqrt(N1*N2) = %.3fdB.\n', psnr_);
psnr_ = abs(difference(reshape(conj(DFTMatrix)*X(:), [N1, N2]), ifft2(X)*sqrt(N1*N2)));  % High PSNR
fprintf('PSNR between reshape(conj(DFTMatrix)*X(:), [N1, N2] and ifft2(X)*sqrt(N1*N2) = %.3fdB.\n', psnr_);
%disp('DFTMatrix ='); disp(DFTMatrix);

%% Test 3D
N1 = 5; N2 = 3; N3 = 4;
fprintf('Test with N1=%d, N2=%d, N3=%d.\n', N1, N2, N3);
DFTMatrix = calcDFTMatrix(N1, N2, N3);
fprintf('Computed matrix has size %d, %d.\n', size(DFTMatrix, 1), size(DFTMatrix, 2));
X = rand(N1, N2, N3);
psnr_ = abs(difference(reshape(DFTMatrix*X(:), [N1, N2, N3]), fftn(X)/sqrt(N1*N2*N3)));  %  High PSNR
fprintf('PSNR between reshape(DFTMatrix*X(:), [N1, N2, N3] and fftn(X)/sqrt(N1*N2*N3) = %.3fdB.\n', psnr_);
psnr_ = abs(difference(reshape(conj(DFTMatrix)*X(:), [N1, N2, N3]), ifftn(X)*sqrt(N1*N2*N3)));  % High PSNR
fprintf('PSNR between reshape(conj(DFTMatrix)*X(:), [N1, N2, N3] and ifftn(X)*sqrt(N1*N2*N3) = %.3fdB.\n', psnr_);
end
