function [A, O, FBig, OzAll, F] = calcSensingMatrixViaPSF_CircConv(psf)
%Compute sensing matrix A from point spread function (psf), only works for small psf
% 2017.04.09_need to fix the constant term (though currently is correct due to coincidence of ).
% Input: 
%       psf: 3D array of size pSz = [Ny, Nx, Nz]
% Output: 
%       A = F'*O*FBig: sensing matrix of (Ny*Nx) * (Ny*Nx*Nz), where we assume fSz = pSz, and gSz = pSz(1:2)
%       O = [O1, ...O_Nz], 
%       OzAll{z} = Oz of size Ny*Nx * Ny*Nz, where Oz main diagonal is the 2D Fourier Transform of the psf(:,:,i)
%       F = DFT matrix of Ny*Nx * Ny*Nz
%       FBig = blkdiag(F, F, ..., F), block diagonal matrix of size Ny*Nx*Nz * Ny*Nx*Nz
% NOTE: fftshift vs ifftshift
% x = circshift(x, floor(size(x)/2)); %fftshift(x)
% x = circshift(x, ceil(size(x)/2)); %ifftshift(x), i.e., circshift(x, -floor(size(x)/2)) as sz = floor(sz/2) + ceil(sz/2)

[Ny, Nx, Nz] = size(psf);

A = zeros(Ny*Nx, Ny*Nx*Nz); 
O = zeros(size(A));
F = calcDFTMatrix(Ny, Nx); %i.e., F = kron(dftmtx(Nx), dftmtx(Ny))/sqrt(Nx*Ny); 
FBig = zeros(Ny*Nx*Nz, Ny*Nx*Nz);
OzAll = cell(1, Nz);
FzInv = conj(F);

for z = 1:Nz
    %Rz =  Fzinv * diag(reshape(fft2(psf(:,:,z)), [],1)) * F;
    % 1. ifftshift so that the 'origin' of f will change from [1,1] to origin = ceil((pSz+1)/2).
    %    NOTE ifftshift(psf(:,:,z)) = circshift(psf(:,:,z), -yxShift), where yxShift = ceil((fSz(1:2)-1)/2); % origin - [1,1]
    % 2. The imaginary part are 0, but matlab still treat it as complex, so we only keep the real part
    yxShift = ceil(([Ny, Nx]-1)/2); % derived from origin - [1,1], also same as floor(fSz(1:2)/2)    
    OzAll{z} = diag(reshape(fft2(circshift(psf(:,:,z), -yxShift)), [],1)); % fft2(x) is same as reshape(sqrt(Ny*Nx)*F*x, Ny, Nx)    
    %OzAll{z} = diag(reshape(fft2(ifftshift(psf(:,:,z))), [],1));
    Az =  real(FzInv * OzAll{z} * F); 
    A(:, (1:Ny*Nx)+Ny*Nx*(z-1)) = Az;  % A = [A1 A2 ... Apz]
    O(:, (1:Ny*Nx)+Ny*Nx*(z-1)) = OzAll{z};
    FBig((1:Ny*Nx)+Ny*Nx*(z-1), (1:Ny*Nx)+Ny*Nx*(z-1)) = F;
end

%FAll = repmat({F}, 1, Nz); FBig= blkdiag(FAll{:});%FBig = []; for nz = 1:Nz FBig = blkdiag(FBig, F); end %Another two ways to compute FBig

end

%% Unit Test
function testMe()
%%
psf2D =zeros(3,3); psf2D(2,2)=2; Ar = calcSensingMatrixViaPSF_CircConv(psf2D); svd(Ar), sum(psf2D(:))
psf2D = round(rand(3)*9); Ar = calcSensingMatrixViaPSF_CircConv(psf2D); svd(Ar), sum(psf2D(:))
psf = cat(3, round(rand(3)*9), round(rand(3)*9)); A = calcSensingMatrixViaPSF_CircConv(psf); svd(A), sum(psf(:))/sqrt(size(psf,3)), sqrt(sum(sum(sum(psf,1),2).^2))
%% 1. A = F'*O*FBig, where O = [O_1 O_2 ... O_Nz]; F is unitary matrix for 2D FFT,  FBig = 
rng('default');
psf = round(rand(4,5,7)*9); 
[Ny, Nx, Nz] = size(psf);
M = Ny*Nx;
N = Ny*Nx*Nz;

A = calcSensingMatrixViaPSF_CircConv(psf); 
% normalize A by its largest singular value which is 
sMax = svds(A, 1);
psf = psf/sMax;
[A, O, FBig, OzAll, F] = calcSensingMatrixViaPSF_CircConv(psf); 



fprintf('\n------------------------------Verify A = F''*O*FBig------------------------------\n');
difference(F'*O*FBig, A)

% verify with MFMForward and MFMBackward
rng('default'); f = rand(size(psf));
OTF = complex(zeros(size(psf), class(psf)));
for z = 1:Nz
    OTF(:,:,z) = fft2(psf(:,:,z));
end
% MFMForward
fprintf('\n------------------------------Verify A*f(:) = MFMForward(f, OTF, gSz)------------------------------\n');
gSz = [Ny, Nx]; %circular convolution
g = reshape(A*f(:), gSz);
g2 = MFMForward(f, OTF, gSz);
difference(g2, g);
% MFMBackward
fprintf('\n------------------------------Verify A.''*g(:) = MFMBackward(g, OTF, fSz)------------------------------\n');
ATg = reshape(A'*g(:), size(f));
ATg2 = MFMBackward(g, OTF, size(f));
difference(ATg2, ATg);

n=1; figure(n); set(n, 'units', 'normalized', 'outerposition', [0.1, 0.1, 0.8, 0.8]); clear n;
imagesc(abs(O)); axis image; colormap gray; colorbar;
title('magnitude of O');
%multAxes(@imagesc, {real(O), imag(O)}); multAxes(@colorbar); multAxes(@axis, 'image'); 

%% 2. Singular Value decomposition of A = U0*S0*V0' = UUnsorted*SUnsorted*VUnsorted' = U*S*V', where
% U0, S0, V0 are real matrices from Matlab svd() output
% UUnsorted = F, SUnsorted = diag(sqrt(diag(O*O'))), VUnsorted = FBig*O.'*diag(1./diag(S)) are complex matrices
% U, S, V are UUnsorted,SUnsorted,VUnsorted sorted from largest singular value to smallest singular values, they are complex matrices 

% A = U0*S0*V0'
[U0, S0, V0] = svd(A);
S0 = S0(:, 1:M); 
s0 = diag(S0);
V0 = V0(:, 1:M);

% A = UUnsorted*SUnsorted*VUnsorted'
UUnsorted = F;
sUnsorted = sqrt(diag(O*O')); 
SUnsorted = diag(sUnsorted); 
VUnsorted = FBig*O.'*diag(1./sUnsorted);

% A = U*S*V' after sort
[s, idx2] = sort(sUnsorted, 'descend');
U = UUnsorted(:, idx2);
S = diag(s);
V = VUnsorted(:, idx2);



% 2.1. Verify SVD:  A = U0*S0*V0' = UUnsorted*SUnsorted*VUnsorted' = U*S*V'
fprintf('\n------------------------------Display singular values of A------------------------------\n');
fprintf('s0=svd(A), sUnsorted=sqrt(diag(O*O''), s=sort(sqrt(diag(O*O'')), ''descend'')\n');
disp([svd(A), sqrt(diag(O*O')), sort(sqrt(diag(O*O')), 'descend')])
fprintf('sqrt(sum(sum(sum(psf,1),2).^2)) = %.4f\n', sqrt(sum(sum(sum(psf,1),2).^2)));
fprintf('sum(psf(:))/sqrt(size(psf,3)) = %.4f\n', sum(psf(:))/sqrt(size(psf,3)));
fprintf('\n------------------------------Verify Eigen Values S = s0------------------------------\n');
difference(S, S0);

fprintf('\n------------------------------Verify A = UUnsorted*SUnsorted*VUnsorted'', where UUnsorted = F, SUnsorted = diag(sqrt(diag(O*O''))); VUnsorted = FBig*O.''*diag(1./diag(SUnsorted))------------------------------\n');
difference(UUnsorted*SUnsorted*VUnsorted', A);
fprintf('\n------------------------------Verify VUnsorted''*VUnsorted = eye(M,M)------------------------------\n');
difference(VUnsorted'*VUnsorted, eye(M,M));
fprintf('\n------------------------------Verify UUnsorted''*UUnsorted = eye(M,M)------------------------------\n');
difference(UUnsorted'*UUnsorted, eye(M,M));

fprintf('\n------------------------------Verify A = U*S*V'', where U, S, V are UUnsorted,SUnsorted,VUnsorted sorted from largest singular value to smallest singular values------------------------------\n');
difference(U*S*V', A);
fprintf('\n------------------------------Verify V''*V = eye(M,M)------------------------------\n');
difference(V'*V, eye(M,M));
fprintf('\n------------------------------Verify U''*U = eye(M,M)------------------------------\n');
difference(U'*U, eye(M,M));


% 2.2 Verify EVD:  AAT = U0*Lambda*U0' = U*Lambda*U' = F*O*O''*F'', ATA = V0*Lambda*V0' = V*Lambda*V' = FBig''*O''*O*FBig
% Eigenvalues are diagonal elements of the diagonal matrix Lambda = S*S = O*O'
% 2.2. Verify  AAT = F*O*O'*F',  ATA = FBig'*O'*O*FBig = FBig*O.'*conj(O)*FBig';  
% Eigenvalues of AAT/ATA are just diag(O*O')' sorted, which are square of singular values of A that has already been verified
ATA = A.'*A;
AAT = A*A.';
lambda = s.^2;
Lambda = diag(lambda);

fprintf('\n------------------------------Verify AAT = U0*Lambda*U0''------------------------------\n');
difference(U0*Lambda*U0', AAT);
fprintf('\n------------------------------Verify AAT = U*Lambda*U''------------------------------\n');
difference(U*Lambda*U', AAT);
fprintf('\n------------------------------Verify AAT = F*O*O''*F''------------------------------\n');
difference(F*O*O'*F', AAT); %difference(F'*conj(O)*O.'*F, AAT);


fprintf('\n------------------------------Verify ATA = V0*Lambda*V0''------------------------------\n');
difference(V0*Lambda*V0', ATA);
fprintf('\n------------------------------Verify ATA = V*Lambda*V''------------------------------\n');
difference(V*Lambda*V', ATA);
fprintf('\n------------------------------Verify ATA = FBig''*O''*O*FBig------------------------------\n');
difference(FBig'*O'*O*FBig, ATA); %difference(FBig*O.'*conj(O)*FBig', ATA);


% 2.3.  How to relate complex eigenvectors in U to real eigenvectors in U0
        % U(:,3), U(:,4) are complex conjugate, 
        % U(:,3) = (U0(:,3)+i*U0(:,4))/sqrt(2)*exp(i*theta);
        % U(:,4) = (U0(:,3)-i*U0(:,4))/sqrt(2)*exp(-i*theta), 
        % where theta = -atan(U0(1,4)/U0(1,3)) = 0.2614 ( close to pi/12)
fprintf('\n------------------------------[U0(:,1:4) U(:,1:4)]------------------------------\n');
[U0(:,1:4) U(:,1:4)]
fprintf('\n-------------------------why following is wrong after normalization-----[U0(:,1:4) U(:,1:4)]------------------------------\n');
theta = -atan(U0(1,4)/U0(1,3)); 
w = exp(1i*theta);
f3 = (U0(:,3)+1i*U0(:,4))/sqrt(2)*w; difference(f3, U(:,3))
f4 = (U0(:,3)-1i*U0(:,4))/sqrt(2)*1/w; difference(f4, U(:,4))
v3 = (U(:,3)/w + U(:,4)*w)/sqrt(2); difference(v3, U0(:,3));
v4 = -1i*(U(:,3)/w - U(:,4)*w)/sqrt(2); difference(v4, U0(:,4));
[U0(:,3:4) f3 f4]
%% 3. Low rank approximation
% A = s1*u1*v1' + s2*u2*v2' + ..., where s1 = S(1,1) = s(1) , u1 = U(:,1), v1 = V(:,1);
% AAT = d1*u1*u1' + d2*u2*u2' + ...
% ATA = d1*v1*v1' + d2*v2*v2' + ..., where d1 = Lambda(1,1) = lambda(1) = s(1)^2
% Eckart-Young-Mirsky Throem says the closet rank k matrix to A is 
%     Ak = s1*u1*v1' + s2*u2*v2' + .... + sk*uk*vk'
%    ||A-B|| >= || A - Ak|| = s(k+1) for all matrices B of rank k


% approximate AAT with first few eigen value decompositions
k = 1;  %k=1; k=2; k=3:4; k=5;6; eigen vector 3:4, 5:6 etc correponding to same eigen value, so we can pickup either the complex ones or the real ones. adjecent equal  1st and 2nd equal, 3rd and going different, but k=
ATA_rankK = V(:,k)*Lambda(k, k)*V(:,k)';
ATA_rankK0 = V0(:,k)*Lambda(k, k)*V0(:,k)';


fprintf('\n------------------------------Compare ATA_rankK (maybe complex) and ATA_rankK0 (real)------------------------------\n');
difference(ATA_rankK, ATA_rankK0);
fprintf('\n------------------------------Compare abs(ATA_rankK) and ATA_rankK0------------------------------\n');
difference(abs(ATA_rankK), ATA_rankK0);
n=2; figure(n); set(n, 'units', 'normalized', 'outerposition', [0.1, 0.1, 0.8, 0.8]); clear n;
multAxes(@imagesc, {ATA, ATA_rankK0, abs(ATA_rankK)}); linkAxesXYZLimColorView();  multAxes(@colorbar); multAxes(@axis, 'image'); multAxes(@title, {'ATA', 'ATA_rankK0', 'abs(ATA_rankK)'});
fprintf('\n------------------------------Compare diag(diag(ATA)), ATA_rankK, ATA------------------------------\n');
fprintf('norm(diag(diag(ATA))-ATA) = %.2e, norm(ATA_rankK0-ATA) = %.2e, norm(ATA_rankK-ATA) = %.2e, lambda(k(end)+1)= %.2e \n', norm(diag(diag(ATA))-ATA), norm(ATA_rankK0-ATA), norm(ATA_rankK-ATA), lambda(k(end)+1));
fprintf('FrobeniusNorm(ATA_rankK0-ATA) = %.2e, FrobeniusNorm(ATA_rankK0-ATA) = %.2e, FrobeniusNorm(ATA_rankK-ATA) = %.2e, sqrt(sum(lambda(numel(k)+1:end).^2))= %.2e \n', norm(diag(diag(ATA))-ATA, 'fro'), norm(ATA_rankK0-ATA, 'fro'), norm(ATA_rankK-ATA, 'fro'), sqrt(sum(lambda(numel(k)+1:end).^2)));

%% Compare the approximate (preconditioner) and exact inverse of H = DInv + ATA; with 
fprintf('\n------------------------------Compare D = DInv^(-1),  HdiagInv = (DInv + diag(diag(ATA)))^(-1) , HkInv = (DInv + ATA_rankK)^(-1) with HInv = (DInv + ATA)^(-1)------------------------------\n');
rng('default');
d0Inv = rand(N, 1); % s./x, not  x./s
d0Inv = d0Inv/mean(d0Inv)*mean(diag(ATA));
% make d0Inv has large scale range from very small number to very large number
[~, idx] = sort(d0Inv, 'ascend');
third = floor(N/3);
scale0 = 1e-5; %1e-10; 1e-5;
curIdx = idx(1:third);       d0Inv(curIdx) = d0Inv(curIdx)* scale0;
curIdx = idx(third*2+1:end); d0Inv(curIdx) = d0Inv(curIdx)/scale0;

for scale = 1%10.^[-10 -5, 0, 5, 10]
    fprintf('----------------Scale = %.2e:\n', scale);
    dInv = d0Inv*scale;
    DInv = diag(dInv);
    D = diag(1./diag(DInv));  % just ignor ATA;
    HInv = inv(DInv + ATA);
    HdiagInv = diag(1./(diag(DInv) + diag(ATA)));   
    HkInv = inv(DInv + ATA_rankK);     
    %v = V(:,k); lambda = Lambda(k,k); Hk2 = D - D*lambda*v*v'*D/(1+lambda*v'*D*v);
    Hk0Inv = inv(DInv + ATA_rankK0);
    HInvNorm = norm(HInv);
    HInvNormF = norm(HInv, 'fro');
    fprintf('After normalizing by norm(HInv): norm(D-HInv) = %.2e, norm(HdiagInv-HInv) = %.2e, norm(Hk0Inv-HInv) = %.2e, norm(HkInv-HInv) = %.2e.\n', norm(D-HInv)/HInvNorm,norm(HdiagInv-HInv)/HInvNorm, norm(Hk0Inv-HInv)/HInvNorm, norm(HkInv-HInv)/HInvNorm);
    fprintf('After normalizing by FrobeniusNorm(HInv): FrobeniusNorm(D-HInv) = %.2e, FrobeniusNorm(HdiagInv-HInv) = %.2e, FrobeniusNorm(Hk0Inv-HInv) = %.2e, FrobeniusNorm(HkInv-HInv) = %.2e.\n', norm(D-HInv, 'fro')/HInvNormF, norm(HdiagInv-HInv, 'fro')/HInvNormF, norm(Hk0Inv-HInv, 'fro')/HInvNormF, norm(HkInv-HInv, 'fro')/HInvNormF);
end
n=3; figure(n); set(n, 'units', 'normalized', 'outerposition', [0.1, 0.1, 0.8, 0.8]); clear n;
multAxes(@imagesc, {HInv, D, HdiagInv, Hk0Inv, abs(HkInv)});  linkAxesXYZLimColorView();  multAxes(@colorbar); multAxes(@axis, 'image'); multAxes(@title, {'HInv', 'D', 'HdiagInv', 'Hk0Inv', 'abs(HkInv)'});
%%
rng('default'); x = rand(N, 1); x = x/norm(x);
fprintf('\n------------------------------Compare diag(diag(ATA))*x, ATA_rankK*x with ATA*x------------------------------\n');
normalizer = norm(ATA*x);
fprintf('Normalized by ATA*x: norm((diag(diag(ATA))-ATA)*x) = %.2e, norm((ATA_rankK0-ATA)*x) = %.2e, norm((ATA_rankK-ATA)*x) = %.2e.\n', norm((diag(diag(ATA))-ATA)*x)/normalizer, norm((ATA_rankK0-ATA)*x)/normalizer, norm((ATA_rankK-ATA)*x)/normalizer);
normalizer = norm(HInv*x);
fprintf('\n------------------------------Compare D*x,  HdiagInv*x, HkInv*x with HInv*x------------------------------\n');
fprintf('Normalized by HInv*x: norm((D-HInv)*x) = %.2e, norm((HdiagInv-HInv)*x) = %.2e, norm((Hk0Inv-HInv)*x) = %.2e, norm((HkInv-HInv)*x) = %.2e.\n', norm((D-HInv)*x)/normalizer, norm((HdiagInv-HInv)*x)/normalizer, norm((Hk0Inv-HInv)*x)/normalizer, norm((HkInv-HInv)*x)/normalizer);

%% condition number: 
DInv = diag(dInv);
DRoot = diag(1./sqrt(dInv)); %DInv = diag(dInv)*scale;
D = diag(1./dInv); 
% scale0 = 1e-10:   2.9e+19 vs  2.5e+12  vs 745
% scale0 = 1e-5:    2.2e+11 vs  2.5e+7   vs 745
cond(DInv+ATA)
cond(eye(N,N)+DRoot*ATA*DRoot)
cond(eye(M,M)+A*D*A') %even better conditioned
%% approximate (DInv+ATA)^(-1) = D - D*A'*(I+A*D*A')^(-1)*A*D
Hsmall = eye(M,M) + A*D*A'; 
HInv2 = D - D*A'*inv(Hsmall)*A*D;
difference(HInv, HInv2); % should be the same

%% first split DInv as DBig+DSmall, where DBig have big digagonal elements, and DSmall has very small diagonal elements (much smaller than diagonal of ATA)
% Dsmall+ATA is approximately ATA
% DInv+ATA = DBig+ATA
% approximate (DInv+ATA)^(-1) = D - D*A'*(I+A*D*A')^(-1)*A*D



%% k approximation of Hsmall
k = 1;
Ak = U(:,k)*S(k, k)*V(:,k)';
difference(A, Ak)
norm(Ak-A)/norm(A)
difference(A*D*A', Ak*D*Ak')
norm(Ak*D*Ak' - A*D*A')/norm(A*D*A')

Hsmallk = eye(M,M) + Ak*D*Ak';
norm(Hsmallk - Hsmall)/norm(Hsmall)

HsmallInv = inv(Hsmall);
HsmallInvk = inv(Hsmallk);
norm(HsmallInvk - HsmallInv)/norm(HsmallInv)

H2kInv = D - D*A'*inv(Hsmallk)*A*D;
difference(HInv, H2kInv)
norm(H2kInv-HInv)/HInvNorm

multAxes(@imagesc, {HInv, D, HdiagInv, Hk0Inv, abs(HkInv), abs(H2kInv)});  linkAxesXYZLimColorView();  multAxes(@colorbar); multAxes(@axis, 'image'); multAxes(@title, {'HInv', 'D', 'HdiagInv', 'Hk0Inv', 'abs(HkInv)',  'abs(H2kInv)'});



%%
Hsmallk = eye(M,M) + A*D


%%
tt = [svd(ATA)'; svd(D)'; svd(ATA+D)'];
tt
tt(:, 47:end)
%% nono! A and A2 has circular shift
%sVec = squeeze(sum(sum(psf,1),2)); sVec = sVec/norm(sVec, 2); norm(sVec), norm(A*sVec)


difference(AAT, U0(:,1)*lambda(1)*U0(:,1)')

%% small 2*2 cases
clc
%DInv = diag([1e5, 1e-5]);
DInv = diag([1e10, 1e-10]);
v1 = [1; 1]; 
v2 = [1; -1];
lambda1 = 100;
lambda2 = 1;
ATA = lambda1*v1*v1'+lambda2*v2*v2';
% 101    99
% 99   101
A = [ 10    10
     -1     1];
inv(DInv+ATA)
D = inv(DInv);
D - D*A'*inv(eye(2)+A*D*A')*A*D % large error due to bad conditioning
D*(eye(2)- A'*inv(eye(2)+A*D*A')*A*D) % large error due to bad conditioning
inv(DInv)
inv(ATA)
inv(DInv+lambda1*v1*v1')
D - D*lambda1*v1*inv(1+lambda1*v1'*D*v1)*v1'*D % strange, this is better, than D - D*A'*inv(eye(2)+A*D*A')*A*D
D*(eye(2)- (lambda1*v1*inv(1+lambda1*v1'*D*v1)*v1')*D) % 
inv(DInv+diag(diag(ATA)))
end