function [x, k, relRes] = cg(A, b, x0, tol, maxIter)
%CG Conjugate Gradient Method
% solve Ax = b with tol and maxIter
% It's tricky when x is 3d, b is 2d, and A is operator/function handle goes from 3d to 2d
% 
%   2017.05.15_Xiang Huang
%   2018.02.01_There is still no preconditioner here, so be careful just mannually precondition A matrix and make transfers
%   before use it, or just use the Matlab function pcg
%  similar speed as the pcg function provided by matlab.
%  TODO: change the order of arguments to perform similarly as matlab function [x,flag,relres,iter,resvec] = pcg(A,b,tol,maxit,M1,M2,x0



if nargin == 0; testMe(); return; end % Unit Test
%% parse parameters
N = numel(b);
if ~exist('x0', 'var') || isempty(x0)
    x0 = zeros(N,1);
end
if ~exist('tol', 'var') || isempty(tol)
    tol = 1e-8;
end
tol_rTr = tol^2*(b'*b);

if ~exist('maxIter', 'var') || isempty(maxIter)
    maxIter = 10;
end

% Creat function handles AFun and AtFun, will only use the function handles form later
if isnumeric(A)
    % A is a matrix (numeric array)
    %AtFun = @(x) A'*x;
    AFun  = @(x) A*x;
elseif isa(A, 'function_handle')
    % A is a function handle
   % assert(isa(AtFun,'function_handle'), 'The function handle for transpose of A is missing');
    AFun = A;
else
    error('A need to be either a numeric array or a function handle');
end

%% Initialize x, r, d
%x = zeros(1,N);
% if exist x0, then do following, esle d0 = 0;
x = x0;
r = b - AFun(x);
d = r; 
rTr_km = r' * r;
%% iteration from k-1 to k
% Notations:   rTr_k = r(k)'*rk; 
%              rTr_km = r(k-1)'*r(k-1); where m means minus 1

for k = 1:min(N, maxIter)
    % precompute A*d (only need one of this most expensive operation per iteration)
    Ad = AFun(d);
    
    dTAd = d'*Ad;
    if dTAd < tol_rTr
        warning('divide by number close to 0 %11.5e, cg may not be stable. but still keep iterate', dTAd);
    end
    
    % update x and r after caclulate step size alpha
    alpha = rTr_km / dTAd;
    x = x + alpha * d;    

    % Without this correction step: cg performs same as matlab's pcg, 
    % With this step, cg is better than pcg when A is bad conditioned, total iterations cg=216, pcg=715 in a interpoint method example
    if mod(k,10) ~=  0 % do the correction in 10 or 20 steps doesn't matter too much
        r = r - alpha * Ad;
    else
        r = b - AFun(x);  % eliminate accumlated floating point operation error
    end
    
    % check new error ||r||, break if it is small enough
    rTr_k = r' * r;
    if rTr_k < tol_rTr
        break
    end
    
    % update search direction for next loop. update rTr.
    d = r + (rTr_k / rTr_km) * d;
    rTr_km = rTr_k;    
end

relRes =  sqrt(rTr_k)/norm(b); %returns the relative residual NORM(B-A*X)/NORM(B)

end

%% Unit Test
function testMe()
% pinv: The computation is based on SVD(A) and any singular values less than a tolerance are treated as zero. Default tol:  MAX(SIZE(A)) * NORM(A) * EPS(class(A))
% cg, pcg, minres, A\b all have problems when the singular value are close to zero
%% Problem 1: 
N = 4; A = diag(1:N);  b = ones(N,1); % A(1,1)=-1; % one negative singular value is ok
x  = cg(A,b);
x  = pcg(A,b);
fprintf('A positive definite: full rank ok conditioned.\n');
fprintf('x, A\\b, pinv(A)*b (pinv(A)*b is always correct)=:\n');
disp([x, A\b, pinv(A)*b])

%% Problem 2: 
N = 4; A = zeros(N,N); A(1,1)=1; b = ones(N,1);
x = cg(A,b);
fprintf('A positive semidefinite: rank deficient.\n');
fprintf('x, A\\b, pinv(A)*b (pinv(A)*b is always correct)=:\n');
disp([x, A\b, pinv(A)*b])

%% Problem 3: positive definite: full rank bad conditioned
N = 4; A = diag(eps(1)*ones(1,N)); A(1,1)=1; b = ones(N,1);
x = cg(A,b)
x = pcg(A,b)
x = minres(A,b)
fprintf('x, A\\b, pinv(A)*b (pinv(A)*b is always correct)=:\n');
fprintf('A positive definite: full rank bad conditioned.\n');
disp([x, A\b, pinv(A)*b])

%% Problem 4: non-negative constraint
N = 4; A = diag(eps(1)*ones(1,N)); A(1,1)=1; xGT = [1,1,0,0]'; b = A*xGT; % A(1,1)=-1; % one negative singular value is ok
x = cg(A,b)

%% solve 0.5*||b-Ax||^2 + regWt* 1'* x, s.t. x>=0
% cg to solve A'Ax = A'b - regWt*ones(N,1)
rng(0,'twister'); M=3; N= 5; 
A = rand(M,N); regWt = 0.01; xGT = zeros(N,1); xGT(1) = 1;  b = A*xGT; 
pinv(A)*b
Acg = A'*A;  bcg = A'*b - regWt*ones(N,1);
x = cg(Acg, bcg)
%x = solveCG(b, A, regWt, varargin)

end