function [x, out, objs, cpuTimes, SNRs, regFunVals, itersCG] = mfipm_NonNeg_XH(n, A, b, tau, opts, xGT)
%function [x, out] = mfipm_NonNeg_XH(n, A, b, tau, opts)
%mfipm_NonNeg_XH: Matrix-free Interior Point solver.
%2017.09.10_XH modified from MFIPM to restrict to x>=0
%
%  The Matrix-free IPM solver for one dimensional Compressed Sensing
%  problems proposed in [1].
%
%    [1] Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich 
%        "Matrix-Free Interior Point Method for CS problems", 
%        ERGO Technical Report, 2012.
%
%  The Matrix-free IPM solves the l1-regularised program:
%
%    min tau*||x||_1 + 0.5*||Ax - b||^2,
%
%  using an infeasible primal-dual interior point algorithm, for details
%  please see at [1].
%
%  Usage
%  -----
%
%    [x out] = mfipm(n, A, b, tau, opts) 
%
%  Input (required)
%  ----------------
%
%    n:   Number of columns of the constraint matrix A.
%    A:   The constraint matrix.
%    b:   The right hand side of the constraints Ax=b.
%    tau: The regularisation parameter.
%
%  Input (optional)
%  ----------------
%  
%    The opts argument is a structure and it is optional. Here is a 
%    short explanation of the available options for opts:
%
%    opts.tol:       Optimality tolerance of the Matrix-free IPM.
%                    Default value: 1.0e-8.
%    opts.maxiters:  Maximum iterations of the Matrix-free IPM.
%                    Default value: 100.
%    opts.tolpcg:    Optimality tolerance for the PCG method.
%                    Default value: 1.0e-4.
%    opts.mxiterpcg: Maximum iterations of the PCG method.
%                    Default value: 200.
%    opts.maxstepsz: Maximum stepsize for primal and dual directions.
%                    Default value 0.999.
%    opts.cnt1:      Centering parameter when predictor directions are 
%                    performed. Set to 0 for pure predictor directions.
%                    Default value 0.1.
%    opts.cnt2:      Centering parameter when predictor directions are 
%                    performed and one of the primal and dual stepsizes of 
%                    the previous iteration were less than opts.ap for last
%                    iteration. 
%                    Set to 0 for pure predictor directions.
%                    Default value 0.5.
%    opts.cnt3:      Centering parameter when corrector directions are 
%                    performed. Set to 1 for perfect centering to the
%                    central path.
%                    Default value 0.8.
%    opts.ap:        If primal or dual stepsize <= ap then use cnt2 as as
%                    centering parameter, otherwise use cnt1.
%                    Default value 0.5.
%    opts.ac:        If primal or dual stepsize after calculating the
%                    predictor direction are <= ac then perform a corrector
%                    update.
%                    Default value 0.1.
%    opts.linsol:    0 to use the Conjugate Gradients method as a linear 
%                    system solver, 1 to use a direct solver.
%                    Default value: 0.
%    opts.verbose:   0 for no printing info, 1 for basic printing info
%                    and 2 for detailed printing info.
%                    Default value: 1.
%
%    ***************************************************************************************
%    If the measurements are noiseless please use opts.tol >= 1.e-8. In case that the
%    measurements are contaminated with noise then please use opts.tol >= 1.0e-6. Regarding
%    the accuracy of the PCG method opts.tolpcg=1.0e-1 to 1.0e-4 should be enough.
%    ***************************************************************************************
%
%  Output
%  ------
%
%    x:               The reconstructed sparse representation.
%    out.totcputime:  Total CPU time required by the Matrix-free IPM.
%    out.iters:       Number of Matrix-free IPM iterations.
%    out.cpusteps:    CPU time used for the calculation of the stepsizes.
%    out.totpcgiters: Total number of PCG iterations.
%    out.nMat:        Number of matrix-vector products.
%    out.predictors:  Number of predictor directions.
%    out.correctors:  Number of corrector directions.
%    out.linsystems:  Total number of linear systems solved. It is equal
%                     to out.predictors + out.correctors.
%    out.status:      1 if the problem was solved successfully, 0 if 
%                     maximum number of iterations reached.
%    out.message:     Termination status of the Matrix-free IPM
%                     solver. One of the following two messages is returned:
%                     i)  'Converged'
%                     ii) 'Maximum iterations reached'
%
%
%  Note
%  ----
%
%    If A is an operator, then it should be passed to the Matrix-free IPM 
%    as an A_operator type. To use A_operator variables please add to your 
%    MATLAB (if it is not already added) the subdirectory:
%    
%      "main mfipm directory"/operators
% 
%    Example: To create an A_operator object, two functions or function 
%    handles for computing A*x and A'*x, respectively, must be given. 
%    Suppose they are AA and AT,
%          AA: a function handle such that AA(x) = A*x,
%          AT: a function handle such that AT(x) = A'*x.
%    Then A can be created as an A_operator by
%          A = A_operator(@(x) AA(x), @(x) AT(x));
%
%    An example for A being an implicit matrix that performs a discrete 
%    cosine transform (DCT) and returns the subset of the DCT result 
%    corresponding to omega is provided below
%
%     function y=pdct(x,picks); y=dct(x); y=y(picks); end
%     function y=pidct(x,n,picks); y=zeros(n,1); y(picks)=x; y=idct(y); end
%     A = A_operator(@(x) pdct(x,omega), @(x) pidct(x,n,omega)); 
%
%    **This example has been copied from the FPC_AS.m in the 
%      FPC_AS_v1.21 software package.
%      url: http://www.caam.rice.edu/~optimization/L1/FPC_AS/ 
%
% Copyright (c) 2012.  Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich.

% Check number of arguments.
if nargin < 4
    error('At least four arguments are required.');
end

% Check mandatory arguments.
if ~isa(A, 'A_operator') && ~(sum(size(A) > 1) == 2)
    error('The argument "A" must be a matrix or of type "A_operator".');
end
sizes_b = size(b);
if sizes_b(1) > 1 && sizes_b(2) > 2
    error('The argument "b" should be a vector.');
end
if sizes_b(2) > 1
    b = b';
end
if length(b) > n
    error('"m" should be less than "n".');
end

% Check optinal arguments.
opts = chkOptargs(opts);

% XH: added intermediate result, Initalize Output Arrays: since we can't store objFunVals(0), we use objFunVals(k+1) to represent steps k = 0, 1, ..., maxIterA
tic;  % start the clock
%N  = numel(x0); M  = numel(b);
regWt = tau;
bNorm  = sqrt(b(:)'*b(:));
regWt_ATb = regWt - A'*b;  % objFunGrad = grad(objFun(x)) = A'Ax + regWt_ATb; NOTE: A'*b won't appear along in any where, always come with regWt - A'*b
% Objective function and dual function values
objFun = @(x, r) 0.5*(r'*r) + regWt*sum(abs(x));  % 2018.02.05_added abs(x) for future to extend from non-negative x to real/complex x.; % h(x),,  overide the more general objFun = @(x,resid,regWt,regFun)0.5*(resid'*resid)+regWt*regFun(x)
dualFun = @(Ax) - 0.5*(Ax'*Ax) + 0.5*bNorm^2;  % if s is dual feasible, then x'*s = objFun(x,r) - dualFun(Ax);
maxIterA = opts.maxiters;
sampledHist = [];
objs = zeros(1, maxIterA+1);
regFunVals  = zeros(1, maxIterA+1); % regulizer function values, such as abs(sum(x(:)) for L1-regulizer
duals = zeros(1, maxIterA+1); % dual objective values
ts = zeros(2, maxIterA+1); % history of sigma_a; sigma_c parameter
itersCG = zeros(1, maxIterA+1); % history of # of iterations in each CG/newton step
relResCG = zeros(1, maxIterA+1); % relative error of pcg solver
cpuTimes = zeros(1, maxIterA+1);
SNRs = zeros(1, maxIterA+1);


% Counters.
out.predictors  = 0;
out.correctors  = 0;
out.totpcgiters = 0;
out.linsystems  = 0;
out.nMat        = 0;
out.iters       = 0;
out.cpusteps    = 0;
out.totcputime  = 0;

% Initialize sizes.
% n2 = 2*n;         % 2017.09.10_XH 
n2 = n;             % 2017.09.10_XH: & todo: just replace all n2 with n
m  = length(b);

% CPU time.
tstart = cputime;

% Calculate linear part coefficients.
ATb      = A'*b;
%c        = tau + [-ATb;ATb];
c        = tau - ATb;    % 2017.09.10_XH
out.nMat = out.nMat + 1;


% Calculate initial point here: Gondzio's Intialization of x & s!
reg  = 1.0e-2*ones(n2,1);
% XH: redefine HMat and PMat with non-negative constraint
isUseNonNegConstraint = true;
if isUseNonNegConstraint
    HMat = @(A, x, T) A'*(A*x) + T.*x; % T is dInv = s./x; 2017.09.10_XH redefined HMat for n*n matrix instead of 2n*2n matrix Hmat, this is to solve (Dinv+A'*A)*x = residual
    Pmat = @(T, m, n, y)  y ./ (T+m/n); % 2017.09.10_XH_redefined Pmat for n*n matrix instead of 2n*2n matrix Pmat
    % Otherchoice 1: diagonal precondioner not used
    %ATAdiag = reshape(sum(A.*A, 1), [], 1); % only good when A is matrix
    %Pmat = @(T, m, n, y)  y ./ (T+ATAdiag); % 2017.09.10_XH_redefined Pmat for n*n matrix instead of 2n*2n matrix Pmat
    % Otherchoice 2: 
    %Pmat = @(T, m, n, y)  y; % 2017.09.10_XH_redefined Pmat for n*n matrix instead of 2n*2n matrix Pmat
    
end
H    = @(q) HMat(A, q, reg);   
Prec = @(p) Pmat(reg, m, n, p);
% 
[z,~,~,itersPcg] = pcg(H,-c,1.0e-2,200,Prec);
out.nMat = out.nMat + 2*itersPcg;
    
z = abs(z) + norm(b);
s = norm(b)*ones(n2,1);

% Initialize the rest of the variables here.
% ATAblkmat = A'*(A*(z(1:n)-z(n+1:end))); % 2017.09.10_XH
Az = A*z;
ATAz = A'*(Az);
out.nMat  = out.nMat + 2;

mu        = z'*s/n2;
dual_e    = s - c - ATAz; % XH's r_d where c = regWt-ATb
normc     = norm(c);

if opts.linsol
   F   = A'*A;
   %FFt = [F,-F;-F,F]; % 2017.09.10_XH
   FFt = F;
end


% XH
k = 0;
cpuTimes(k+1) = toc;
regFunVals(k+1)  = regWt*sum(abs(z));  % 2018.02.05_added abs(x) for future to extend from non-negative x to real/complex x.; %; regFun(x);
objs(k+1) = objFun(z, b-Az);
if ~isempty(xGT)
    [~, SNRs(k+1)] = psnr(z, xGT);
end



% Print info.
if opts.verbose == 2
   fprintf(' %5s | %5s |  %5s   | %8s | %8s | %9s | %10s |%10s \n', ' Info ', 'Iters',...
        'rDGAP', 'rDUALinf','Iters PCG', 'RelRes PCG', 'Primal Step', 'Dual Step');
   fprintf('  -------------------------------------------------------------------------------------- \n');
end

while 1
    % XH
    k = out.iters + 1;
    
    % Theta^{-1} diagonal matrix
    T = s./z;
        
    % Compute affine scaling direction.
    if out.iters ~= 0 && (alphaP < 0.5 || alphaD < 0.5)
        sigma_a = opts.cnt2;
    else
        sigma_a = opts.cnt1;
    end
    
    barrier_a = sigma_a*mu;
    
    f2 = barrier_a - z.*s; % (sigma*mu*e - XSe) = r_c in XH
    
    invZf2 = f2./z; % r_c./x  in XH
    if ~opts.linsol
        H    = @(q) HMat(A, q, T);
        Prec = @(p) Pmat(T, m, n, p);
        % dual_e =  s - c - ATAz = r_d of XH; invZf2 = f2./z = r_c./x in XH
        [dza,~,rrPcg,itersPcg] = pcg(H,dual_e+invZf2,opts.tolpcg,opts.mxiterpcg,Prec); 
        out.nMat = out.nMat + 2*itersPcg;
    else
        H = FFt + diag(T);
        dza = H\(dual_e+invZf2);
    end
    dsa = invZf2 - T.*dza;
    
    % Choose stepsize for the affine scaling direction.
    tstepsstart = cputime; 
    
    alphaP = calcstepsize(z,dza,opts.maxstepsz);
    alphaD = calcstepsize(s,dsa,opts.maxstepsz);
    
    tstepsend = cputime;
    out.cpusteps = out.cpusteps + (tstepsend - tstepsstart);
    
    z_new = z + alphaP*dza;
    s_new = s + alphaD*dsa;
    
    % Print info.
    if opts.verbose == 2 && ~opts.linsol
        fprintf('  %5s | %3d   | %8s | %8s | %5d     |  %1.2e  |   %1.2e  |  %1.2e \n',...
        'Pred.', out.iters, '', '', itersPcg, rrPcg, alphaP, alphaD);
    elseif opts.verbose == 2 && opts.linsol
        fprintf('  %5s | %3d   | %8s | %8s |   %1.2e  |  %1.2e \n',...
        'Pred.', out.iters, '', '', alphaP, alphaD);      
    end
    % XH added 2018.02.07
    mu=z_new'*s_new/(n2); fprintf('Predictor: sigma = %.1e, tolCG=%.1e, xTs = %.8e,  min(x.*s)/(xTs/N) = %.3e,  norm(r_d) = %.8e.\n',sigma_a, opts.tolpcg, mu*n2, min(z_new.*s_new)/mu, norm(dual_e));
        
    out.predictors  = out.predictors + 1;
    out.totpcgiters = out.totpcgiters + itersPcg;
    if k>22
        1;
    end
    % XH
    itersCG(k+1) = itersCG(k+1)+ itersPcg;
    
    
    % Compute centering direction, if needed.
    if alphaP < 0.1 || alphaD < 0.1
        
       % Compute centering parameter.
       % ATAblkmat = A'*(A*(z(1:n)-z(n+1:end))); % 2017.09.10_XH
       ATAz = A'*(A*z);
       out.nMat    = out.nMat + 2;
       dual_e_c  = s_new - c - ATAz;
       sigma_c   = opts.cnt3;
       mu_c      = z_new'*s_new/(n2);
       barrier_c = sigma_c*mu_c;
       

       f2_c = barrier_c - z_new.*s_new;
    
       invZf2_c = f2_c./z_new;
       if ~opts.linsol 
           [dzc,~,rrPcg,itersPcg] = pcg(H,dual_e_c+invZf2_c,opts.tolpcg,...
               opts.mxiterpcg,Prec);
           out.nMat = out.nMat + 2*itersPcg;
       else
           dzc = H\(dual_e_c+invZf2_c);
       end
       dsc = invZf2_c - T.*dzc;
       
       % Compute total direction.
       dz = dza + dzc;
       ds = dsa + dsc;
    
       % Choose stepsize for the final direction.
       tstepsstart = cputime;
       
       alphaP = calcstepsize(z,dz,opts.maxstepsz);
       alphaD = calcstepsize(s,ds,opts.maxstepsz);
       
       tstepsend    = cputime;
       out.cpusteps = out.cpusteps + (tstepsend - tstepsstart);
    
       z_new = z + alphaP*dz;
       s_new = s + alphaD*ds;
       
       % Print info.
       if opts.verbose == 2 && ~opts.linsol
           fprintf('  %5s | %3d   | %8s | %8s | %5d     |  %1.2e  |   %1.2e  |  %1.2e \n',...
           'Corr.', out.iters, '', '', itersPcg, rrPcg, alphaP, alphaD);
       elseif opts.verbose == 2 && opts.linsol
           fprintf('  %5s | %3d   | %8s | %8s |   %1.2e  |  %1.2e \n',...
           'Corr.', out.iters, '', '', itersPcg, rrPcg, alphaP, alphaD);               
       end
       % XH added 2018.02.07
       mu=z_new'*s_new/(n2); fprintf('Corrector: sigma = %.1e, tolCG=%.1e, xTs = %.8e,  min(x.*s)/(xTs/N) = %.3e,  norm(r_d) = %.8e.\n',sigma_c, opts.tolpcg, mu*n2, min(z_new.*s_new)/mu, norm(dual_e));
       out.correctors  = out.correctors + 1;
       out.totpcgiters = out.totpcgiters + itersPcg;
       % XH
       itersCG(k+1) = itersCG(k+1)+ itersPcg;
    end
    
    z = z_new;
    s = s_new;

    % Termination criteria.
    % ATAblkmat = A'*(A*(z(1:n)-z(n+1:end)));  %2017.09.10_XH
    Az = A*z;
    ATAz = A'*(Az);
    out.nMat  = out.nMat + 2;

    mu        = z'*s/n2;
    rel_dgap  = mu/(1.0 + abs(c'*z + 0.5*z'*ATAz + 0.5*norm(b)^2)); % 2018.02.07_This is not true, as norm(dual_e) = norm(r_d) is not getting close to 0. 2017.09.10_XH, may just change to 1+abs(0.5*norm(A*x-b)^2+tau*sum(x))

    dual_e    = s - c - ATAz;
    dual_inf  = norm(dual_e)/normc;
    
    %XH
    cpuTimes(k+1) = toc;
    if ~isempty(xGT);  [~, SNRs(k+1)] = psnr(z, xGT); end
    objs(k+1) = objFun(z, b-Az);
    regFunVals(k+1)  = regWt*sum(abs(z));  % 2018.02.05_added abs(x) for future to extend from non-negative x to real/complex x.; %; regFun(x);

    
    % Print info.
    if opts.verbose == 2
        fprintf('  %5s | %3d   | %1.2e | %1.2e | \n',...
        'final', out.iters, rel_dgap, dual_inf);
    end       

    % x = z(1:n) - z(n+1:end); % 2017.09.10_XH
    x = z;    

    if 0% mu/abs(c'*z + 0.5*z'*ATAz + 0.5*norm(b)^2) < opts.tol % XH mu/objs(k+1) % XH debug to let it run desired iterations
    %if (rel_dgap < opts.tol) %Gondzio used different, assume feasibility, but could turn to infesiable 
        out.status  = 1;
        out.message = 'Converged'; 
        break;
    elseif (out.iters+1 >= opts.maxiters) % (out.iters > opts.maxiters) %XH modified to match our definition of iteration number
        out.status  = 0;
        out.message = 'Maximum iterations reached'; 
        break;
    end

    out.iters = out.iters + 1;
end

out.linsystems = out.linsystems + out.correctors + out.predictors;
tend = cputime;
out.totcputime = tend - tstart;

if opts.verbose >= 1
   fprintf('\nTotal CPU time: %f \n', out.totcputime);
   fprintf('CPU time for stepsizes: %f \n', out.cpusteps);
   fprintf('Iterations: %3d \n', out.iters);
   fprintf('Predictors: %3d \n', out.predictors);
   fprintf('Correctors: %3d \n', out.correctors);
   fprintf('Total linear systems solved: %3d \n', out.linsystems);
   fprintf('Total PCG iterations: %4d \n', out.totpcgiters);
   fprintf('Total nMat: %5d \n', out.nMat);
end

% XH
k = out.iters + 1;
% crop output results array according to the actual iteration k, and print output result
objs = objs(1:(k+1));
regFunVals = regFunVals(1:(k+1));
duals = duals(1:(k+1));
ts = ts(:, 1:(k+1));
itersCG = itersCG(1:(k+1));
relResCG = relResCG(1:(k+1));
cpuTimes = cpuTimes(1:(k+1));
SNRs = SNRs(1:(k+1)); if isempty(xGT); SNRs = []; end
end


% function x = PmatNoneNeg(T, m, n, y)
% %PmatNoneNeg Implements the preconditioner 
% %2017.09.10_chaned from Pmat to Pmat
% % Copyright (c) 2012.  Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich.
% x = y ./ (T+m/n);
% end

