function [x, flag, relRes, realIter, relResEachIter, r] = pcgXH(A, b, tol, maxIter, M1_or_M1InvFun, M2_or_M2InvFun, x0)
%pcgXH: Preconditioned Conjugate Gradient Method that solves Ax = b, with similar usage to 
% [x, flag, relRes, realIter, relResEachIter, r] = pcgXH(A, b, tol, maxIter, []. []. x0)
%   It solves the system A*x = b with intial guess x0. 
%   The algorithm runs iteratively until relative residual relRes =  norm(b-Ax) / norm(b) < tol (flag = 0), 
%   or the iteration# realIter = maxIter (flag = 1).
%   The output relResEachIter contains the relative residual at intial x0, and after each iteration
%   The output r = b - A*x.
%
%   [x, ...] = = pcgXH(A, b, tol, maxIter, M, [], x0)
%   uses matrix M to specify the precontioner. From it, we construct MInvFun(x) = M\x which is the only information needed from the precondioner.
%   Generally the predontioner M is an approximation of A. Hence MInvFun(x) is an approximation of A^{-1}*x
%
%   [x, ...]  = pcgXH(A, b, tol, maxIter, M1, M2, x0)
%   uses matrix M1, M2 to specify the precontioner M=M1*M2. We construct MInvFun(x) = M2\(M1\x) which is the only information needed from the precondioner.
%
%   [x, ...]  = pcgXH(A, b, tol, maxIter, MInvFun, [], x0)
%   uses function handle MInvFun to specify the inverse operation of preconditioner: MInvFun(x) = M^{-1}*x, 
%
%   [x, ...]  = pcgXH(A, b, tol, maxIter, M1InvFun, M2InvFun, x0)
%   rarely used form, uses two function hanldes M1InvFun, M2InvFun that computes M1^{-1}*x and M2^{-1}*x,
%   so that the combined form MInvFun = @(x) M2InvFun(M1InvFun(x)) computes M2^{-1}*(M1^{-1}*x)
%
% Inputs:
%   A:      either N*N matrix or a function handle so that A(x) = A*x.
%   b:      N*1 vector
%   tol:    tolerance of result accuracy, norm(A*x-b)/norm(b) <= tol
%   maxIter: Maximum number of iterations in pcgXH. If not specified, maxIter use default min(N,10)
%   x0:     The initial guess. If not specified, use default zeros(N,1)
%   M1_or_M1InvFun,  M2_or_M2InvFun:  combines to form of the positive definite preconditioner M which is the approximation of A
%           We only need MInvFun() which is the function handle: MInvFun(x) =  M^{-1}*x
%           M = M1*M2, hence M^{-1} = M2^{-1}*M1^{-1}, hence MInvFun(x) = M2InvFun(M1InvFun(x)).
%           M1_or_M1InvFun: If it is a matrix, then it is M1; otherwise it is a function handle M1InvFun wehre M1InvFun(x) = M1^{-1}*x
%                           If it is [] or not specified, then M1 = eye(M,M), M1InvFun(x) = x.
%                           This notation and use same positional variable for M1 or M1InvFun() is similar to pcg() in matlab.
%           M2_or_M2InvFun: Similar.
% Compare with other notations.
% M = P^T*P is what we used in the paper AlgorithmReconstruct3D, we only provide MInvFun() the function handle MInvFun(x) = M\x;
% M = C^T*C is used in ``Numerical Optimization'', Jorge Norcedal and Stephen Wright, 2nd Edition, Page 113, where C = P
% M = M1*M2 is used matlab function pcg(), where  M1 = P^T; M2 = P. NOTE: M=M1*M2 also used in http://netlib.org/linalg/html_templates/node54.html
% pcgXH(A, b, tol, maxIter, x0, MInvFun) is similar to pcg(H, c, tolCG, maxIterCG, MInvFun, [], x0)
%
% Outputs:
%   x:          The computed approximate solution of A*x = b
%   flag:       convergence flag (not implemented as 2018.02.04)
%               0 pcgXH converged to the desired tolerance TOL within MAXIT iterations
%               1 pcgXH iterated MAXIT times but did not converge.
%               2 preconditioner M was ill-conditioned. 
%               3 pcgXH stagnated (two consecutive iterates were the same).
%               4 one of the scalar quantities calculated during pcgXH became too small or too large to continue computing.
%   relRes:     relative residual norm(b-Ax) / norm(b)
%   realIter:   the real iteration number the pcgXH() performed
%   relResEachIter: (realIter+1)*1 vector of the relative residual at each iteration, the fist element is the intial relative residual
%                   norm(b-A*x0)/norm(b), the second element is the relative residual after 1st iteration.
%   r:          N*1 vector of residual b - Ax
%   
%
% Question:  Why don't you just use matlab built-in pcg()? 
% Answer: 
%       We have additional "Error Elimination Step" in pcgXH() which is not there in matlab pcg().
%       Without this step, pcgXH() outputs same results and has similar speed (+-10%) as the matlab pcg().
%       With this step, pcgXH() more accurate results, and has faster speed than the matlab pcg(), when A is bad conditioned and no 
%   preconditoner is used. In an interior point method use pcgXH/pcg to solve the Newton step inexactly,  pcgXH and pcg were called 
%   multiple times, and use 216 and 715 total internal iterations to generate 73.182 dB and 73.075dB results. 
%       Another reason is that we require explictly define MInvFun() to avoid the confusing of the M1 and M2 expression in
%   matlab pcg(), but we require user to specify the MInvFun = @(x) M\x even if the original preconditioner M is a sparse matrix, 
%   or require user to specify the MInvFun = @(x) M2\(M1\x) if M = M1*M2 where M1,M2 may be get from Cholesky decomposition M1*M2 = L*L'
%
% 2017.05.15 Created by XH, orginal version cg without preconditioner
% 2018.02.03 Added preconditioner MInvFun, switched the parameter order to match matlab's function pcg [x,flag,relres,iter,resvec] = pcg(A,b,tol,maxit,M1,M2,x0,varargin)
%            cgXH(A, b, x0, tol, maxIter) ---> pcgXH(A, b, tol, maxIter, x0, MInvFun)
%
%  See also matlab pcg():  [x,flag,relres,iter,resvec] = pcg(A,b,tol,maxit,M1,M2,x0,varargin)

if nargin == 0; testMe(); return; end % Unit Test

%% parse parameters
N = numel(b);
if ~exist('x0', 'var') || isempty(x0)
    x0 = zeros(N,1);
end
if ~exist('tol', 'var') || isempty(tol)
    tol = 1e-8;
end
if ~exist('maxIter', 'var') || isempty(maxIter)
    maxIter = min(N, 10);
end

% Creat function handles if A is a matrix, or copy the function handle if A is a function handle. 
AFun = solver.createFunHandle(A, 'mtimes');

% Create MInvFun() from preconditioner
MInvFun = @(x) x;
if exist('M1_or_M1InvFun', 'var') && ~isempty(M1_or_M1InvFun)
    % If M1_or_M1InvFun is a matrix, then it is M1, we create M1InvFun() for M1\x; 
    % otherwise it is already a function handle M1InvFun, just copy it
    MInvFun = solver.createFunHandle(M1_or_M1InvFun, 'mldivide'); % same as MInvFun = M1InvFun = solver.createFunHandle(M1_or_M1InvFun, 'mldivide');
end
if exist('M2_or_M2InvFun', 'var') && ~isempty(M2_or_M2InvFun)
    M2InvFun = solver.createFunHandle(M2_or_M2InvFun, 'mldivide');
    MInvFun = @(x) M2InvFun(MInvFun(x));
end

%% Initialize x, r, d
bNorm = norm(b); % sqrt(b'*b)
x = x0;
r = b - AFun(x);

% Make sure x, relRes, realIter, relResEachIter and r are all assigned in return, even if return right in the intial step
relRes = norm(r)/bNorm;
realIter = 0;
relResEachIter = relRes;

% Check if initial guess is alreay a good enough solution
if relRes < tol
    flag = 0; % coverged successfully
    return
end

y = MInvFun(r);
if ~all(isfinite(y))  % those flag = 2/4 judements are adpated from matlab pcg()
    flag = 2;
    return
end
d = y; 
rTy_km = r' * y;
if (rTy_km == 0) || isinf(rTy_km)
    flag = 4;
    return
end

% Preallocate vector for realative residuals before start 
relResEachIter = zeros(maxIter+1, 1);    
relResEachIter(1) = relRes;
flag = 1;   % assign the default flag:  the iteration stops due to k reach maxIter, but not other reasons
stag = 0;	% stagnation of the method
maxStagSteps = 3;
%% iteration from k-1 to k
% Notations:   rTy_k =  r(k)'*yk; 
%              rTy_km = r(k-1)'*y(k-1); where m means minus 1
for k = 1:min(N, maxIter)
    % precompute A*d (only need one of this most expensive operation per iteration)
    Ad = AFun(d);
    
    dTAd = d'*Ad;
    if (dTAd <= 0) || isinf(dTAd)
        flag = 4;
        break
    end
%     % The following check seems not necessary, even if dTAd is very small number is ok.     
%     if dTAd < tol^2*bNorm^2
%         warning('divide by number close to 0 %11.5e, cg may not be stable. but still keep iterate', dTAd);
%     end    
    % update x and r after caclulate step size alpha
    alpha = rTy_km / dTAd;
    if isinf(alpha)
        flag = 4;
        break
    end    

     % Check for stagnation of the method    
    if (norm(d)*abs(alpha) < eps*norm(x))
        % too less progress: alpha*d << x
        stag = stag + 1;
    else
        stag = 0;
    end
    if stag >= maxStagSteps
        flag = 3;
        break;
    end
    
    x = x + alpha * d;    
    
    % Without this Error Elimination Step: this algorithm performs same as matlab's pcg(). 
    % With this step, this algorithm is better than pcg when A is bad conditioned, total iterations cg=216, pcg=715 in a interpoint method example
    if mod(k, 20) ~=  0 % do the correction in 10 or 20 steps doesn't matter too much, don't go above
        r = r - alpha * Ad;
    else
        r = b - AFun(x);  % eliminate accumlated floating point operation error
    end
    
    % Convergence check: check new error ||r||, break if it is small enough
    relRes =  norm(r)/bNorm; %returns the relative residual norm(b-A*x)/norm(b)
    relResEachIter(k+1) = relRes;   
    if relRes < tol
        flag = 0; % coverged successfully, the only place we have good solution (except the case inital x0 is already good)
        break
    end
    
    % update search direction for next loop. update rTr.
    y = MInvFun(r);
    if ~all(isfinite(y))
        flag = 2;
        break
    end
    rTy_k =  r' * y;
    if (rTy_k == 0) || isinf(rTy_k)
        flag = 4;
        break
    end
    beta = (rTy_k / rTy_km);
    if (beta == 0) || isinf(beta)
        flag = 4;
        break
    end
    d = y + beta * d;
    rTy_km = rTy_k; 
end

 
realIter = k;
relResEachIter = relResEachIter(1 : k+1);

end



%% Unit Test
function testMe()
% pinv: The computation is based on SVD(A) and any singular values less than a tolerance are treated as zero. Default tol:  MAX(SIZE(A)) * NORM(A) * EPS(class(A))
% cg, pcg, minres, A\b all have problems when the singular value are close to zero
%% Problems from Matlab
%%
n1 = 21; 
A = gallery('moler',n1);  
b1 = sum(A,2);
tol = 1e-6;  
maxit = 15;  
M1 = spdiags((1:n1)',0,n1,n1);
[x1,flag1,rr1,iter1,rv1] = pcg(A,b1,tol,maxit,M1);
[x1XH,flag1XH,rr1XH,iter1XH,rv1XH] = pcgXH(A,b1,tol,maxit,M1, []);
difference(x1, x1XH)
fprintf('pcg vs pcgXH: flag: %d, %d, relative residual: %.4e, %.4e, iterations: %d, %d.\n', flag1, flag1XH, rr1, rr1XH, iter1, iter1XH);
%[x1XH,flag1XH,rr1XH,iter1XH,rv1XH] = pcg(A,b1,tol,maxit,M1);
%%
curPcgAll = {@pcg, @pcgXH};
for nn = 1:1%numel(curPcgAll)
    curPcg = curPcgAll{nn};
    A = delsq(numgrid('S',100));
    b = ones(size(A,1),1);
    [x0,fl0,rr0,it0,rv0] = curPcg(A,b,1e-8,100);
    L = ichol(A);
    [x1,fl1,rr1,it1,rv1] = curPcg(A,b,1e-8,100,L,L');
    L = ichol(A,struct('michol','on'));
    [x2,fl2,rr2,it2,rv2] = curPcg(A,b,1e-8,100,L,L');
    if strcmp(func2str(curPcg), 'pcg')
        rv0 = rv0/norm(b);
        rv1 = rv1/norm(b);
        rv2 = rv2/norm(b);
    end
    figure(nn); clf; 
    semilogy(0:it0,rv0,'b.');
    hold on;
    semilogy(0:it1,rv1,'r.');
    semilogy(0:it2,rv2,'k.');
    legend('No Preconditioner','IC(0)','MIC(0)');
    grid on; grid minor;
    title(func2str(curPcg));
    xlabel('iteration number');
    ylabel('relative residual');
    hold off;
end
%% Problem 1: 
N = 4; A = diag(1:N);  b = ones(N,1); % A(1,1)=-1; % one negative singular value is ok
x  = cg(A,b);
x  = pcg(A,b);
fprintf('A positive definite: full rank ok conditioned.\n');
fprintf('x, A\\b, pinv(A)*b (pinv(A)*b is always correct)=:\n');
disp([x, A\b, pinv(A)*b])

%% Problem 2: 
N = 4; A = zeros(N,N); A(1,1)=1; b = ones(N,1);
x = cg(A,b);
fprintf('A positive semidefinite: rank deficient.\n');
fprintf('x, A\\b, pinv(A)*b (pinv(A)*b is always correct)=:\n');
disp([x, A\b, pinv(A)*b])

%% Problem 3: positive definite: full rank bad conditioned
N = 4; A = diag(eps(1)*ones(1,N)); A(1,1)=1; b = ones(N,1);
x = cg(A,b)
x = pcg(A,b)
x = minres(A,b)
fprintf('x, A\\b, pinv(A)*b (pinv(A)*b is always correct)=:\n');
fprintf('A positive definite: full rank bad conditioned.\n');
disp([x, A\b, pinv(A)*b])

%% Problem 4: non-negative constraint
N = 4; A = diag(eps(1)*ones(1,N)); A(1,1)=1; xGT = [1,1,0,0]'; b = A*xGT; % A(1,1)=-1; % one negative singular value is ok
x = cg(A,b)

%% solve 0.5*||b-Ax||^2 + regWt* 1'* x, s.t. x>=0
% cg to solve A'Ax = A'b - regWt*ones(N,1)
rng(0,'twister'); M=3; N= 5; 
A = rand(M,N); regWt = 0.01; xGT = zeros(N,1); xGT(1) = 1;  b = A*xGT; 
pinv(A)*b
Acg = A'*A;  bcg = A'*b - regWt*ones(N,1);
x = cg(Acg, bcg)
%x = solveCG(b, A, regWt, varargin)

end