function printfig(filename, widthheight, varargin)
%save current figure to png/pdf file
%printfig(filename, widthheight, 'fontsize', 18, 'dpi', 450, 'hf', 3, 'cleanTitle', false);
%Usage
% printfig('1.png', [1600, 1000])
% printfig('1.pdf', [1600, 1200], 'fontsize', 18) %set all font size to 18
% printfig('1.pdf', [1600, 1200], 'dpi', 450); %set dpi
% printfig('1.pdf', [1600, 1200], 'cleanTitle', false); % print include the current title (not clean current title), default is to clean
% printfig('1.pdf', [1600, 1200], 'hf', 3); % print figure(3) rather than current figure
% printfig('1.pdf', [1600, 1200], 'fontsize', 18, 'dpi', 450, 'hf', 3, 'cleanTitle', false);
% Inputs:
%   filename:    to save .png file name (could either contain .png or not contain .png suffix), if not specified, use current
%       time as filename yyyymmdd_hhMM.png
%   widthheight: the width and  height in pixel of the saved .png file, if not specifid, use figure's current size
%   'fontsize': the font size used in print, if not specified, use figure's current font size
%   
%  Xiang Huang: xianghuang@gmail.com
%   2017.02.16 created
%   2017.03.02 revised, added dpi
%   2017.05.10 revised, added fontsize
%   NOTE: the saved figure's layout doesn't depend on current figure if width height specified

%% parsing filename, widthheight
if ~exist('filename', 'var') || isempty(filename)
    %filename = datestr(now, 'yyyymmdd_hhMM.png');
    [filenameonly, pathStr] = uiputfile('*.*','Save to pdf/png/tif/jpg file:');
    if filenameonly == 0; return; end
    filename = [pathStr, filenameonly];
end
if exist(filename, 'file')
    b = questdlg('Overwrite Existing File?', 'Warning!', 'Yes', 'No', 'No');
    if strcmp(b, 'No') || isempty(b); return; end
end

%% parsing name/value paired parameters.
p = inputParser;
p.addParameter('hf',    gcf,    @(x) isa(x, 'matlab.ui.Figure'));   % handle of figure to save
p.addParameter('cleanTitle', true,  @(x) any(x==[true, false, 1, 0])); % whether save current figure with title
p.addParameter('dpi',       300,    @(x) validateattributes(x, {'numeric'}, {'positive', 'integer'}));
p.addParameter('fontsize',  [],     @(x) validateattributes(x, {'numeric'}, {'positive', 'integer'}));
%p.addParameter('leftbottom',      [0, 0],  @(x) validateattributes(x, {'numeric'}, {'2d'}));  % original point in inches
p.parse(varargin{:});
cellfun(@(x) evalin('caller', sprintf('%s=p.Results.%s;', x, x)), fields(p.Results)); % e.g. eval('fSz = p.Results.fSz'); Or use in.fSz after set in =p.Results;

%% Backup previous settings
paperType0 = get(hf,'PaperType');
paperUnits0 = get(hf,'PaperUnits');
units0 = get(hf,'Units');
paperPositionMode0 = get(hf, 'PaperPositionMode');
paperPosition0 = get(hf,'PaperPosition');
paperSize0 = get(hf,'PaperSize');
%font size of all
if ~isempty(fontsize)
    hTextAll = findall(hf,'-property','FontSize');
    fontSizeAll0 = get(hTextAll, 'Fontsize');
end

if cleanTitle
    axArray = findobj(hf, 'type', 'axes', '-depth', 1); % only check one depth
    axArray = axArray(end:-1:1);  % reverse the order so that the first one is the first axes, but this only applies to 1D grid        
    titleAll0  = arrayfun(@(x) get(get(x, 'Title'), 'String'), axArray, 'UniformOutput', false);        
    multAxes(@title, '');
end


% Make changing paper type possible, units to be the same
set(hf,'PaperType','<custom>');
set(hf,'PaperUnits','inches', 'Units', 'inches');
% Set the page size and position
if ~exist('widthheight', 'var') || isempty(widthheight)
    position = get(hf,'Position');
    widthheight = position(3:4);
else
   widthheight = widthheight/dpi;
end
set(hf, 'PaperPositionMode', 'manual');
set(hf,'PaperPosition',  [0, 0, widthheight]);
set(hf,'PaperSize',  widthheight);

% change font size
if ~isempty(fontsize)
    set(hTextAll, 'FontSize', fontsize);
end

% Save to png/pdf
[~, ~, ext] =  fileparts(filename);
ext2printformat = containers.Map({'.pdf', '.png', '.tif', '.tiff', '.jpeg', '.jpg'}, {'-dpdf', '-dpng', '-dtiff', '-dtiff', '-djpg', '-djpg'});
assert(isKey(ext2printformat, lower(ext)), 'Printed filename must be specified with one of following extentions: .png, .pdf, .tif, .jpg.');
print(hf, filename, ext2printformat(lower(ext)), sprintf('-r%d',dpi));

%% Restore the previous settings
set(hf,'PaperType', paperType0);
set(hf,'PaperUnits', paperUnits0);
set(hf,'Units', units0);
set(hf,'PaperPosition', paperPosition0);
set(hf,'PaperSize', paperSize0);
set(hf, 'PaperPositionMode', paperPositionMode0);
if ~isempty(fontsize)
    for n = 1:numel(hTextAll)
        set(hTextAll(n), 'FontSize', fontSizeAll0{n});
    end
end
if cleanTitle
    multAxes(@title, titleAll0);
end
end