function showPhotonPerFrame(imgFile, maskObjFile, EMGain, pages, bgFile)
%preprocess.showPhotonPerFrame(imgFile, maskObjFile, EMGain, pages, bgFile)
% Inputs: 
%       refer to preprocess.cleanBackground() for meanings
% Outputs:
%      NONE, just show a figure
%Usage:
%   preprocess.showPhotonPerFrame('/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/PSF_Ave.tif', '/sandbox/smallWorlds/20160809_Bacterial_Confocal_PSFs/PSF_Ave_Mask_Central100Slices.bmp', 750)
%
%Reference:
%   preprocess.cleanBackground.m
%
% Changes:
%  2017.12.05_fixed the background photon negative bug, can take 3D image mask
%

%% process inputs
% set imgFile, imgFilePath, imgFileNameNoExt
if ~exist('imgFile', 'var') || isempty(imgFile)
    [imgFileName, imgFilePath] = uigetfile('*.tif',' select img file');
    [~, imgFileNameNoExt] = fileparts(imgFileName);
    imgFile = fullfile(imgFilePath, imgFileName);
else
    [imgFilePath, imgFileNameNoExt] = fileparts(imgFile);
end

% set maskObjFile
if ~exist('maskObjFile', 'var') || isempty(maskObjFile)
    maskObjFile = fullfile(imgFilePath, [imgFileNameNoExt, '_mask.bmp']); % default maskObjFile
    if ~exist(maskObjFile, 'file')
        [maskObjFileName, maskObjFilePath] = uigetfile(fullfile(imgFilePath, '*.bmp'),' select mask file for img');
        maskObjFile = fullfile(maskObjFilePath, maskObjFileName);                
    end
end

% set pages
fileInfo = imfinfo(imgFile);   % return tiff structure, one element per page
if ~exist('pages', 'var') || isempty(pages)
    % Default, read all pages.
    pages = 1:numel(fileInfo);
else
    % User specified pages, verify if they are the subset of 1:numel(tiffFileInfo)
    assert(all(ismember(pages, 1:numel(fileInfo))), sprintf('User specified pages are not subset of the valid page from 1 to %d', numel(fileInfo)));
end

% set EMGain
if ~exist('EMGain', 'var') || isempty(EMGain)
    prompt = 'What is the EMGain? Enter a number: ';
    EMGain = input(prompt);
end

% set bgFile
if ~exist('bgFile', 'var') || isempty(bgFile)
    bgFile = fullfile(imgFilePath, [imgFileNameNoExt, '_bg.tif']); %'_bgOutlier.tif';               
    assert(exist(bgFile, 'file')>0,  sprintf('Cannot find the background file %s.', bgFile));    
end



%% read object tiff
PRECISION = 'single';
img3D = cast(tif.read(imgFile, pages), PRECISION);

[Ny, Nx, Nz] = size(img3D);

%% read object background tiff
fprintf('Use background file: %s\n', bgFile);
bg3D = cast(tif.read(bgFile, pages), class(img3D));


%% Read mask/ maskObj3D, 2017.12.04 updated to handle 3D mask file
switch lower(maskObjFile(end-2:end))
    case {'bmp', 'png'}
        mask = imread(maskObjFile); mask = mask>0; % read and set the binary 2D /3D mask
        %assert(ndims(mask)==2, sprintf('Mask is %dD. Currently only handle 2D mask!', ndims(mask)));
        assert( all(size(mask)==[Ny, Nx]), sprintf('Image and Mask has different heightxwidth: %dx%d vs %dx%d', Ny, Nx, size(mask,1), size(mask,2)));
        maskObj3D = repmat(mask, [1, 1, Nz]);
    case 'tif'
        % 3D mask
        maskObj3D = tif.read(maskObjFile, pages); maskObj3D = maskObj3D > 0;  % 
        assert(all(size(maskObj3D)==[Ny, Nx, Nz]), sprintf('Image and Mask has different size  %dx%dx%d vs %dx%dx%d', Ny, Nx, Nz, size(maskObj3D,1), size(maskObj3D,2),  size(maskObj3D,3)));
    otherwise
        error('unkown file format %s', lower(maskObjFile(end-2:end)));
    
end

%% camera
camPsf = CAMERA('EMGain', EMGain);  
camOffSet = mean(camPsf.camOffSet(:)); % should be around  491.5

img3DZeroOutMask = img3D; 
img3DZeroOutMask(~maskObj3D) = 0; 

imgObjWithoutBg3D = zeros(size(img3D), 'like', img3D); 
imgObjWithoutBg3D(maskObj3D) = img3D(maskObj3D) - bg3D(maskObj3D); 

imgObjWithoutBgNonNeg3D = imgObjWithoutBg3D; 
imgObjWithoutBgNonNeg3D(imgObjWithoutBgNonNeg3D<0) = 0; 

%%
photonAll = [squeeze(sum(sum(img3D - camOffSet,1),2))';  
            squeeze(sum(sum(img3DZeroOutMask - camOffSet*maskObj3D,1),2))'; 
            squeeze(sum(sum(imgObjWithoutBg3D,1),2))'; 
            squeeze(sum(sum(imgObjWithoutBgNonNeg3D,1),2))'; 
            squeeze(sum(sum(bg3D - camOffSet,1),2))']/camPsf.gain;  % 2017.12.05 fixed, was sum(bg3D-camOffset), but bg3D are 0 rather than camOffSet for pixels outside maskBg
figure('units','normalized','outerposition',[0 0 1 1])
myXlabel = sprintf('z (frame %d to %d)', pages(1), pages(end));
LegendNameAll = {'Orignal', 'Original Inside Obj Mask', 'Object with Bg Removed', 'Object with Bg Removed & NonNeg', 'Background'};
subplot(221); semilogy(pages, photonAll);  title('Photon Counts - Overview');legend(LegendNameAll{1:5});  xlabel(myXlabel); ylabel('total photons per z slice'); grid minor;
subplot(222); plot(pages, photonAll(2:4, :));  title('Photon Counts for Pixels Inside Object Mask'); legend(LegendNameAll{2:4});   xlabel(myXlabel); ylabel('total photons per z slice'); grid minor;
subplot(223); plot(pages, photonAll(1,:));   title('Photon Counts in Original Image'); xlabel('depth (z)'); ylabel('total photons per z slice');grid minor;
subplot(224); plot(pages, photonAll(5, :));  title('Background');   xlabel(myXlabel); ylabel('total photons per z slice'); grid minor;

%%
end